###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2007, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
#! /usr/bin/env python 

from itertools import groupby

from Products.ZenHub.HubService import HubService
from Products.ZenHub.PBDaemon import translateError

from Products.ZenEvents.ZenEventClasses import Status_IpService
from Products.ZenModel.Device import Device

from Procrastinator import Procrastinate
from ThresholdMixin import ThresholdMixin

from twisted.internet import defer

from twisted.spread import pb
class ServiceConfig(pb.Copyable, pb.RemoteCopy):
    
    def __init__(self, svc):
        self.device = svc.hostname()
        self.component = svc.name()
        self.ip = svc.getManageIp()
        self.port = svc.getPort()
        self.sendString = svc.getSendString()
        self.expectRegex = svc.getExpectRegex()
        self.timeout = svc.zStatusConnectTimeout
        self.failSeverity = svc.getFailSeverity()
        self.key = svc.key()
pb.setUnjellyableForClass(ServiceConfig, ServiceConfig)

class StatusConfig(HubService, ThresholdMixin):

    def __init__(self, dmd, instance):
        HubService.__init__(self, dmd, instance)
        self.config = self.dmd.Monitors.Performance._getOb(self.instance)
        self.procrastinator = Procrastinate(self.notify)

    @translateError
    def remote_propertyItems(self):
        return self.config.propertyItems()

    @translateError
    def remote_serviceStatus(self):
        zem = self.dmd.ZenEventManager
        status = zem.getAllComponentStatus(Status_IpService)
        devices = set(d.id for d in self.config.devices.objectValuesGen())
        return [x for x in status.items() if x[0][0] in devices]

    @translateError
    def remote_services(self, unused):
        devCat = self.dmd.Devices.deviceSearch
        compCat = self.dmd.Devices.componentSearch
        devids = [d.id for d in self.config.devices.objectValuesGen()]
        devbrains = devCat(id=devids)
        compbrains = compCat.search(
                         query_request = dict(getCollectors='zenstatus',
                                              monitored=True,
                                              getParentDeviceName=tuple(devids)),
                         sort_index='getParentDeviceName')
        unindex_data = compCat._catalog.indexes['getParentDeviceName']._unindex
        def _devname(brain):
            return unindex_data[brain.getRID()]
        bydevice = dict((k, tuple(g)) for k, g in groupby(compbrains, key=_devname))
        result = []
        for brain in devbrains:
            if (brain.id not in bydevice or
                not brain.getObject().monitorDevice()):
                continue
            for svc in bydevice[brain.id]:
                svc = svc.getObject()
                if svc.getProtocol()=='tcp':
                    result.append(ServiceConfig(svc))
        return result

    @translateError
    def remote_getDefaultRRDCreateCommand(self):
        return self.config.getDefaultRRDCreateCommand()

        
    def update(self, object):
        if not self.listeners: return

        # the PerformanceConf changed
        from Products.ZenModel.PerformanceConf import PerformanceConf
        if isinstance(object, PerformanceConf):
            for listener in self.listeners:
                listener.callRemote('setPropertyItems', object.propertyItems())
        self.procrastinator.doLater()

    def notify(self, unused):
        lst = []
        for listener in self.listeners:
            lst.append(listener.callRemote('notifyConfigChanged'))
        return defer.DeferredList(lst)

    def deleted(self, obj):
        for listener in self.listeners:
            if isinstance(obj, Device):
                listener.callRemote('deleteDevice', obj.id)
