###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2007, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################

__doc__="""DefGraphPoint

Handles GraphPoints that define an rrd DEF
"""

from GraphPoint import GraphPoint
from Globals import InitializeClass


def manage_addDefGraphPoint(context, id, REQUEST = None):
    ''' This is here so than zope will let us copy/paste/rename
    graphpoints.
    '''
    gp = DefGraphPoint(id)
    context._setObject(gp.id, gp)
    if REQUEST:
        return context.callZenScreen(REQUEST)


class DefGraphPoint(GraphPoint):

    meta_type = 'DefGraphPoint'
    
    rrdFile = ''
    dsName = 'ds0'
    step = ''
    start = ''
    end = ''
    cFunc = 'AVERAGE'
    rFunc = ''
    
    _properties = GraphPoint._properties + (
        {'id':'rrdFile', 'type':'string', 'mode':'w'},
        {'id':'dsName', 'type':'string', 'mode':'w'},
        {'id':'step', 'type':'string', 'mode':'w'},
        {'id':'start', 'type':'string', 'mode':'w'},
        {'id':'end', 'type':'string', 'mode':'w'},
        {'id':'cFunc', 'type':'string', 'mode':'w'},
        {'id':'rFunc', 'type':'string', 'mode':'w'},        
        )
    

    def getDescription(self):
        return '%s %s' % (self.rrdFile, self.dsName)


    def getType(self):
        return 'DEF'


    def getGraphCmds(self, cmds, context, rrdDir, addSummary, idx, 
                            multiid=-1, prefix=''):
        ''' Build the graphing commands for this graphpoint
        '''
        if not (self.rrdFile and self.dsName and self.cFunc):
            return cmds
        
        rrdFile = self.talesEval(self.rrdFile, context, rrdDir=rrdDir)    
            
        dest = self.getDsName(self.id, multiid, prefix)
        gopt = 'DEF:%s=%s:%s:%s' % (
                    dest,
                    rrdFile,
                    self.dsName,
                    self.cFunc)
        if self.step:
            gopt += ':step=%s' % self.step
        if self.start:
            start = self.talesEval(self.start, context)
            gopt += ':start=%s' % start.replace(':', '\:')
        if self.end:
            end = self.talesEval(self.end, context)
            gopt += ':end=%s' % end.replace(':', '\:')
        if self.rFunc:
            gopt += ':reduce=%s' % self.rFunc        
        return cmds + [gopt]


InitializeClass(DefGraphPoint)
