###############################################################################
#
#   Copyright (c) 2006 Zenoss, Inc. all rights reserved.
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License.
#
###############################################################################

#
# Contained below are functions that are associated with installing or
# configuring Zenoss.  These include things like initializing the
# MySQL database or populating the Zeo database.  You will not find
# functions here handle external dependency compilation or Zenoss
# downloading (those are found in build-functions)
#

test -r shared-functions.sh && . shared-functions.sh

# return a list of pids and commands of zenoss related processes
get_running()
{
    FORMAT="pid,command"
    if [ "`uname`" = "SunOS" ]; then
        FORMAT="pid,comm"
    fi

    if [ -z "${OS_USERNAME}" ] ; then
        export OS_USERNAME=zenoss
    fi
    id ${OS_USERNAME} >/dev/null 2>&1 || OS_USERNAME=${USER:-${LOGNAME}}

    # NB: buildbot is used to create packages at Zenoss
    ps -o ${FORMAT} -U ${OS_USERNAME} | grep python | egrep -v 'grep|buildbot' | egrep 'zeo|zope|zen' 
}

# use ps to see if zenoss processes are running.  fail if any are found
check_running()
{
    PS=`get_running`
    if [ "$PS" != "" ] ; then
        echo
        echo Looks like there are some stray Zenoss processes running.
        echo ----------------------------------------------------------
        get_running
        echo ----------------------------------------------------------
        fail "Please kill these processes before running the install"
    fi
}

# kills off processes related to zenoss
kill_running()
{
    for pid in \
        `get_running | awk '{print $1}'`
    do
        kill $pid
    done
}

# attempt to create the events database and login to it.  fail if unsuccessful
create_database()
{

    if [ "${MYSQLHOST}" = "localhost" ]; then
        MYSQL_HOST_PERM="localhost"
    else
        MYSQL_HOST_PERM="%"
    fi

    # NB: MySQL 4x will allow *ANYONE* to log in.  Need to check if it's a valid
    #     user according to the MySQL CURRENT_USER() function
    current_user=`echo "SELECT CURRENT_USER();" | mysql --port=${MYSQLPORT} -h ${MYSQLHOST} -u ${MYSQLROOTUSER} --password="${MYSQLROOTPASSWD}" | awk -F\@ '!/CURRENT_USER\(\)/ { print $1; }'`

    if [ -z "$current_user" ] ; then
        fail "During the MySQL install, the MySQL root user (${MYSQLROOTUSER}) or password is incorrect "
    fi

    mysql --port=${MYSQLPORT} -h ${MYSQLHOST} -u ${MYSQLROOTUSER} --password="${MYSQLROOTPASSWD}" <<EOF
        grant all on $MYSQLEVENTDB.* to '$MYSQLUSER'@'${MYSQL_HOST_PERM}' identified by '$MYSQLPASS';
        grant super, file on *.* to '$MYSQLUSER'@'${MYSQL_HOST_PERM}'; 
        grant SELECT on mysql.user to '$MYSQLUSER'@'${MYSQL_HOST_PERM}'; 
        grant all on $MYSQLEVENTDB.* to '$MYSQLUSER'@'%' identified by '$MYSQLPASS';
        grant SELECT on mysql.user to '$MYSQLUSER'@'%'; 
        flush privileges;
EOF
    if [ $? -ne 0 ];then
        echo "mysql --port=${MYSQLPORT} -h ${MYSQLHOST} -u ${MYSQLROOTUSER} --password=\"${MYSQLROOTPASSWD}\" <<EOF"
        echo "        grant all on $MYSQLEVENTDB.* to '$MYSQLUSER'@'${MYSQL_HOST_PERM}' identified by '$MYSQLPASS';"
        echo "        grant super, file on *.* to '$MYSQLUSER'@'${MYSQL_HOST_PERM}';" 
        echo "        grant SELECT on mysql.user to '$MYSQLUSER'@'${MYSQL_HOST_PERM}'; "
        echo "        grant all on $MYSQLEVENTDB.* to '$MYSQLUSER'@'%' identified by '$MYSQLPASS';"
        echo "        grant SELECT on mysql.user to '$MYSQLUSER'@'%'; "
        echo "        flush privileges;"
        echo "EOF"
        fail "Failure while granting MySQL permissions."
    fi

    # attempt to login to the database using the mysql user
    mysql -h ${MYSQLHOST} -u ${MYSQLUSER} --port=${MYSQLPORT} -p"${MYSQLPASS}" < /dev/null >/dev/null || \
        fail "During the MySQL install, the supplied MySQL user (${MYSQLUSER}) or password is incorrect "
}

# populate the events database with some initial information
run_zeneventbuild()
{
    # Don't create the database if it exists
    if [ -f backup.tar ]; then
        echo "The backup.tar file exists.  Skipping the install of the event tables into MySQL"
        return
    fi
    
    echo "Creating events database using zenevent build..." >> zenbuild.log
    $ZENHOME/bin/zeneventbuild $MYSQLHOST $MYSQLUSER $MYSQLPASS $MYSQLEVENTDB $MYSQLPORT 2>>zenbuild.log || fail "Failed to install event tables into MySQL"
}

# create a zope instance 
run_mkzopeinstance()
{
    USERNAME=${OS_USERNAME}
    id ${USERNAME} 2>&1 >/dev/null || USERNAME=$USER
    if [ ! -f backup.tar ] ; then
        $PYTHON $ZOPEHOME/mkzopeinstance \
            --dir="$ZENHOME" \
            --user="admin:$ZOPEPASSWORD" || fail Unable to create Zope instance.
        if [ -f $ZENHOME/etc/zenoss.conf ] ; then
            mv $ZENHOME/etc/zenoss.conf $ZENHOME/etc/zope.conf
        fi
        # Interestingly, mkzopeinstance seems to be setting _everything_ in $ZENHOME/bin to
        # 700 as a side-effect.  This ends up killing the carefully crafted suid perms for zensocket.
        # Blaze a remedial trail for now until we can get mkzopeinstance to behave.
        chmod 755 $ZENHOME/bin/*
        chmod 04755 $ZENHOME/bin/zensocket
        replace "<<INSTANCE_HOME>>" "$ZENHOME" $ZENHOME/etc/zope.conf
        replace "effective-user zenoss" "effective-user $USERNAME" $ZENHOME/etc/zope.conf
    fi
}

# create a zeodb instance
run_mkzeoinstance()
{
    for i in zeoctl runzeo
    do
       # Move easy install versions of these scripts out of the way
       # so mkzeoinstance will generate the non-easy install versions we require.
       if [ -f "$ZENHOME/bin/$i" ];then
          if grep EASY-INSTALL-ENTRY-SCRIPT $ZENHOME/bin/$i 2>/dev/null 1>&2;then
             mv $ZENHOME/bin/$i $ZENHOME/bin/${i}.easy_install
          fi
       fi
    done
    if [ ! -f backup.tar ] ; then
        $PYTHON $ZOPEHOME/mkzeoinstance $ZENHOME || fail "Unable to create the Zeo object database (ZODB)"
    else
        echo "[skipping] $PYTHON $ZOPEHOME/mkzeoinstance $ZENHOME"
        echo "           since backup.tar already exists."
    fi
    for i in zeoctl runzeo
    do
       # Double check that these scripts were created in the way we need.
       if [ -f "$ZENHOME/bin/$i" ];then
          if grep EASY-INSTALL-ENTRY-SCRIPT $ZENHOME/bin/$i 2>/dev/null 1>&2;then
             fail "$ZENHOME/bin/$i is still in EASY-INSTALL-ENTRY-SCRIPT format.  Suspect mkzeoinstance did not run properly."
          fi
       else
          fail "Missing $ZENHOME/bin/$i after run_mkzeoinstance."
       fi
    done
    replace "/usr/bin/python" "$PYTHON" $ZENHOME/etc/zeo.conf
}

# start up zeodb
start_zeodb()
{
    echo 'Starting Zope Object Database'
    #$ZENHOME/bin/zeoctl -p zeoctl start || fail "Unable to start the Zeo object database (ZODB)"
    $ZENHOME/bin/zeoctl start || fail "Unable to start the Zeo object database (ZODB)"
}

# shut down the zeodb
stop_zeodb()
{
    echo 'Stopping Zope Object Database'
    #$ZENHOME/bin/zeoctl -p zeoctl stop || fail "Unable to stop Zeo object database (ZODB)"
    $ZENHOME/bin/zeoctl stop || fail "Unable to stop Zeo object database (ZODB)"
}

# load objects into the zodb
run_zenbuild()
{
    if [ ! -f backup.tar ] ; then
        echo Loading initial Zenoss objects into the Zeo database
        echo   '(this can take a few minutes)'
        $ZENHOME/bin/zenbuild --evtport $MYSQLPORT -s $MYSQLHOST -u $MYSQLUSER -p "$MYSQLPASS" -d $MYSQLEVENTDB 2>>zenbuild.log || \
              fail "Unable to create the initial Zenoss object database"
    fi

    if [ "$MIGRATE" != "n" ]; then
        run_zenmigrate  >>zenbuild.log 2>&1 || fail "Data migration failed"
    fi

}

# migrate the Zenoss objects using the migrate steps in ZenPacks and in base product
run_zenmigrate()
{
    echo "Migrating data..."
    $ZENHOME/bin/zenmigrate
}

# starts the Zope server
start_zope()
{
    echo 'Starting Zope Server'

    # This works properly with Zope 2.12 now that ZOPEHOME != ZENHOME  
    # Before we were getting "has no attribute 'instancehome' because the 
    # zope.conf wasn't getting loaded properly.
    $ZENHOME/bin/zopectl start || fail "Unable to start Zope"

    # $ZENHOME/bin/runzope -C $ZENHOME/etc/zope.conf || fail "Unable to start Zope"
}

# restart the snmpd process
restart_snmpd()
{
    if /etc/init.d/snmpd status 2>/dev/null 1>&2 ;then
        /etc/init.d/snmpd restart
    else
        /etc/init.d/snmpd start
    fi
}

# restart the mysql process
restart_mysql()
{
    for script in \
        /etc/init.d/mysql \
        /etc/init.d/mysqld
    do
        if [ -x ${script} ]; then
            if ${script} status 2>/dev/null 1>&2 ; then
                ${script} restart
            else
                ${script} start
            fi
            return
        fi
    done

    # If we get here we probably don't have MySQL -- doh!
    fail "No startup script for MySQL found -- exiting!"
}

# adds localhost as a device via the web UI
add_localhost_device()
{
    ZOPE_HOST=localhost
    if [ -x /usr/bin/host ]
    then
	for h in localhost localhost.localdomain "`hostname`"
	do
	    if /usr/bin/host "$h" >/dev/null 
	    then
                ZOPE_HOST="$h"
            fi
	done
    fi
    HOST_CLASS_PATH="/Server/Linux"
    URL="http://${ZOPE_HOST}:${ZOPE_LISTEN_PORT}/zport/dmd/DeviceLoader/loadDevice?deviceName=${ZOPE_HOST}&devicePath=${HOST_CLASS_PATH}"

    # add localhost as a device
    # wget>=1.11 does basic auth differently, so must use auth-no-challenge
    # option
    if [ -z "`wget --help | grep auth-no-challenge`" ]
    then
        WGET="wget"
    else
        WGET="wget --auth-no-challenge"
    fi
    $WGET -O /dev/null \
        --no-http-keep-alive \
        --user=${ZOPE_USERNAME} \
        --password=${ZOPE_PASSWORD} \
        ${URL} 

}

# updates the conf files in ${ZENHOME}/etc.  if the user already has a
# conf file then the example conf file will not be laid down on top of
# the existing conf file.  if the conf file already exists it is left
# in place.
update_conf_files()
{
    for EXAMPLE in \
        `ls ${ZENHOME}/etc/*.example`
    do
        CONF=`echo $EXAMPLE | cut -d. -f-2`
        
        if [ ! -s ${CONF} ]; then
            cp $EXAMPLE $CONF
        fi
    done
}

# Find all the root-owned files & directories under $ZENHOME and change them to something reasonable.
# Typically this is 'zenoss:zenoss'.
#
# The group may be specified as an optional second parameter.  Otherwise it defaults to the
# primary group associated with the specified owner.
#
fix_zenhome_owner_and_group()
{
    owner=$1

    # Really only makes sense to be remediating ownership if we're currently running/installing
    # as root.

    if [ "`id -u`" = "0" ];then
        if [ -z "${ZENHOME}" ];then
            fail "\$ZENHOME not set.  Unable to fix file ownership under \$ZENHOME."
        fi

        if [ "${ZENHOME}" = "/" ];then
            fail "\$ZENHOME unwisely set to /.  Abandoning request to change file ownership under /."
        fi

        if [ -z "${owner}" ];then
            fail "Null owner parameter passed to fix_zenhome_owner().  Unable to fix file ownership under ${ZENHOME}."
        else
            if id ${owner} 2>/dev/null 1>&2 ;then
                # If group is not explicitly passed in, then default to the effective group for the owner.
                group=${2:-`id -gn ${owner}`}

                # All pre-reqs validated.
                for fixme in `find ${ZENHOME} -type f -o -type d -user root | grep -v zensocket`
                do
                    chown ${owner}:${group} ${fixme}
                done
            else
                fail "Invalid owner (${owner}) passed to fix_zenhome_owner().  Unable to fix file ownership under ${ZENHOME}."
            fi
        fi
    fi
}
