/*!
 * Copyright (c) 2009 Zenoss, Inc.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * For complete information please visit http://www.zenoss.com/oss/
 */
(function(){ // Local scope
Ext.override(Ext.ux.grid.livegrid.GridView, {
    updateLiveRows: function(index, forceRepaint, forceReload, suspendLoadEvent)
    {
        // Zenoss change beginning
        // Allow us to optionally allow the store load event to fire
        suspendLoadEvent = Ext.isDefined(suspendLoadEvent) ? suspendLoadEvent : true;
        // End change (another below)

        var inRange = this.isInRange(index);

        if (this.isBuffering) {
            if (this.isPrebuffering) {
                if (inRange) {
                    this.replaceLiveRows(index, forceRepaint);
                } else {
                    this.showLoadMask(true);
                }
            }

            this.fireEvent('cursormove', this, index,
                           Math.min(this.ds.totalLength,
                           this.visibleRows-this.rowClipped),
                           this.ds.totalLength);

            this.requestQueue = index;
            return;
        }

        var lastIndex  = this.lastIndex;
        this.lastIndex = index;
        var inRange    = this.isInRange(index);

        var down = lastIndex < index;

        if (inRange && forceReload !== true) {

            // repaint the table's view
            this.replaceLiveRows(index, forceRepaint);
            // has to be called AFTER the rowIndex was recalculated
            this.fireEvent('cursormove', this, index,
                       Math.min(this.ds.totalLength,
                       this.visibleRows-this.rowClipped),
                       this.ds.totalLength);
            // lets decide if we can void this method or stay in here for
            // requesting a buffer update
            if (index > lastIndex) { // scrolling down

                down = true;
                var totalCount = this.ds.totalLength;

                // while scrolling, we have not yet reached the row index
                // that would trigger a re-buffer
                if (index+this.visibleRows+this.nearLimit <= this.ds.bufferRange[1]) {
                    return;
                }

                // If we have already buffered the last range we can ever get
                // by the queried data repository, we don't need to buffer again.
                // This basically means that a re-buffer would only occur again
                // if we are scrolling up.
                if (this.ds.bufferRange[1]+1 >= totalCount) {
                    return;
                }
            } else if (index < lastIndex) { // scrolling up

                down = false;
                // We are scrolling up in the first buffer range we can ever get
                // Re-buffering would only occur upon scrolling down.
                if (this.ds.bufferRange[0] <= 0) {
                    return;
                }

                // if we are scrolling up and we are moving in an acceptable
                // buffer range, lets return.
                if (index - this.nearLimit > this.ds.bufferRange[0]) {
                    return;
                }
            } else {
                return;
            }

            this.isPrebuffering = true;
        }

        // prepare for rebuffering
        this.isBuffering = true;

        var bufferOffset = this.getPredictedBufferIndex(index, inRange, down);

        if (!inRange) {
            this.showLoadMask(true);
        }

        this.ds.suspendEvents();
        var sInfo  = this.ds.sortInfo;

        var params = {};
        if (this.ds.lastOptions) {
            Ext.apply(params, this.ds.lastOptions.params);
        }

        params.start = bufferOffset;
        params.limit = this.ds.bufferSize;


        if (sInfo) {
            params.dir  = sInfo.direction;
            params.sort = sInfo.field;
        }

        var opts = {
            forceRepaint     : forceRepaint,
            callback         : this.liveBufferUpdate,
            scope            : this,
            params           : params,
            // Zenoss change begins
            //suspendLoadEvent : true
            suspendLoadEvent : suspendLoadEvent
            // Zenoss change ends
        };

        this.fireEvent('beforebuffer', this, this.ds, index,
            Math.min(this.ds.totalLength, this.visibleRows-this.rowClipped),
            this.ds.totalLength, opts
        );

        this.ds.load(opts);
        this.ds.resumeEvents();
    },

    adjustBufferInset : function(){
        var liveScrollerDom = this.liveScroller.dom;
        var g = this.grid, ds = g.store;
        var c  = g.getGridEl();
        var elWidth = c.getSize().width;

        // hidden rows is the number of rows which cannot be
        // displayed and for which a scrollbar needs to be
        // rendered. This does also take clipped rows into account
        var hiddenRows;
        if (ds.totalLength == this.visibleRows-this.rowClipped) {
          hiddenRows = 0;
        } else {
          hiddenRows = Math.max(0, ds.totalLength-(this.visibleRows-this.rowClipped));
        }

        if (hiddenRows === 0) {
            this.scroller.setWidth(elWidth);
            liveScrollerDom.style.display = 'none';
            return;
        } else {
            this.scroller.setWidth(elWidth-(this.scrollOffset||0));
            liveScrollerDom.style.display = '';
        }

        var scrollbar = this.cm.getTotalWidth()+this.scrollOffset > elWidth;

        // adjust the height of the scrollbar
        var contHeight = liveScrollerDom.parentNode.offsetHeight - this.hdHeight;
        if ( ds.totalLength > 0 && scrollbar ) {
            contHeight -= this.horizontalScrollOffset;
        }

        liveScrollerDom.style.height = Math.max(contHeight, this.horizontalScrollOffset*2)+"px";

        if (this.rowHeight == -1) {
            return;
        }

        this.liveScrollerInset.style.height = (hiddenRows === 0 ? 0 : contHeight+(hiddenRows*this.rowHeight))+"px";
     },

    getPredictedBufferIndex : function(index, inRange, down)
    {
        /*
        if (!inRange) {
            if (index + this.ds.bufferSize >= this.ds.totalLength) {
                return this.ds.totalLength - this.ds.bufferSize;
            }
            // I can't tell how this calculation is supposed to make sense.  If
            // index is outside the buffer range in the lesser direction and
            // visibleRows is larger than bufferSize/2, you just get an offset
            // larger than index, meaning it'll try to request data again,
            // forever. If you just do what you would normally do if in range,
            // it works fine. -- Ian
            return Math.max(0, (index + this.visibleRows) - Math.round(this.ds.bufferSize/2));
        }
        */
        if (!down) {
            return Math.max(0, (index-this.ds.bufferSize)+this.visibleRows);
        }

        if (down) {
            return Math.max(0, Math.min(index, this.ds.totalLength-this.ds.bufferSize));
        }
    }

});       
})(); // End local scope
(function(){ // Local scope

/**
 * Base namespace to contain all Zenoss-specific JavaScript.
 */
Ext.namespace('Zenoss');

/**
 * Namespace for anonymous scripts to attach data to avoid dumping it into
 * the global namespace.
 */
Ext.namespace('Zenoss.env');

Ext.QuickTips.init();

/*
Ext.state.Manager.setProvider(new Ext.state.CookieProvider({
    expires: new Date(new Date().getTime()+(1000*60*60*24*30))
}));
*/

/*
 * Hook up all Ext.Direct requests to the connection error message box.
 */
Ext.Direct.on('event', function(e){
    if ( Ext.isDefined(e.result) && Ext.isDefined(e.result.asof) ) {
        Zenoss.env.asof = e.result.asof || null;
    }
});

Ext.Direct.on('event', function(e){
    if (Ext.isDefined(e.result) && Ext.isDefined(e.result.msg)) {
        var success = e.result.success || false;
        if (success) {
            Zenoss.message.success(e.result.msg);
        }
        else {
            Zenoss.message.error(e.result.msg);
        }
    }
});

Ext.Direct.on('exception', function(e) {
    Ext.Msg.show({
        title: _t('Server Exception'),
        msg: '<p>' + _t('The server reported the following error:') + '</p>' +
            '<p class="exception-message">' + e.message + '</p>' +
            '<p>' + _t('The system has encountered an error.') + ' ' +
            _t('Please reload the page.') + '</p>' ,
        buttons: { yes: _t('Reload Page'), cancel: _t('Dismiss') },
        minWidth: 300,
        fn: function(buttonId, text, opt) {
            if ('yes' == buttonId) {
                window.location.reload();
            }
        }
    });
});

/*
 * Hide the server exception MessageBox if we get a good response. Primarily
 * used to have the event console starting functioning after a temporary
 * inability to reach the server.
 */
Ext.Direct.on('event', function(e){
    var message_box = Ext.Msg.getDialog();
    if (message_box != null && message_box.title == 'Server Exception') {
        if (Ext.isDefined(e.result)) {
            Ext.Msg.hide();
        }
    }
});

Ext.namespace('Zenoss.flares');

/**
 * An invisible layer that contains all the Flares.
 * Must be the highest layer in order for Flares to show up.
 */
Zenoss.flares.Container = Ext.extend(Ext.Container, {
    _layoutDelay: null,
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            id: 'flare-container',
            baseCls: 'x-flare-container',
            alignment: 't-t',
            width: '100%',
            zindex: 400000,
            items: []
        });

        Zenoss.flares.Container.superclass.constructor.call(this, config);
    },
    onRender : function(ct, position) {
        this.el = ct.createChild({
            cls: this.baseCls + '-layer',
            children: [{
                cls: this.baseCls + '-body'
            }]
        });
        this.body = this.el.child('.' + this.baseCls + '-body');
        this.el = new Ext.Layer({ zindex: this.zindex }, this.el);

        Zenoss.flares.Container.superclass.onRender.apply(this, arguments);
        this.el.alignTo(document, this.alignment);
    },
    getLayoutTarget: function() {
        return this.body;
    },
    onShow: function() {
        this.el.show();
        Zenoss.flares.Container.superclass.onShow.apply(this, arguments);
    },
    onRemove: function() {
        // Move the sticky items back to the top. Wait a few microseconds to do it in case more items are
        // being removed at the same time.
        if ( !this._layoutDelay ) {
            this._layoutDelay = new Ext.util.DelayedTask(this.doLayout, this);
        }
        this._layoutDelay.delay(500);
    },
    show: function() {
        if ( !this.rendered ) {
            this.render(Ext.getBody());
        }
        Zenoss.flares.Container.superclass.show.apply(this, arguments);
    },
    onLayout: function() {
        Ext.each(this.items.items, function(item, index, items) {
            if ( item.canLayout() ) {
                if ( index == 0 ) {
                    item.anchorTo(this.el, this.alignment);
                }
                else {
                    item.anchorTo(items[index - 1].el, 'bl');
                }
            }
        }, this);
        Zenoss.flares.Container.superclass.onLayout.apply(this, arguments);
    }
});

/**
 * The UI manager for flares.
 */
Zenoss.flares.Manager = {
    container: new Zenoss.flares.Container(),
    INFO: 'x-flare-info',
    ERROR: 'x-flare-error',
    WARNING: 'x-flare-warning',
    DEBUG: 'x-flare-debug',
    SUCCESS: 'x-flare-success',
    CRITICAL: 'x-flare-critical',
    /**
     * Add the flare to the container and show it.
     *
     * @param flare Zenoss.flares.Flare
     */
    flare: function(flare) {
        flare.setAnimateTarget(Zenoss.flares.Manager.container.el);
        Zenoss.flares.Manager.container.add(flare);
        Zenoss.flares.Manager.container.doLayout();
        flare.show();
    },
    /**
     * Format a message and create a Flare.
     *
     * @param message string A message template
     * @param type string One of the status types assigned to this class (ex: INFO, ERROR)
     * @param args array Optional orguments to fill in the message template
     */
    _formatFlare: function(message, type, args) {
        args = Array.prototype.slice.call(args, 1);
        var flare = new Zenoss.flares.Flare(message, args, { iconCls: type });
        Zenoss.flares.Manager.flare(flare);
        return flare;
    },
    /**
     * Show a Flare with the info status.
     *
     * @param message string A message template
     * @param args mixed Optional orguments to fill in the message template
     */
    info: function(message, args) {
        return Zenoss.flares.Manager._formatFlare(message, Zenoss.flares.Manager.INFO, arguments);
    },
    error: function(message, args) {
        return Zenoss.flares.Manager._formatFlare(message, Zenoss.flares.Manager.ERROR, arguments);
    },
    warning: function(message, args) {
        return Zenoss.flares.Manager._formatFlare(message, Zenoss.flares.Manager.WARNING, arguments);
    },
    debug: function(message, args) {
        return Zenoss.flares.Manager._formatFlare(message, Zenoss.flares.Manager.DEBUG, arguments);
    },
    critical: function(message, args) {
        return Zenoss.flares.Manager._formatFlare(message, Zenoss.flares.Manager.CRITICAL, arguments);
    },
    success: function(message, args) {
        return Zenoss.flares.Manager._formatFlare(message, Zenoss.flares.Manager.SUCCESS, arguments);
    }
};

/**
 * Flares are growl like flash messages. Used for transient notifications. Flares are
 * managed by Zenoss.flares.Manager.
 *
 * Example:
 *
 * Zenoss.flares.Manager.info('{0} was saved as {1}.', itemName, newItemName);
 */
Zenoss.flares.Flare = Ext.extend(Ext.Window, {
    _task: null,
    _closing: false,
    focus: Ext.emptyFn,
    constructor: function(message, params, config) {
        if ( Ext.isArray(message) ) {
            var children = [];
            Ext.each(message, function(m) {
                children.push({ tag: 'li', html: m });
            });

            message = Ext.DomHelper.markup({
                tag: 'ul',
                children: children
            });
        }

        Ext.applyIf(config, {
            headerAsText: false,
            bodyCssClass: config.iconCls || 'x-flare-info',
            baseCls: 'x-flare',
            plain: false,
            draggable: false,
            shadow: false,
            closable: true,
            resizable: false,
            delay: 5000, // How long to show the message for
            alignment: 't-t',
            duration: 0.2, // How long to do the opening slide in
            hideDuration: 1, // How long to do the closing fade out
            template: new Ext.Template(message, { compiled: true } ),
            dismissOnClick: true
        });

        Ext.applyIf(config, {
            bodyCfg: {
                cls: 'x-flare-body',
                children: [
                    {cls: 'x-flare-icon'},
                    {
                        cls: 'x-flare-message',
                        html: config.template.apply(params)
                    }
                ]
            }
        });

        Zenoss.flares.Flare.superclass.constructor.call(this, config);
    },
    initEvents: function() {
        Zenoss.flares.Flare.superclass.initEvents.apply(this, arguments);

        if ( this.dismissOnClick ) {
            this.mon(this.el, 'click', function() { this.hide(); }, this);
        }
    },
    initComponent: function() {
        if ( this.delay ) {
            this._task = new Ext.util.DelayedTask(this.hide, this);
        }

        Zenoss.flares.Flare.superclass.initComponent.call(this);
    },
    _afterShow: function() {
        Zenoss.flares.Flare.superclass.show.call(this);
        if ( this._task ) {
            this._task.delay(this.delay);
       }
    },
    /**
     * Make this Flare "stick". It will not fade away and must manually be dismissed by the user.
     */
    sticky: function() {
        if ( this._task ) {
            this._task.cancel();
            delete this._task;
        }
    },
    animShow: function() {
        this.el.slideIn('t', {
            duration: this.duration,
            callback: this._afterShow,
            scope: this
        });
    },
    close: function() {
        this.hide();
    },
    animHide: function() {
        this._closing = true;
        this.el.ghost("b", {
            duration: this.hideDuration,
            remove: false,
            callback : function () {
                this.destroy();
            }.createDelegate(this)
        });
    },
    canLayout: function() {
        if ( !this._closing ) {
            // Only move if it's not already closing
            return Zenoss.flares.Flare.superclass.canLayout.apply(this, arguments);
        }
    }
});

Ext.namespace('Zenoss.messaging');
/**
 * Message for Zenoss.messaging.Messenger. This is for back-compat and should not be used directly.
 */
Zenoss.messaging.Message = Ext.extend(Object, {
    INFO: 0,                // Same as in messaging.py
    WARNING: 1,                // Same as in messaging.py
    CRITICAL: 2,                // Same as in messaging.py
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            body: '',
            priority: this.INFO,
            sticky: false
        });
        Ext.apply(this, config);
    }
});

/**
 * An interface to the old messaging API. This is for back-compat and should not be used directly.
 */
Zenoss.messaging.Messenger = Ext.extend(Ext.util.Observable, {
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            interval: 30000
        });
        Ext.apply(this, config);
        Zenoss.messaging.Messenger.superclass.constructor.call(this, config);
        this.addEvents('message');
    },
    init: function() {
        this._task = new Ext.util.DelayedTask(function(){
            this.checkMessages();
        }, this);
        this._task.delay(this.interval);
        this.checkMessages();
    },
    checkMessages: function() {
        Zenoss.remote.MessagingRouter.getUserMessages({}, function(results) {
            Ext.each(results.messages, function(m) {
                this.send(m);
            }, this);
        }, this);
    },
    send: function(msgConfig) {
        var message = new Zenoss.messaging.Message(msgConfig);
        this.fireEvent('message', this, message);

        var flare;
        if ( message.priority  === Zenoss.messaging.Message.WARNING ) {
            flare = Zenoss.flares.Manager.warning(message.body);
        }
        else if ( message.priority  === Zenoss.messaging.Message.CRITICAL ) {
            flare = Zenoss.flares.Manager.critical(message.body)
        }
        else {
            flare = Zenoss.flares.Manager.info(message.body);
        }

        if ( message.sticky ) {
            flare.sticky();
        }
    }
});

Zenoss.messenger = new Zenoss.messaging.Messenger();

Ext.onReady(function() {
    Zenoss.flares.Manager.container.show();
    Zenoss.messenger.init();
});

/**
 * Inform the user with a message. This is usually represented with a Flare. Use
 * this interface to alert users.
 */
Zenoss.message = {
     /**
     * Show a message with the info status.
     *
     * @param message string A message template
     * @param args mixed Optional orguments to fill in the message template
     */
    info: function(message, args) {
        Zenoss.flares.Manager.info.apply(null, arguments);
    },
    error: function(message, args) {
        Zenoss.flares.Manager.error.apply(null, arguments);
    },
    warning: function(message, args) {
        Zenoss.flares.Manager.warning.apply(null, arguments);
    },
    debug: function(message, args) {
        Zenoss.flares.Manager.debug.apply(null, arguments);
    },
    /**
     * These messages have a critical error icon and stay until dismissed by the user.
     */
    critical: function(message, args) {
        Zenoss.flares.Manager.critical.apply(null, arguments).sticky();
    },
    success: function(message, args) {
        Zenoss.flares.Manager.success.apply(null, arguments);
    }
}

/*
* Add the ability to specify an axis for autoScroll.
* autoScroll: true works just as before, but now can also do:
* autoScroll: 'x'
* autoScroll: 'y'
*
* Code by Tom23, http://www.extjs.com/forum/showthread.php?t=80663
*/

Ext.Element.prototype.setOverflow = function(v, axis) {
    axis = axis ? axis.toString().toUpperCase() : '';
    var overflowProp = 'overflow';
    if (axis == 'X' || axis == 'Y') {
        overflowProp += axis;
    }
    if(v=='auto' && Ext.isMac && Ext.isGecko2){ // work around stupid FF 2.0/Mac scroll bar bug
        this.dom.style[overflowProp] = 'hidden';
        (function(){this.dom.style[overflowProp] = 'auto';}).defer(1, this);
    }else{
        this.dom.style[overflowProp] = v;
    }
};

Ext.override(Ext.Panel, {
    setAutoScroll : function() {
        if(this.rendered && this.autoScroll){
            var el = this.body || this.el;
            if(el){
                el.setOverflow('auto', this.autoScroll);
            }
        }
    }
});

var origGetDragData = Ext.grid.GridDragZone.prototype.getDragData;
Ext.override(Ext.grid.GridDragZone, {
    getDragData: function(e) {
        var t = Ext.lib.Event.getTarget(e);
        // If it's a link, set the target to the ancestor cell so the browser
        // doesn't do the default anchor-drag behavior. Otherwise everything
        // works fine, so proceed as normal.
        if (t.tagName=='A') {
            e.target = e.getTarget('div.x-grid3-cell-inner');
        }
        return origGetDragData.call(this, e);
    }
});


/**
 * @class Zenoss.PlaceholderPanel
 * @extends Ext.Panel
 * A custom panel that displays text in its center. This panel is styled
 * with custom CSS to look temporary. It is used to show devs the names of
 * panels and should not be seen by users.
 * @constructor
 * @param {Object} config
 * @cfg {String} text The text to be displayed in the center of the panel.
 */
Zenoss.PlaceholderPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        Ext.apply(config, {
            cls: 'placeholder',
            layout: 'fit',
            border: false,
            items: [{
                baseCls: 'inner',
                border: false,
                html: config.text,
                listeners: {'resize':function(ob){
                        ob.getEl().setStyle({'line-height':
                            ob.getEl().getComputedHeight()+'px'});
                }}
            }]
        });
        Zenoss.PlaceholderPanel.superclass.constructor.apply(
            this, arguments);
    }
});

/**
 * @class Zenoss.LargeToolbar
 * @extends Ext.Toolbar
 * A toolbar with greater height and custom CSS class. Used at the top of
 * several screens, including the event console.
 * @constructor
 */
Zenoss.LargeToolbar = Ext.extend(Ext.Toolbar, {
    constructor: function(config) {
        Ext.applyIf(config, {
            cls: 'largetoolbar',
            height: 45,
            border: false,
            style:'border-top:1px solid #b8b8b8;border-bottom:1px solid #949494;'
        });
        Zenoss.LargeToolbar.superclass.constructor.apply(
            this, arguments);
    }
});

Ext.reg('largetoolbar', Zenoss.LargeToolbar);

/**
 * @class Zenoss.ExtraHooksSelectionModel
 * @extends Ext.grid.RowSelectionModel
 * A selection model that fires extra events.
 */
Zenoss.ExtraHooksSelectionModel = Ext.extend(
   Ext.ux.grid.livegrid.RowSelectionModel, {
    suppressDeselectOnSelect: false,
    initEvents: function() {
        Zenoss.ExtraHooksSelectionModel.superclass.initEvents.call(this);
        this.addEvents('rangeselect');
        this.on('beforerowselect', function(){
            if (this.suppressDeselectOnSelect) {
                this.selectingRow = true;
            }
        }, this);
    },
    clearSelections: function() {
        if (this.selectingRow) {
            this.suspendEvents();
        }
        Zenoss.ExtraHooksSelectionModel.superclass.clearSelections.apply(this, arguments);
        if (this.selectingRow) {
            this.resumeEvents();
            this.selectingRow = false;
        }
    },
    selectRange: function (startRow, endRow, keepExisting) {
        this.suspendEvents();
        Zenoss.ExtraHooksSelectionModel.superclass.selectRange.apply(
            this, arguments);
        this.resumeEvents();
        this.fireEvent('rangeselect', this);
    }
});


/**
 * @class Zenoss.PostRefreshHookableDataView
 * @extends Ext.DataView
 * A DataView that fires a custom event after the view has refreshed.
 * @constructor
 */
Zenoss.PostRefreshHookableDataView = Ext.extend(Ext.DataView, {
    constructor: function(config) {
        Zenoss.PostRefreshHookableDataView.superclass.constructor.apply(
            this, arguments);
        this.addEvents(
            /**
             * @event afterrefresh
             * Fires after the view has been rendered.
             * @param {DataView} this
             */
            'afterrefresh');
    }
});

Ext.extend(Zenoss.PostRefreshHookableDataView, Ext.DataView, {
    /**
     * This won't survive upgrade.
     */
    refresh: function(){
        this.clearSelections(false, true);
        this.el.update("");
        var records = this.store.getRange();
        if(records.length < 1){
            if(!this.deferEmptyText || this.hasSkippedEmptyText){
                this.el.update(this.emptyText);
            }
            this.hasSkippedEmptyText = true;
            this.all.clear();
            return;
        }
        this.tpl.overwrite(this.el, this.collectData(records, 0));
        this.fireEvent('afterrefresh', this);
        this.all.fill(Ext.query(this.itemSelector, this.el.dom));
        this.updateIndexes(0);
    }
});


/**
 * @class Zenoss.LiveGridInfoPanel
 * @extends Ext.Toolbar.TextItem
 * Toolbar addition that displays, e.g., "Showing 1-10 of 100 events"
 * @constructor
 * @grid {Object} the GridPanel whose information should be displayed
 */
Zenoss.LiveGridInfoPanel = Ext.extend(Ext.Toolbar.TextItem, {

    displayMsg : 'Displaying {0} - {1} of {2} events',
    emptyMsg: 'No events',
    cls: 'livegridinfopanel',

    initComponent: function() {
        this.setText(this.emptyMsg);
        if (this.grid) {
            this.grid = Ext.getCmp(this.grid);
            var me = this;
            this.view = this.grid.getView();
            this.view.init = this.view.init.createSequence(function(){
                me.bind(this);
            }, this.view);
        }
        Zenoss.LiveGridInfoPanel.superclass.initComponent.call(this);
    },
    updateInfo : function(rowIndex, visibleRows, totalCount) {
        var msg = totalCount === 0 ?
            this.emptyMsg :
            String.format(this.displayMsg, rowIndex+1,
                          rowIndex+visibleRows, totalCount);
        this.setText(msg);
    },
    bind: function(view) {
        this.view = view;
        var st = view.ds;

        /*
        st.on('loadexception',   this.enableLoading,  this);
        st.on('beforeload',      this.disableLoading, this);
        st.on('load',            this.enableLoading,  this);
        */
        view.on('rowremoved',    this.onRowRemoved,   this);
        view.on('rowsinserted',  this.onRowsInserted, this);
        view.on('beforebuffer',  this.beforeBuffer,   this);
        view.on('cursormove',    this.onCursorMove,   this);
        view.on('buffer',        this.onBuffer,       this);
        //view.on('bufferfailure', this.enableLoading,  this);

    },
    onCursorMove : function(view, rowIndex, visibleRows, totalCount) {
        this.updateInfo(rowIndex, visibleRows, totalCount);
    },

    onRowsInserted : function(view, start, end) {
        this.updateInfo(view.rowIndex, Math.min(view.ds.totalLength,
                view.visibleRows-view.rowClipped), view.ds.totalLength);
    },
    onRowRemoved : function(view, index, record) {
        this.updateInfo(view.rowIndex, Math.min(view.ds.totalLength,
                view.visibleRows-view.rowClipped), view.ds.totalLength);
    },
    beforeBuffer : function(view, store, rowIndex, visibleRows, totalCount,
                            options) {
        //this.loading.disable();
        this.updateInfo(rowIndex, visibleRows, totalCount);
    },
    onBuffer : function(view, store, rowIndex, visibleRows, totalCount) {
        //this.loading.enable();
        this.updateInfo(rowIndex, visibleRows, totalCount);
    }
});
Ext.reg('livegridinfo', Zenoss.LiveGridInfoPanel);

/**
 * @class Zenoss.FilterGridView
 * @extends Ext.grid.GridView
 * A GridView that includes a row below the header with a row filter.
 * @constructor
 */
Zenoss.FilterGridView = Ext.extend(Ext.ux.grid.livegrid.GridView, {
    rowHeight: 22,
    rowColors: false,
    liveSearch: true,
    selectionChanged: false,
    _valid: true,
    constructor: function(config) {
        if (typeof(config.displayFilters)=='undefined')
            config.displayFilters = true;
        Zenoss.FilterGridView.superclass.constructor.apply(this,
            arguments);
        Ext.applyIf(this.lastOptions, this.defaultFilters || {});
    },
    lastOptions: {},
    initEvents: function(){
        Zenoss.FilterGridView.superclass.initEvents.call(this);
        this.addEvents('filterchange');
        this.addEvents('filtertoggle');
    },
    initData: function(ds, cm) {
        this.un('beforebuffer', this.onBeforeBuffer,  this);
        cm.un('hiddenchange',   this.updateHeaders,   this);

        this.on('beforebuffer', this.onBeforeBuffer,  this);
        cm.on('hiddenchange',   this.updateHeaders,   this);

        Zenoss.FilterGridView.superclass.initData.call(this, ds, cm);

    },
    // Return an object representing the state of the grid
    getFilterParams: function(hash) {
        var sinfo = this.grid.store.sortInfo,
            o = {
                sort: sinfo.field,
                dir: sinfo.direction
            };
        this.applyFilterParams({params:o});
        if (hash) {
            o.hashcheck = this.grid.lastHash;
        }
        return o;
    },
    // Gather the current values of the filter and apply them to a given
    // object.
    applyFilterParams: function(options) {
        var params = this.lastOptions || {},
            i, filter, oldformat, query, dt;
        options = options || {};
        for(i=0;i<this.filters.length;i++){
            filter = this.filters[i];
            query = filter.getValue();
            if (query) {
                params[filter.id] = query;
                if (filter.xtype=='datefield'){
                    dt = new Date(query);
                    query = dt.format(
                        Zenoss.date.UniversalSortableDateTime);
                }
            } else {
                delete params[filter.id];
            }
        }
        Ext.apply(options.params, {
            params: Ext.util.JSON.encode(params),
            uid: this.contextUid
        });
        // Store them for later, just in case
        this.lastOptions = params;
    },
    setContext: function(uid) {
        this.contextUid = uid;
        this.updateLiveRows(this.rowIndex, true, true, false);
    },
    updateLiveRows: function(index, forceRepaint, forceReload) {
        if ( this.isValid() ) {
            return Zenoss.FilterGridView.superclass.updateLiveRows.call(this, index, forceRepaint, forceReload);
        }
        return false;
    },
    onBeforeLoad: function(store, options) {
        this.applyFilterParams(options);
        return Zenoss.FilterGridView.superclass.onBeforeLoad.call(this,
            store, options);
    },
    onBeforeBuffer: function(view, store, rowIndex, visibleRows,
                             totalCount, options) {
        this.applyFilterParams(options);
        return true;
    },
    getFilterCt: function() {
        return Ext.select('.x-grid3-filter');
    },
    hideFilters: function() {
        this.setFiltersDisplayed(false);
    },
    showFilters: function() {
        this.setFiltersDisplayed(true);
    },
    clearFilters: function() {
        this._valid = true;
        Ext.each(this.filters, function(ob){
            // clear out any original values
            // (for instance when we come here from the device page)
            if (ob.originalValue) {
                ob.originalValue = null;
                ob.setValue(null);
            }
            ob.reset();
        }, this);
        this.updateLiveRows(this.rowIndex, true, true, false);
    },
    validateFilters: function() {
        var valid = true;
        Ext.each(this.filters, function(ob){
            if ( ob.isValid != undefined ) {
                valid = ob.isValid(true) && valid;
            }
        }, this);
        this._valid = valid;
        return valid;
    },
    getErrors: function() {
        var errors = [];
        Ext.each(this.filters, function(ob){
            if ( ob.getActiveError != undefined ) {
                var err = ob.getActiveError();
                if ( err ) {
                    errors.push(err);
                }
            }
        }, this);

        return errors;
    },
    setFiltersDisplayed: function(bool) {
        // For now, always show the filters The rest of the filter-hiding
        // stuff is still in place, so just remove this and put back the
        // menu item when we're ready to do so.
        bool = true;
        this.displayFilters = bool;
        this.getFilterCt().setDisplayed(bool);
        this.fireEvent('filtertoggle');
        if(bool) this.renderEditors();
        this.layout();
    },
    renderFilterRow: function() {
        var cs = this.getColumnData(),
            ct = new Ext.Template(
                '<td class="x-grid3-col x-grid3-cell',
                ' x-grid3-td-{id} {css}" style="{style}" tabIndex="-1"',
                ' {cellAttr}> <div class="x-grid3-cell-inner',
                ' x-grid3-col-{id}" unselectable="on" {attr}>',
                '{value}</div></td>'),
            buf = [],
            p = {},
            rp = {},
            rt = new Ext.Template('<tr {display} class="x-grid3-filter"',
                '>{cells}</tr>'),
            i, len, c;
        for (i=0,len=cs.length; i<len; i++) {
            if (this.cm.isHidden(i)) continue;
            c = cs[i];
            p.id = c.id;
            p.css = 'x-grid3-cell-filter';
            p.attr ='id="filtergrid-'+c.id+'"';
            p.cellAttr = "";
            p.value = '';
            buf[buf.length] = ct.apply(p);
        }
        rp.tstyle = 'width:'+this.getTotalWidth()+';';
        rp.cols = buf.length;
        rp.cells = buf.join("");
        //rp.display = this.displayFilters?'':'style="display:none"'
        rp.display = '';
        return rt.apply(rp);
    },
    filters: [],
    /*
     * this.reset() has an annoying habit of blurring existing filter
     * fields and taking away row stripes. This is the resetting part of
     * the code, but we use updateLiveRows() instead of refresh(), which
     * fixes the problem.
     */
    nonDisruptiveReset: function() {
        this.ds.modified = [];
        //this.grid.selModel.clearSelections(true);
        this.rowIndex      = 0;
        this.lastScrollPos = 0;
        this.lastRowIndex = 0;
        this.lastIndex    = 0;
        this.adjustVisibleRows();
        this.adjustScrollerPos(-this.liveScroller.dom.scrollTop,
            true);
        this.showLoadMask(false);
        //this.reset(false);
        this.updateLiveRows(this.rowIndex, true, true);
    },
    renderEditors: function() {
        Ext.each(this.filters, function(ob){ob.destroy();});
        this.filters = [];
        var cs = this.getColumnData(),
            i, len, c, config, fieldid, id;
        for (i=0,len=cs.length; i<len; i++) {
            if (this.cm.isHidden(i)) continue;
            c = cs[i];
            fieldid = c.id;
            id = 'filtergrid-' + fieldid;
            config = this.cm.config[i].filter;
            if (config===false) {
                config = {xtype: 'panel', reset: function(){},
                          getValue: function(){}};
                this.filters[this.filters.length] = Ext.create(config);
                continue;
            } else if (!config) {
                config = {xtype:'textfield'};
            }
            Ext.apply(config, {
                id:fieldid,
                enableKeyEvents: true,
                selectOnFocus: true,
                listeners: {
                    render: function(){
                        Zenoss.registerTooltipFor(fieldid);
                    }
                }
            });
            Ext.applyIf(config, {
                validationDelay: 500,
                validateOnBlur: false
            });

            var filter = new Ext.ComponentMgr.create(config);
            if (this.lastOptions) {
                var newValue = this.lastOptions[fieldid];
                filter.setValue(newValue);
            }
            filter.setWidth('100%');
            this.filters[this.filters.length] = filter;

            filter.liveSearchTask = new Ext.util.DelayedTask(function() {
                this.validFilter('');
            }, this);

            if (filter instanceof Ext.form.TextField) {
                filter.on('valid', this.validFilter, this);
                filter.on('invalid', this.onInvalidFilter, this);
            }

            if (filter instanceof Zenoss.MultiselectMenu ||
                filter instanceof Ext.form.DateField)
            {
                filter.on('select', function(field, e) {
                    view = Ext.getCmp('events_grid').getView();
                    view.selectionChanged = true;
                    this.liveSearchTask.delay(100);
                }, filter);
                filter.on('change', function(field, e) {
                    view = Ext.getCmp('events_grid').getView();
                    view.selectionChanged = true;
                    this.liveSearchTask.delay(100);
                }, filter);
            }
            new Ext.Panel({
                frame: false,
                border: false,
                layout: 'fit',
                renderTo: id,
                items: filter
            });
        }
    },
    isValid: function() {
        return this._valid;
    },
    /**
     * Strange bug on live grid when you clear a filter, the live grid
     * reloads, but doesn't render properly. You get a scrollbar but nothing else.
     * This just completely reloads the grid's datastore.
     **/
    reloadOnEmpty: function(filter) {
        var forceReload = true;
        if (Ext.isEmpty(filter.getValue())) {
            this.reset(forceReload);
        }
    },
    validFilter: function() {
        // Check all filters to determine if we are valid
        this.validateFilters();
        this.fireEvent('filterchange', this);

        if (this.liveSearch || this.selectionChanged) {
            this.nonDisruptiveReset();
            this.selectionChanged = false;
        }
    },
    onInvalidFilter: function() {
        // If one filter is invalid, all are
        this._valid = false;
        this.fireEvent('filterchange', this);

        var errors = this.getErrors();
        if ( errors ) {
            Zenoss.message.error(errors);
        }
        else {
            Zenoss.message.error(_t('Error loading, grid has invalid filters.'));
        }
    },
    updateHeaders: function() {
        Zenoss.FilterGridView.superclass.updateHeaders.call(this);
        this.renderEditors();
    },
    renderUI: function() {
        Zenoss.FilterGridView.superclass.renderUI.call(this);
    },
    renderHeaders : function(){
        var html = Zenoss.FilterGridView.superclass.renderHeaders.call(this);
        html = html.slice(0, html.length-8);
        html = html + this.renderFilterRow() + '</table>';
        return html;
    },
    getState: function(){
        // Update last options from the filter widgets
        this.applyFilterParams();
        // Iterate over last options, setting defaults where appropriate
        var options = {};
        Ext.iterate(this.lastOptions, function(k){
            var defaults = this.defaultFilters || {},
                dflt = defaults[k],
                opt = this.lastOptions[k];
            if (dflt) {
                var match;
                if (Ext.isDate(dflt) && Ext.isDate(opt)) {
                    var delta = Math.abs(dflt.getTime() - opt.getTime());
                    // If they're within a second, they match. We don't
                    // have finer resolution in the UI.
                    match = delta <= 1000;
                } else {
                    match = dflt==opt;
                }
                if (!match) {
                    options[k] = opt;
                }
            } else {
                options[k] = opt;
            }
        }, this);
        return {
            displayFilters: this.displayFilters,
            options: options
        };
    },
    getFilterButton: function(){
        return Ext.getCmp(this.filterbutton);
    },
    onRowSelect: function(row) {
        if (this.rowColors) {
            this.addRowClass(row, this.selectedRowClass + '-rowcolor');
        }
        Zenoss.FilterGridView.superclass.onRowSelect.apply(this, arguments);
    },
    onRowDeselect: function(row) {
        if (this.rowColors) {
            this.removeRowClass(row, this.selectedRowClass + '-rowcolor');
        }
        Zenoss.FilterGridView.superclass.onRowDeselect.apply(this, arguments);
    },
    onRowOver: function(e, t) {
        if (this.rowColors) {
            if((row = this.findRowIndex(t)) !== false && !e.within(this.getRow(row), true)){
                this.addRowClass(row, 'x-grid3-row-over-rowcolor');
            }
        }
        Zenoss.FilterGridView.superclass.onRowOver.apply(this, arguments);
    },
    onRowOut: function(e, t) {
        if (this.rowColors) {
            if((row = this.findRowIndex(t)) !== false && !e.within(this.getRow(row), true)){
                this.removeRowClass(row, 'x-grid3-row-over-rowcolor');
            }
        }
        Zenoss.FilterGridView.superclass.onRowOut.apply(this, arguments);
    },
    getRowClass: function(record, index) {
        var stateclass = record.get('eventState')=='New' ?
                            'unacknowledged':'acknowledged';
        var sev = Zenoss.util.convertSeverity(record.get('severity'));
        var rowcolors = this.rowColors ? 'rowcolor rowcolor-' : '';
        var cls = rowcolors + sev + '-' + stateclass + ' ' + stateclass;
        return cls;
    },
    toggleRowColors: function(bool){
        this.rowColors = bool;
        Ext.state.Manager.set('rowcolor', bool);
        this.updateLiveRows(this.rowIndex, true, false);
    },
    toggleLiveSearch: function(bool){
        this.liveSearch = bool;
        Ext.state.Manager.set('livesearch', bool);
    },
    applyState: function(state) {
        // For now, always show the filters The rest of the filter-hiding
        // stuff is still i'll gn place, so just remove this and put back the
        // menu item when we're ready to do so.
        this.displayFilters = true; //state.displayFilters;
        // End always show filters
        this.lastOptions = state.options;
        // Apply any default filters specified in the constructor
        Ext.applyIf(this.lastOptions, this.defaultFilters || {});
        /*
        var btn = Ext.getCmp(this.filterbutton);
        btn.on('render', function(){
            this.setChecked(state.displayFilters);
        }, btn);
        */
    },
    resetFilters: function(){
        this.lastOptions = {};
        Ext.applyIf(this.lastOptions, this.defaultFilters || {});
        //this.getFilterButton().setChecked(false);
    }
});

/**
 * @class Zenoss.FilterGridPanel
 * @extends Ext.grid.GridPanel
 * A GridPanel that uses a FilterGridView.
 * @constructor
 */
Zenoss.FilterGridPanel = Ext.extend(Ext.ux.grid.livegrid.GridPanel, {
    initStateEvents: function(){
        Zenoss.FilterGridPanel.superclass.initStateEvents.call(this);
        this.mon(this.view, 'filterchange', this.saveState, this);
        this.mon(this.view, 'filtertoggle', this.saveState, this);
    },
    getView : function(){
        if(!this.view){
            this.view = new Zenoss.FilterGridView(this.viewConfig);
        }
        return this.view;
    },
    restoreURLState: function() {
        var qs = window.location.search.replace(/^\?/, ''),
            state = Ext.urlDecode(qs).state,
            noop;
        if (state) {
            try {
                state = Ext.decode(Zenoss.util.base64.decode(state));
                this.applyState(state);

            } catch(e) { noop(); }
        }
    },
    clearURLState: function() {
        var qs = Ext.urlDecode(window.location.search.replace(/^\?/, ''));
        if (qs.state) {
            delete qs.state;
            qs = Ext.urlEncode(qs);
            if (qs) {
                window.location.search = '?' + Ext.urlEncode(qs);
            } else {
                window.location.search = '';
            }
        }
    },
    getPermalink: function() {
        var l = window.location,
            path = l.protocol + '//' + l.host + l.pathname,
            st = Zenoss.util.base64.encode(Ext.encode(this.getState()));
        return path + '?state=' + st;
    },
    initState: function() {
        Zenoss.FilterGridPanel.superclass.initState.apply(this, arguments);
        this.restoreURLState();
        var livesearchitem = Ext.getCmp(this.view.livesearchitem);
        if(livesearchitem) {
            var liveSearch = Ext.state.Manager.get('livesearch');
            if (!Ext.isDefined(liveSearch)){
                liveSearch = livesearchitem.checked;
            }else{
                livesearchitem.on('render', function(){
                    this.setChecked(liveSearch);
                },livesearchitem);
            }
            this.view.liveSearch = liveSearch;
        }
        var rowColors = Ext.state.Manager.get('rowcolor');
        this.view.rowColors = rowColors;
        if (this.view.rowcoloritem) {
            var rowcoloritem = Ext.getCmp(this.view.rowcoloritem);
            rowcoloritem.on('render', function(){
                this.setChecked(rowColors);
            }, rowcoloritem);
        }
    },
    getState: function(){
        var val = Zenoss.FilterGridPanel.superclass.getState.call(this);
        var filterstate = this.getView().getState();
        val.filters = filterstate;
        return val;
    },
    applyState: function(state){
        // We need to remove things from the state that don't apply to this
        // context, so we'll ditch things referring to columns that aren't
        // in the initial config.
        var availcols = Ext.pluck(this.initialConfig.cm.config, 'id'),
            cols = [],
            filters = {};
        Ext.each(state.columns, function(col){
            if (availcols.indexOf(col.id)>-1) cols.push(col);
        });
        Ext.iterate(state.filters.options, function(op){
            if (availcols.indexOf(op)>-1) {
                filters[op] = state.filters.options[op];
            }
        });
        state.columns = cols;
        state.filters.options = filters;
        // Apply the filter information to the view, the rest to this
        this.getView().applyState(state.filters);
        Zenoss.FilterGridPanel.superclass.applyState.apply(this, arguments);
    },
    resetGrid: function() {
        Ext.state.Manager.clear(this.getItemId());
        var view = this.getView();
        view.resetFilters();
        Zenoss.remote.EventsRouter.column_config({}, function(result){
            var results = [];
            Ext.each(result, function(r){
                results[results.length] = Ext.decode(r);
            });
            var cm = new Ext.grid.ColumnModel(results);
            this.store.sortInfo = this.store.defaultSort;
            this.reconfigure(this.store, cm);
            view.fitColumns();
            view.showLoadMask(true);
            view.nonDisruptiveReset();
            this.saveState();
            this.clearURLState();
        }, this);
    },
    setContext: function(uid) {
        this.view.setContext(uid);
    },
    hideFilters: function() { this.getView().hideFilters(); },
    showFilters: function() { this.getView().showFilters(); },
    clearFilters: function() { this.getView().clearFilters(); }
});

Ext.reg('filtergridpanel', Zenoss.FilterGridPanel);

/**
 * @class Zenoss.MultiselectMenu
 * @extends Ext.Toolbar.Button
 * A combobox-like menu that allows one to toggle each option, and is able
 * to deliver its value like a form field.
 * @constructor
 */
Zenoss.MultiselectMenu = Ext.extend(Ext.Toolbar.Button, {
    makeItemConfig: function(text, value) {
        var config = {
            hideOnClick: false,
            handler: function() {
                this.fireEvent('change');
            }.createDelegate(this),
            value: value,
            text: text
        };
        return config;
    },
    constructor: function(config) {
        config.menu = config.menu || [];
        Zenoss.MultiselectMenu.superclass.constructor.apply(this, arguments);
        if (Ext.isDefined(config.store)) {
            this.hasLoaded = false;
            config.store.on('load', function(s, rows) {
                this.menu.removeAll();
                Ext.each(rows, function(row){
                    var cfg = this.makeItemConfig(row.data.name, row.data.value);
                    cfg.checked = (this.defaultValues.indexOf(row.data.value)>-1);
                    this.menu.add(cfg);
                }, this);
                this.hasLoaded = true;
            }, this);
            config.store.load();
        } else {
            this.hasLoaded = true;
            Ext.each(config.source, function(o){
                var cfg = this.makeItemConfig(o.name, o.value);
                cfg.checked = !Ext.isDefined(o.checked);
                this.menu.add(cfg);
            }, this);
        }
    },
    reset: function() {
        this.setValue();
    },
    getValue: function() {
        if (!this.hasLoaded) {
            return this.defaultValues;
        }
        var result = [];
        Ext.each(this.menu.items.items, function(item){
            if (item.checked) result[result.length] = item.value;
        });
        return result;
    },
    setValue: function(val) {
        if (!val) {
            this.constructor(this.initialConfig);
        } else {
            Ext.each(this.menu.items.items, function(item){
                var shouldCheck = false;
                try{
                    shouldCheck = val.indexOf(item.value)!=-1;
                } catch(e) {var _x;}
                item.setChecked(shouldCheck);
            });
        }
    }

});
Ext.reg('multiselectmenu', Zenoss.MultiselectMenu);

/**
 * @class Zenoss.StatefulRefreshMenu
 * @extends Ext.Menu
 * A refresh menu that is able to save and restore its state.
 * @constructor
 */
Zenoss.StatefulRefreshMenu = Ext.extend(Ext.menu.Menu, {
    constructor: function(config) {
        config.stateful = true;
        config.stateEvents = ['itemclick'];
        Zenoss.StatefulRefreshMenu.superclass.constructor.apply(this, arguments);
    },
    getState: function() {
        //returning raw value doesn't work anymore; need to wrap in object/array
        return [this.trigger.interval];
    },
    applyState: function(interval) {
        //old cookie value not being in an array and we can't get the value, so
        //default to 60
        var savedIntveral = interval[0] || 60;
        var items = this.items.items;
        Ext.each(items, function(item) {
            if (item.value == savedIntveral)
                item.setChecked(true);
        }, this);
        this.trigger.on('afterrender', function() {
            this.trigger.setInterval(savedIntveral);
        }, this);
    }
});

Ext.reg('statefulrefreshmenu', Zenoss.StatefulRefreshMenu);

/**
 * @class Zenoss.RefreshMenu
 * @extends Ext.SplitButton
 * A button that manages refreshing and allows the user to set a polling
 * interval.
 * @constructor
 */
Zenoss.RefreshMenuButton = Ext.extend(Ext.SplitButton, {
    constructor: function(config) {
        var menu = {
            xtype: 'statefulrefreshmenu',
            id: 'evc_refresh',
            trigger: this,
            items: [{
                xtype: 'menutextitem',
                cls: 'refreshevery',
                text: 'Refresh every'
            },{
                xtype: 'menucheckitem',
                text: '1 second',
                value: 1,
                group: 'refreshgroup'
            },{
                xtype: 'menucheckitem',
                text: '5 seconds',
                value: 5,
                group: 'refreshgroup'
            },{
                xtype: 'menucheckitem',
                text: '10 seconds',
                value: 10,
                group: 'refreshgroup'
            },{
                xtype: 'menucheckitem',
                text: '30 seconds',
                value: 30,
                group: 'refreshgroup'
            },{
                xtype: 'menucheckitem',
                text: '1 minute',
                value: 60,
                checked: true,
                group: 'refreshgroup'
            },{
                xtype: 'menucheckitem',
                text: 'Manually',
                value: -1,
                group: 'refreshgroup'
            }]
        };
        config.menu = menu;
        Zenoss.RefreshMenuButton.superclass.constructor.apply(this,
            arguments);
        this.refreshTask = new Ext.util.DelayedTask(this.poll, this);
        this.menu.on('itemclick', function(item){
            this.setInterval(item.value);
        }, this);
        //60 is the default interval; it matches the checked item above
        this.setInterval(60);
    },
    setInterval: function(interval) {
        this.interval = interval;
        this.refreshTask.delay(this.interval*1000);
    },
    poll: function(){
        if (this.interval>0) {
            if ( !this.disabled ) {
                this.pollHandler();
            }
            this.refreshTask.delay(this.interval*1000);
        }
    }
});
Ext.reg('refreshmenu', Zenoss.RefreshMenuButton);


// FIXME: Refactor this to be much, much smarter about its own components.
Zenoss.DetailPanel = Ext.extend(Ext.Panel, {
    isHistory: false,
    constructor: function(config){
        config.onDetailHide = config.onDetailHide || function(){var _x;};
        config.layout = 'border';
        config.border = false;
        config.defaults = {border:false};
        config.items = [{
            id: 'evdetail_hd',
            region: 'north',
            layout: 'border',
            height: 50,
            cls: 'evdetail_hd',
            defaults: {border: false},
            items: [{
                region: 'west',
                width: 77,
                layout: 'hbox',
                defaults: {border: false},
                items: [{
                    id: 'severity-icon',
                    cls: 'severity-icon'
                },{
                    id: 'evdetail-sep',
                    cls: 'evdetail-sep'
                }]
            },{
                region: 'center',
                id: 'evdetail-summary',
                html: ''
            },{
                region: 'east',
                id: 'evdetail-tools',
                layout: 'hbox',
                width: 57,
                defaults: {border: false},
                items: [{
                    id: 'evdetail-popout',
                    cls: 'evdetail-popout'
                },{
                    id: 'evdetail_tool_close',
                    cls: 'evdetail_close'
                }]
            }]
        },{
            id: 'evdetail_bd',
            region: 'center',
            defaults: {
                frame: false,
                border: false
            },
            autoScroll: true,
            layout: 'table',
            layoutConfig: {
                columns: 1,
                tableAttrs: {
                    style: {
                        width: '90%'
                    }
                }
            },
            cls: 'evdetail_bd',
            items: [{
                id: 'evdetail_props',
                cls: 'evdetail_props',
                html: ''
            },{
                id: 'show_details',
                cls: 'show_details',
                hidden: true,
                html: 'Show more details...'
            },{
                id: 'full_event_props',
                cls: 'full_event_props',
                hidden: true,
                html: ''
            },{
                id: 'evdetail-log-header',
                cls: 'evdetail-log-header',
                hidden: true,
                html: '<'+'hr/><'+'h2>LOG<'+'/h2>'
            },{
                xtype: 'form',
                id: 'log-container',
                defaults: {border: false},
                frame: true,
                layout: 'table',
                style: {'margin-left':'3em'},
                hidden: true,
                labelWidth: 1,
                items: [{
                    id: 'detail-logform-evid',
                    xtype: 'hidden',
                    name: 'evid'
                },{
                    style: 'margin:0.75em',
                    width: 300,
                    xtype: 'textfield',
                    name: 'message',
                    hidden: Zenoss.Security.doesNotHavePermission('Manage Events'),
                    id: 'detail-logform-message'
                },{
                    xtype: 'button',
                    type: 'submit',
                    name: 'add',
                    hidden: Zenoss.Security.doesNotHavePermission('Manage Events'),
                    text: 'Add',
                    handler: function(btn, e){
                        var form = Ext.getCmp('log-container'),
                            vals = form.getForm().getValues(),
                            params = {history:this.isHistory};
                        Ext.apply(params, vals);
                        Zenoss.remote.EventsRouter.write_log(
                         params,
                         function(provider, response){
                             Ext.getCmp(
                                 'detail-logform-message').setRawValue('');
                             Ext.getCmp(config.id).load(
                                 Ext.getCmp(
                                     'detail-logform-evid').getValue());
                        });
                    }
                }]
            },{
                id: 'evdetail_log',
                cls: 'log-content',
                hidden: true
            }]
        }];
        Zenoss.DetailPanel.superclass.constructor.apply(this, arguments);
    },
    setSummary: function(summary){
        var panel = Ext.getCmp('evdetail-summary');
        if (panel && panel.el){
            panel.el.update(summary);
        }
    },
    setSeverityIcon: function(severity){
        var panel = Ext.getCmp('severity-icon');
        this.clearSeverityIcon();
        panel.addClass(severity);
    },
    clearSeverityIcon: function() {
        var panel = Ext.getCmp('severity-icon');
        Ext.each(Zenoss.env.SEVERITIES,
            function(sev){
                sev = sev[1];
                panel.removeClass(sev.toLowerCase());
            }
        );
    },
    update: function(event) {
        // For the Event Detail Page, set up the page
        // links. This is to make sure they link to the correct place
        // when we go to the new UI

        // device_link
        event.device_link = Zenoss.render.Device(event.device_url,
                                                event.device_title);
        // component_link
        if (event.component_url) {
            event.component_link = Zenoss.render.default_uid_renderer(
                                    event.component_url,
                                    event.component_title);
        }else {
            event.component_link = event.component_title;
        }

        // eventClass_link
        event.eventClass_link = Zenoss.render.EventClass(event.eventClass_url,
                                                        event.eventClass);

        var top_prop_template = new
            Ext.XTemplate.from('detail_table_template');
        var full_prop_template = new
            Ext.XTemplate.from('fullprop_table_template');
        var log_template = new Ext.XTemplate.from('log_table_template');
        var severity = Zenoss.util.convertSeverity(event.severity),
            html = top_prop_template.applyTemplate(event),
            prophtml = full_prop_template.applyTemplate(event),
            loghtml = log_template.applyTemplate(event);


        this.setSummary(event.summary);
        this.setSeverityIcon(severity);
        Ext.getCmp('evdetail_props').el.update(html);
        Ext.getCmp('full_event_props').el.update(prophtml);
        Ext.getCmp('evdetail_log').el.update(loghtml);
        Ext.getCmp('detail-logform-evid').setValue(event.evid);
    },
    wipe: function(){
        this.clearSeverityIcon();
        this.setSummary('');
        Ext.getCmp('show_details').hide();
        Ext.getCmp('evdetail-log-header').hide();
        Ext.getCmp('evdetail_log').hide();
        Ext.getCmp('evdetail_props').hide();
        Ext.getCmp('full_event_props').hide();
        Ext.getCmp('log-container').hide();
    },
    show: function(){
        Ext.getCmp('show_details').show();
        Ext.getCmp('evdetail-log-header').show();
        Ext.getCmp('evdetail_log').show();
        Ext.getCmp('evdetail_props').show();
        if (this.isPropsVisible)
            Ext.getCmp('full_event_props').show();
        Ext.getCmp('log-container').show();
    },
    isPropsVisible: false,
    showProps: function(){
        var el = Ext.getCmp('full_event_props'),
            tgl = Ext.getCmp('show_details');
        if (!el.hidden){
            el.hide();
            this.isPropsVisible = false;
            tgl.body.update('Show more details...');
        } else {
            el.show();
            this.isPropsVisible = true;
            tgl.body.update('Hide details');
        }
    },
    popout: function(){
         var evid = Ext.getCmp('detail-logform-evid').getValue(),
             url = this.isHistory ? 'viewHistoryDetail' : 'viewDetail';
         url = url +'?evid='+evid;
         window.open(url, evid.replace(/-/g,'_'),
             "status=1,width=600,height=500");
    },
    bind: function(){
        var showlink = Ext.getCmp('show_details').getEl(),
            btn = Ext.getCmp('evdetail_tool_close').getEl(),
            pop = Ext.getCmp('evdetail-popout').getEl();

        showlink.un('click', this.showProps);
        showlink.on('click', this.showProps);
        if (btn){
            btn.un('click', this.onDetailHide);
            btn.on('click', this.onDetailHide);
        }
        if (pop){
            pop.un('click', this.popout, this);
            pop.on('click', this.popout, this);
        }
    },
    load: function(event_id){
        Zenoss.remote.EventsRouter.detail({
                evid:event_id,
                history:this.isHistory
            },
            function(result){
                var event = result.event[0];
                this.update(event);
                this.bind();
                this.show();
            }, this);
    }
});
Ext.reg('detailpanel', Zenoss.DetailPanel);


/**
 * @class Zenoss.ColumnFieldSet
 * @extends Ext.form.FieldSet
 * A FieldSet with a column layout
 * @constructor
 */
Zenoss.ColumnFieldSet = Ext.extend(Ext.form.FieldSet, {
    constructor: function(userConfig) {

        var baseConfig = {
            items: {
                layout: 'column',
                border: false,
                items: userConfig.__inner_items__,
                defaults: {
                    layout: 'form',
                    border: false,
                    bodyStyle: 'padding-left: 15px'
                }
            }
        };

        delete userConfig.__inner_items__;
        var config = Ext.apply(baseConfig, userConfig);
        Zenoss.ColumnFieldSet.superclass.constructor.call(this, config);

    } // constructor
}); // Zenoss.ColumnFieldSet

Ext.reg('ColumnFieldSet', Zenoss.ColumnFieldSet);


/**
 * General utilities
 */
Ext.namespace('Zenoss.util');

Zenoss.util.addLoadingMaskToGrid = function(grid){
    // load mask stuff
    grid.store.proxy.on('beforeload', function(){
        var container = this.container;
        container._treeLoadMask = container._treeLoadMask || new Ext.LoadMask(this.container);
        var mask = container._treeLoadMask;
        mask.show();
    }, grid);
    grid.store.proxy.on('load', function(){
        var container = this.container;
        container._treeLoadMask = container._treeLoadMask || new Ext.LoadMask(this.container);
        var mask = container._treeLoadMask;
        mask.hide();
    }, grid);
}

Zenoss.env.SEVERITIES = [
    [5, 'Critical'],
    [4, 'Error'],
    [3, 'Warning'],
    [2, 'Info'],
    [1, 'Debug'],
    [0, 'Clear']
];

Zenoss.util.convertSeverity = function(severity){
    if (Ext.isString(severity)) return severity;
    var sevs = ['clear', 'debug', 'info', 'warning', 'error', 'critical'];
    return sevs[severity];
};

Zenoss.util.convertStatus = function(stat){
    var stati = ['New', 'Acknowledged', 'Suppressed'];
    return stati[stat];
};

Zenoss.util.render_severity = function(sev) {
    return Zenoss.render.severity(sev);
};

Zenoss.util.render_status = function(stat) {
    return Zenoss.render.evstatus(stat);
};

Zenoss.util.render_linkable = function(name, col, record) {
    var url = record.data[col.id + '_url'];
    var title = record.data[col.id + '_title'] || name;
    if (url) {
        return '<a href="'+url+'">'+title+'</a>';
    } else {
        return title;
    }
};


Zenoss.util.render_device_group_link = function(name, col, record) {

    var links = record.data.DeviceGroups.split('|'),
        returnString = "",
        link = undefined;
    // return a pipe-deliminated set of links to the ITInfrastructure page
    for (var i = 0; i < links.length; i++) {
        link = links[i];
        if (link) {
            returnString +=  '&nbsp;|&nbsp;' + Zenoss.render.DeviceGroup(link, link) ;
        }
    }

    return returnString;
};


Zenoss.util.base64 = {
    base64s : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_",
    encode: function(decStr){
        if (typeof btoa === 'function') {
             return btoa(decStr);
        }
        var base64s = this.base64s,
            i = 0,
            encOut = "",
            bits, dual, x, y, z;
        while(decStr.length >= i + 3){
            x = (decStr.charCodeAt(i) & 0xff) <<16;
            i++;
            y = (decStr.charCodeAt(i) & 0xff) <<8;
            i++;
            z = decStr.charCodeAt(i) & 0xff;
            i++;
            bits = x | y | z;
            encOut += base64s.charAt((bits & 0x00fc0000) >>18) +
                      base64s.charAt((bits & 0x0003f000) >>12) +
                      base64s.charAt((bits & 0x00000fc0) >> 6) +
                      base64s.charAt((bits & 0x0000003f));
        }
        if(decStr.length -i > 0 && decStr.length -i < 3){
            dual = Boolean(decStr.length -i -1);
            x = ((decStr.charCodeAt(i) & 0xff) <<16);
            i++;
            y = (dual ? (decStr.charCodeAt(i) & 0xff) <<8 : 0);
            bits = x | y;
            encOut += base64s.charAt((bits & 0x00fc0000) >>18) +
                      base64s.charAt((bits & 0x0003f000) >>12) +
                      (dual ? base64s.charAt((bits & 0x00000fc0) >>6) : '=') + '=';
        }
        return(encOut);
    },
    decode: function(encStr){
        if (typeof atob === 'function') {
            return atob(encStr);
        }
        var base64s = this.base64s;
        var bits;
        var decOut = "";
        var i = 0;
        for(; i<encStr.length; i += 4){
            bits = (base64s.indexOf(encStr.charAt(i)) & 0xff) <<18 |
                   (base64s.indexOf(encStr.charAt(i +1)) & 0xff) <<12 |
                   (base64s.indexOf(encStr.charAt(i +2)) & 0xff) << 6 |
                   base64s.indexOf(encStr.charAt(i +3)) & 0xff;
            decOut += String.fromCharCode((bits & 0xff0000) >>16,
                                          (bits & 0xff00) >>8, bits & 0xff);
        }
        if(encStr.charCodeAt(i -2) == 61){
            return(decOut.substring(0, decOut.length -2));
        }
        else if(encStr.charCodeAt(i -1) == 61){
            return(decOut.substring(0, decOut.length -1));
        }
        else {
            return(decOut);
        }
    }
};

// two functions for converting IP addresses
Zenoss.util.dot2num = function(dot) {
    var d = dot.split('.');
    return ((((((+d[0])*256)+(+d[1]))*256)+(+d[2]))*256)+(+d[3]);
};

Zenoss.util.num2dot = function(num) {
    var d = num % 256;
    for (var i = 3; i > 0; i--) {
        num = Math.floor(num/256);
        d = num%256 + '.' + d;
    }
    return d;
};

Zenoss.util.setContext = function(uid) {
    var ids = Array.prototype.slice.call(arguments, 1);
    Ext.each(ids, function(id) {
        Ext.getCmp(id).setContext(uid);
    });
};

/**
 * Doing Ext.each's job for it. If it's an array, do Ext.each, if the obj has an each function, use it instead.
 * @param {Object} obj Array or MixedCollection works here
 * @param {Function} filterFn
 * @param {Object} scope
 */
Zenoss.util.each = function(obj, filterFn, scope) {
    if ( Ext.isFunction(obj.each) ) {
        obj.each(filterFn, scope);
    }
    else {
        Ext.each(obj, filterFn, scope);
    }
}

/**
 * Return an array filtered by function argument; Filter function should
 * return true if a value should be included in the filtered result
 * @param {Object} arr; array to be filtered
 * @param {Object} filterFn; function used to filter
 */
Zenoss.util.filter = function(arr, filterFn, scope) {
    var result = [];

    Zenoss.util.each(arr, function(val) {
        var include = filterFn.call(scope || this, val);
        if (include) {
            result.push(val);
        }
    });

    return result;
};

/**
 * Copies all the properties of values to orig only if they already exist.
 * @param {Object} orig The receiver of the properties
 * @param {Object} values The source of the properties
 * @return {Object} returns orig
 **/
Zenoss.util.applyNotIf = function(orig, values) {
    var k;
    if (orig) {
        for (k in values) {
            if (k in orig) {
                orig[k] = values[k];
            }
        }
    }
    return orig;
};

/**
 * This converts server side types to Ext Controls,
 * it first looks for specific types based on the field name
 * and then reverts to a translation of the type.
 * @param {string} fieldId the "name" of the field (e.g. eventClass)
 * @param {string} type can be string, int, etc
 * @returns {string} The "xtype" of the control
 **/
Zenoss.util.getExtControlType = function(fieldId, type) {
    var customControls = {
        'eventClass': 'EventClass',
        'severity': 'Severity',
        'dsnames': 'DataPointItemSelector'
    },
    types = {
        'int': 'numberfield',
        'string': 'textfield',
        'boolean': 'checkbox',
        'text': 'textarea'
    };

    // see if a component of this type is registered (then return it)
    if (Ext.ComponentMgr.isRegistered(fieldId)) {
        return fieldId;
    }

    // check our conversions defined above
    if (customControls[fieldId]) {
        return customControls[fieldId];
    }

    // default to "textfield" if we don't have it set up yet"
    return (types[type] || 'textfield');
};

/**
 * Proxy that will only allow one request to be loaded at a time.  Requests
 * made while the proxy is already loading a previous requests will be discarded
 */
Zenoss.ThrottlingProxy = Ext.extend(Ext.data.DirectProxy, {
    constructor: function(config){
        Zenoss.ThrottlingProxy.superclass.constructor.apply(this, arguments);
        this.loading = false;
        //add event listeners for throttling
        this.addListener('beforeload', function(proxy, options){
            if (!proxy.loading){
                proxy.loading = true;
                return true;
            }
            return false;
        });
        this.addListener('load', function(proxy, options){
            proxy.loading = false;
        });
        this.addListener('exception', function(proxy, options){
            proxy.loading = false;
        });

    }
});

/**
 * Zenoss date patterns and manipulations
 */
Ext.namespace('Zenoss.date');

/**
 * A set of useful date formats. All dates should come from the server as
 * ISO8601Long, but we may of course want to render dates in many different
 * ways.
 */
Ext.apply(Zenoss.date, {
    ISO8601Long:"Y-m-d H:i:s",
    ISO8601Short:"Y-m-d",
    ShortDate: "n/j/Y",
    LongDate: "l, F d, Y",
    FullDateTime: "l, F d, Y g:i:s A",
    MonthDay: "F d",
    ShortTime: "g:i A",
    LongTime: "g:i:s A",
    SortableDateTime: "Y-m-d\\TH:i:s",
    UniversalSortableDateTime: "Y-m-d H:i:sO",
    YearMonth: "F, Y"
});

// Fix an IE bug
Ext.override(Ext.Shadow, {
    realign: Ext.Shadow.prototype.realign.createInterceptor(
        function(l, t, w, h) {
            if (Ext.isIE) {
                var a = this.adjusts;
                a.h = Math.max(a.h, 0);
            }
        }
    )
});

// Force checkbox to fire valid
var oldcbsetvalue = Ext.form.Checkbox.prototype.setValue;
Ext.override(Ext.form.Checkbox, {
    setValue: function(v) {
        var result = oldcbsetvalue.call(this, v);
        this.fireEvent('valid', this);
        return result;
    }
});

String.prototype.startswith = function(){
    return (this.match('^'+str)==str)
};

String.prototype.endswith = function(){
    return (this.match(str+'$')==str)
};

})(); // End local scope
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

/* package level */
(function() {
     // NOTE: All permissions are handled on the server, this is just to
     // enhance the user experience
     Ext.ns('Zenoss.Security');
     
     // Defined in ZenUI3/security/security.py this is a dictionary of all the
     // permissions the current user has on the current context.
     var all_permissions = _global_permissions(),
         callbacks = [];

     /**
      * The main method for ACL on the front end. It asks
      * if the current user has this permission.
      * The permissions are defined in Zope
      * 
      * NOTE: The permissions are not case-sensitive here
      * @returns True/False if the user has permission or not
      **/
     Zenoss.Security.hasPermission = function(permission) {
         // uses all_permissions as a closure
         return all_permissions[permission.toLowerCase()];
     };

     /**
      * Convenience method, it makes the Hide and Disable properties
      * easier to read.
      * For instance:
      *      config {
      *           hidden: Zenoss.Security.doesNotHavePermission('Manage DMD');
      *      };
      * @returns True if the user does NOT have permission
      **/
     Zenoss.Security.doesNotHavePermission = function(permission) {
         return (!this.hasPermission(permission));           
     };     

     /**
      * Add an callback to be executed every time the permissions
      * are changed, usually this is done by changing context.
      * Example Usage:
      *     Zenoss.Security.onPermissionChange(function() {
      *         Ext.getCmp('foo').setDisabled(Zenoss.Security.doesNotHavePermission('Manage DMD'));
      * });
      *@param callback: function to execute
      *@param scope[Optional]: scope of the callback
      **/
     Zenoss.Security.onPermissionsChange = function(callback, scope) {
         if (scope) {
             callbacks.push(callback.createDelegate(scope));
         }else {
             callbacks.push(callback);   
         }       
     };
     
     /**
      * If the context you are working on changes call this
      * method to update the security permissions for that new context
      **/
     Zenoss.Security.setContext = function(uid) {
         var params = {
             uid:uid
         };
         function callback(response) {
             var i;
             if (response.success) {
                 all_permissions = response.data;
                 if (callbacks) {
                     for (i = 0; i < callbacks.length; i += 1) {
                         callbacks[i]();
                     }
                 }
             }
         }
         Zenoss.remote.DetailNavRouter.getSecurityPermissions(params, callback);    
         
     };
     
}());/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

(function() {
     Ext.ns('Zenoss.Action');
          
     Zenoss.Action = Ext.extend(Ext.Action, {
         constructor: function(config) {
             // if they set the permissions config property
             // and the logged in user does not have permission, hide this element
             if (config.permission){
                 config.disabled = Zenoss.Security.doesNotHavePermission(config.permission);
             }
             
             // call the parent's constructor
             Zenoss.Action.superclass.constructor.apply(this, arguments);   
         }             
     });

     Ext.reg('Action', Zenoss.Action);
}());
(function(){

var _tm = {
    'DeviceLocation':   ["^/zport/dmd/Locations(/[A-Z][^/]*)*/?$"],
    'DeviceGroup':      ["^/zport/dmd/Groups(/[A-Z][^/]*)*/?$"],
    'DeviceClass':      ["^/zport/dmd/Devices(/[A-Z][^/]*)*/?$"],
    'Device':           ["^/zport/dmd/.*/devices/[^/]*/?$"],
    'EventClass':       ["^/zport/dmd/Events(/[A-Z][^/]*)*/?$"],
    'Network':          ["^/zport/dmd/Networks(/(?!ipaddresses)[^/]+)*/?$"],
    'IpAddress':        ["^/zport/dmd/.*/ipaddresses/[^/]*/?$"],
    'Process':          ["^/zport/dmd/Processes(.*)$"],
    'IpInterface':      ["^/zport/dmd/Devices/.*/devices/.*/os/interfaces/.*"]
};

var T = Ext.ns('Zenoss.types');

Ext.apply(T, {

    TYPES: {},

    getAllTypes: function() {
        var result = [];
        for (var k in T.TYPES) {
            if (true) {
                result.push(k);
            }
        }
        return result;
    }, // getAllTypes

    type: function(uid) {
        var _f;
        for (var type in T.TYPES) {
            if (T.TYPES[type]) {
                _f = true;
                Ext.each(T.TYPES[type], function(test) {
                    if (!_f) return;
                    _f = test.test(uid);
                });
                if (_f) return type;
            }
        }
        return null;
    }, // getType

    register: function(config) {
        function addRegex(k, t) {
            var types = T.TYPES[k] = T.TYPES[k] || [];
            if (!(t instanceof RegExp)) {
                t = new RegExp(t);
            }
            if (!(t in types)) types.push(t);
        }
        for (var k in config) {
            var t = config[k];
            if (Ext.isString(t)) {
                addRegex(k, t);
            } else if (Ext.isArray(t)) {
                Ext.each(t, function(r) {
                    addRegex(k, r);
                });
            }

        }
    } // register

}); // Ext.apply

T.register(_tm);

})(); // End local scope
(function(){

Ext.ns('Zenoss.render');

// templates for the events renderer
var iconTemplate = new Ext.Template(
    '<td class="severity-icon-small {severity} {noevents}">{count}</td>');
iconTemplate.compile();

var rainbowTemplate = new Ext.Template(
    '<table class="eventrainbow eventrainbow_cols_{count}"><tr>{cells}</tr></table>');
rainbowTemplate.compile();

var upDownTemplate = new Ext.Template(
    '<span class="status-{0}{2}">{1}</span>');
upDownTemplate.compile();

function convertToUnits(num, divby, unitstr, places){
    unitstr = unitstr || "B";
    places = places || 2;
    divby = divby || 1024.0;
    var units = [];
    Ext.each(['', 'K', 'M', 'G', 'T', 'P'], function(p){
        units.push(p+unitstr);
    });
    var sign = 1;
    if (num < 0) {
        num = Math.abs(num);
        sign = -1;
    }
    var i;
    for (i=0;i<units.length;i++) {
        if (num<divby) {
            break;
        }
        num = num/divby;
    }
    return (num*sign).toFixed(places) + units[i];
}

function pingStatusBase(bool) {
    if (Ext.isString(bool)) {
        bool = bool.toLowerCase();
        if (bool=='none') {
            return 'Unknown';
        }
        bool = bool=='up';
    }
    var str = bool ? 'Up' : 'Down';
    return str;
}

Ext.apply(Zenoss.render, {

    bytesString: function(num) {
        return num===0 ? '0' :convertToUnits(num, 1024.0, 'B');
    },

    memory: function(mb) {
        return (mb === 0) ? '0' : convertToUnits(mb, 1024.0, 'B', 2);
    },

    cpu_speed: function(speed) {
        if (speed) {
            var n = parseFloat(speed);
            if (isNaN(n)) {
                return speed;
            } else {
                return n.toFixed(2) + ' GHz'; // XXX I18N unfriendly
            }
        } else {
            return speed;
        }
    },

    pingStatus: function(bool) {
        var str = pingStatusBase(bool);
        return upDownTemplate.apply([str.toLowerCase(), str]);
    },

    pingStatusLarge: function(bool) {
        var str = pingStatusBase(bool);
        return upDownTemplate.apply([str.toLowerCase(), str, '-large']);
    },

    upDownUnknown: function(status,displayString){
        return upDownTemplate.apply([status.toLowerCase(),displayString]);
    },

    upDownUnknownLarge: function(status,displayString){
        return upDownTemplate.apply([status.toLowerCase(),displayString,'-large']);
    },

    ipAddress: function(ip) {
        if (Ext.isObject(ip)) {
            ip = ip.name;
        }
        if (!ip||ip=='0.0.0.0') {
            return '';
        }
        return Ext.isString(ip) ? ip : Zenoss.util.num2dot(ip);
    },

    severity: function(sev) {
        return '<div class="severity-icon-small '+
            Zenoss.util.convertSeverity(sev) +
            '"'+'><'+'/div>';
    },

    // renders availability as a percentage with 3 digits after decimal point
    availability: function(value) {
        return Ext.util.Format.number(value*100, '0.000%');
    },

    evstatus: function(evstatus) {
        return '<div class="status-icon-small-'+evstatus.toLowerCase()+'"><'+'/div>';
    },

    events: function(value, count) {
        var result = '',
            sevs = ['critical', 'error', 'warning', 'info', 'debug', 'clear'];
        count = count || 3;
        Ext.each(sevs.slice(0, count), function(severity) {
            var noevents = (0 === value[severity]) ? 'no-events' : '';
            result += iconTemplate.apply({
                severity: severity,
                count:value[severity],
                noevents: noevents
            });
        });
        return rainbowTemplate.apply({cells: result, count: count});
    },

    locking: function(obj) {
        /*
        * Expects an object with keys updates, deletion, events, all boolean
        */
        var from = [];
        if (obj.updates) {
            from.push('updates');
        }
        if (obj.deletion) {
            from.push('deletion');
        }
        if (!Ext.isEmpty(from)) {
            var l = _t('Locked from ') + from.join(_t(' and '));
            if (obj.events) {
                l += "<br/>"+_t("Send event when blocked");
            } else {
                l += "<br/>"+_t("Do not send event when blocked");
            }
            return l;
        } else {
            return _t('Unlocked');
        }
    },

    locking_icons: function(obj) {
        /*
        * Expects an object with keys updates, deletion, events, all boolean
        * Returns images representing locking status.
        */
        var tpl = new Ext.Template(
            '<img border="0" src="locked-{str}-icon.png"',
            'style="vertical-align:middle"/>'),
            result = '';
        tpl.compile();
        if (obj.updates) {
            result += tpl.apply({str:'update'});
        }
        if (obj.deletion) {
            result += tpl.apply({str:'delete'});
        }
        if (obj.events) {
            result += tpl.apply({str:'sendevent'});
        }
        return result;
    },

    /*
     * Given a uid, determines the type of the object and passes rendering
     * off to the appropriate rendering function.
     * e.g. Zenoss.render.link('/zport/dmd/Devices/Server') =>
     * <a href="/zport/dmd/itinfrastructure#devices:/Devices/Server/Linux">...
     *
     * Can also just accept a url and name for wrapping in an anchor tag, by
     * passing in null for the first argument.
     */
    link: function(uid, url, name) {
        if (!url) {
            var dflt = 'default_uid_renderer',
                type = Zenoss.types.type(uid) || dflt,
                renderer = Zenoss.render[type];
            if (renderer) {
                return renderer(uid, name);
            }
        }
        if (url && name) {
            return '<a class="z-entity" href="'+url+'">'+name+'</a>';
        }
    },

    default_uid_renderer: function(uid, name) {
        // Just straight up links to the object.
        var parts;
        if (!uid) {
            return uid;
        }
        if (Ext.isObject(uid)) {
            name = uid.name;
            uid = uid.uid;
        }
        if (!name) {
            parts = uid.split('/');
            name = parts[parts.length-1];
        }
        return Zenoss.render.link(null, uid, name);
    },

    linkFromGrid: function(name, col, record) {
        var item;
        if (typeof(record.data[col.id]) == 'object') {
            item = record.data[col.id];

            if (item.uid) {
                return Zenoss.render.link(item.uid, null, item.text);
            }
            return item.text;
        }

        return name;
    },

    Device: function(uid, name) {
        // For now, link to the old device page
        return Zenoss.render.link(null, uid+'/devicedetail', name);
    },

    DeviceClass: function(uid, name) {
        var value = uid.replace(/^\/zport\/dmd\/Devices/, '');
        value = value.replace(/\/devices\/.*$/, '');
        var url = '/zport/dmd/itinfrastructure#devices:.zport.dmd.Devices' + value.replace(/\//g,'.');
        if (!Ext.isString(name)) name = value;
        return Zenoss.render.link(null, url, name);
    },

    DeviceLocation: function(uid, name) {
        var value = uid.replace(/^\/zport\/dmd\/Locations/, '');
        value = value.replace(/\/devices\/.*$/, '');
        var url = '/zport/dmd/itinfrastructure#locs:.zport.dmd.Locations' + value.replace(/\//g,'.');
        if (!Ext.isString(name)) name = value;
        return Zenoss.render.link(null, url, name);
    },

    DeviceGroup: function(uid, name) {
        var value = uid.replace(/^\/zport\/dmd\/Groups/, '');
        value = value.replace(/\/devices\/.*$/, '');
        var url = '/zport/dmd/itinfrastructure#groups:.zport.dmd.Groups' + value.replace(/\//g,'.');
        if (!Ext.isString(name)) name = value;
        return Zenoss.render.link(null, url, name);
    },

    DeviceComponent: function(name, col, record) {
        var item = record.data[col.id];
        if (item.uid){
            // TODO once these pages are built fix the link
            return Zenoss.render.default_uid_renderer(item.uid, item.text);
        }
        return item.text;
    },

    EventClass: function(uid, name) {
        return Zenoss.render.default_uid_renderer(uid, name);
    },

    IpServiceClass: function(value, metadata, record, rowIndex, colIndex, store) {
        // this is intended to set directly as a column renderer instead of
        // using Types.js. See the Ext.grid.ColumnModel.setRenderer
        // documentation
        var uid = record.data.serviceClassUid.replace(/\//g, '.');
        return Zenoss.render.serviceClass('ipservice', uid, value);
    },
    
    WinServiceClass: function(value, metadata, record, rowIndex, colIndex, store) {
        // this is intended to set directly as a column renderer instead of
        // using Types.js. See the Ext.grid.ColumnModel.setRenderer
        // documentation
        var uid = record.data.serviceClassUid.replace(/\//g, '.');
        return Zenoss.render.serviceClass('winservice', uid, value);
    },

    serviceClass: function(page, uid, name) {
        var url = String.format('/zport/dmd/{0}#navTree:{1}', page, uid);
        return Zenoss.render.link(null, url, name);
    },

    nextHop: function(value, metadata, record, rowIndex, colIndex, store) {
        var link = "";
        if (value && value.uid && value.id) {
            link += Zenoss.render.IpAddress(value.uid, value.id);
            if (value.device && value.device.uid && value.device.id) {
                link += " (";
                link += Zenoss.render.Device(value.device.uid, value.device.id);
                link += ")";
            }
        }
        return link;
    },

    IpInterface: function(uid, name) {
        var deviceUid = uid.split('/os/interfaces/')[0];
        var url = deviceUid + '/devicedetail#deviceDetailNav:IpInterface:' + uid;
        return Zenoss.render.link(null, url, name);
    },

    IpAddress: function(uid, name) {
        return Zenoss.render.Network(uid, name);
    },

    Network: function(uid, name) {
        var url = '/zport/dmd/networks#networks:' + uid.replace(/\//g, '.');
        if (!name) {
            var parts = uid.split('/');
            name = parts[parts.length-1];
        }
        return Zenoss.render.link(null, url, name);
    },
    
    Process: function(uid, name) {
        var url = '/zport/dmd/process#processTree:' + uid.replace(/\//g, '.');
        if (!name) {
            var parts = uid.split('/');
            name = parts[parts.length-1];
        }
        return Zenoss.render.link(null, url, name);
    }

}); // Ext.apply

})(); // End local namespace
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/
(function () {
Ext.ns('Zenoss');

Zenoss.FlexButton = Ext.extend(Ext.Button, {
    constructor: function(config) {
        if ( config.menu && config.menu.items && config.menu.items.length == 1 ) {
            // Only has one menu item, lets just not have it display a menu
            config.originalMenu = Ext.apply({}, config.menu);

            // probably don't want to inherit the text
            var menuConfig = config.menu.items[0]
            Ext.apply(config, {listeners: menuConfig.listeners});
            Ext.destroyMembers(config, 'menu');
        }

        Zenoss.FlexButton.superclass.constructor.call(this, config);
    },
    add: function(config) {
        if ( !this.menu ) {
            // this button does not have a menu yet so we need to initialize it
            // update the config with things that may have changed since creation
            var menuConfig = {};
            if ( this.initialConfig.originalMenu ) {
                // originally had a menu, just use it
                Ext.apply(menuConfig, this.initialConfig.originalMenu);
            }
            else {
                // Have to generate a menu config from the original button
                menuConfig.items = [{
                    text: this.getText() ? this.getText() : this.tooltip,
                    listeners: this.initialConfig.listeners
                }];
            }

            // Clear out properties that should be handled by the menu item
            this.on('render', function() {
                this.events['click'].clearListeners();
            }); // *TODO* This code makes me feel dirty; there must be a better way
            this.clearTip();

            this.menu = Ext.menu.MenuMgr.get(menuConfig);
        }

        return this.menu.add(config);
    }
});

Ext.reg('FlexButton', Zenoss.FlexButton);
})();(function(){

Ext.ns('Zenoss', 'Zenoss.i18n');

// Provide a default; this gets filled in later when appropriate.
Zenoss.i18n._data = Zenoss.i18n._data || {};

Zenoss.i18n.translate = function(s, d) {
    var t = Zenoss.i18n._data[s];
    return t ? t : (d ? d: s);
};

// Shortcut
window._t = Zenoss.i18n.translate;

})();
/*
 * Tooltips.js
 */
(function(){ // Local scope
/*
 * Zenoss.registerTooltip
 *
 * Make QuickTips also accept a target component or component ID to attach a
 * tooltip to. It will attempt to discover the correct Ext.Element to which it
 * should attach the tip.
 *
 * Accepts the same config options as the Ext.ToolTip constructor.
 *
 */

Zenoss.TIPS = {};

/*
* Zenoss.ToolTip
* Causes tooltips to remain visible if the mouse is over the tip itself,
* instead of hiding as soon as the mouse has left the target.
*/
Zenoss.ToolTip = Ext.extend(Ext.ToolTip, {
   constructor: function(config) {
       Zenoss.ToolTip.superclass.constructor.call(this, config);
       this.on('render', this.attachHoverEvents, this);
   },
   attachHoverEvents: function() {
       var el = this.getEl();
       el.on('mouseenter', this.onMouseEnter, this);
       el.on('mouseleave', this.onTargetOut, this);
   },
   onMouseEnter: function(e) {
       this.clearTimer('hide');
   }
});

Zenoss.registerTooltip = function(config) {
    var t, target = config.target,
        initialConfig = Ext.apply({}, config),
        cmp = Ext.getCmp(target);
    if (typeof(cmp)!="undefined") {
        cmp.on('destroy', function(){
            Zenoss.TIPS[target] = initialConfig;
        });
        if (cmp.btnEl) {
            config.target = cmp.btnEl;
        }
    } else {
        Ext.ComponentMgr.onAvailable(target, function(t){
            t.tooltip = config;
        });
    }
    if ((t=Ext.get(target))) {
        var tip = new Zenoss.ToolTip(config);
        Zenoss.TIPS[target] = tip;
    } else {
        Zenoss.TIPS[target] = initialConfig;
    }
}; // Zenoss.registerTooltip

/*
 * Zenoss.registerTooltipFor
 * 
 * Looks up any tooltips for a component id and registers them. Used for items
 * that don't exist when Zenoss.registerTooltip was called.
 *
 * If a tooltip is already registered, don't reregister it.
 *
 */
Zenoss.registerTooltipFor = function(target) {
    var t;
    if (Ext.isDefined(t = Zenoss.TIPS[target])) {
        if (!(t instanceof Ext.ToolTip)) {
            Zenoss.registerTooltip(t);
        }
    }
}; // Zenoss.registerTooltipFor


})(); // End local scope
(function(){
    Ext.ns('Zenoss.VTypes');

    var ip_regex = new RegExp("(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)");
    var hex_regex = new RegExp("^#?([a-f]|[A-F]|[0-9]){3}(([a-f]|[A-F]|[0-9]){3,5})?$");
    var numcmp_regex = new RegExp("^(\>=|\<=|\>|\<|=)?\s*([0-9]+)$");
    var alpha_num_space = new RegExp(/[a-z_\s\d]/i);

    /**
     * These are the custom validators defined for
     * our zenoss forms. The xtype/vtype custom is to have the
     * name of the vtype all lower case with no word separator.
     *
     **/
    var vtypes = {
        /**
         * Allows int comparisons like 4,>2,<=5
         */
        numcmp: function(val, field) {
            return numcmp_regex.test(val);
        },
        numcmpText: _t('Enter a valid comparison (ex: 4, <2, >=1)'),

        /**
         * The number must be greater than zero. Designed for us in NumberFields
         **/
        positive: function(val, field) {
            return (val >= 0);
        },
        positiveText: _t('Must be greater than or equal to 0'),

        /**
         * Between 0 and 1 (for float types)
         **/
        betweenzeroandone: function(val, field) {
            return (val >= 0 && val <=1);
        },
        betweenzeroandoneText: _t('Must be between 0 and 1'),
        ipaddress: function(val, field) {
            return ip_regex.test(val);
        },
        ipaddressText: _t('Invalid IP address'),

        /**
         * Hex Number (for colors etc)
         **/
        hexnumber: function(val, field) {
            return hex_regex.test(val);
        },
        hexnumberText: _t('Must be a 6 or 8 digit hexadecimal value.'),

        /**
         * Modifies alpha number to allow spaces
         **/
        alphanumspace: function(val, field) {
            return alpha_num_space.test(val);
        },
        alphanumspaceText: _t('Must be an alphanumeric value or a space '),
        alphanumspaceMask: alpha_num_space
    };

    Ext.apply(Ext.form.VTypes, vtypes);
}());

/*
 * Utility for keeping track of navigation among subcomponents on a page and
 * restoring that state on page load. 
 */
(function(){

var H = Ext.History;
H.DELIMITER = ':';

H.selectByToken = function(token) {
    if(token) {
        var parts = token.split(H.DELIMITER),
            mgr = Ext.getCmp(parts[0]),
            remainder = parts.slice(1).join(H.DELIMITER);
        if (mgr) {
            mgr.selectByToken(remainder);
        }
    }
};

H.on('change', H.selectByToken);

})();
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2009, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

Zenoss.DisplayField = Ext.extend(Ext.form.DisplayField, {
    constructor: function(config) {
        Ext.applyIf(config, {
            fieldClass: 'display-field'
        });
        Zenoss.DisplayField.superclass.constructor.call(this, config);
    },
    setRawValue: function(v) {
        if (v && Ext.isIE && typeof v === 'string') {
            v = v.replace(/\n/g, '<br/>');
        }
        Zenoss.DisplayField.superclass.setRawValue.call(this, v);
    }
});

Ext.reg('displayfield', Zenoss.DisplayField);


Zenoss.EditorWithButtons = Ext.extend(Ext.Editor, {
    onRender: function(ct, position) {
        Zenoss.EditorWithButtons.superclass.onRender.apply(this, arguments);
        this.editorpanel = new Ext.Panel({
            frame: true,
            bodyStyle: Ext.isIE ? 'padding-bottom:5px' : '',
            buttonAlign: 'left',
            buttons: [{
                text:_t('Save'), 
                handler: this.completeEdit.createDelegate(this)
            }, {
                text:_t('Cancel'), 
                handler: this.cancelEdit.createDelegate(this)
            }],
            items: this.field
        });
        if (Ext.isIE) {
            // IE can't set the layer width properly; always gets set to 11189
            this.el.setWidth(450);
        }
        this.editorpanel.render(this.el).show();
    },
    onBlur: function(){
        // do nothing
    }
});

Ext.reg('btneditor', Zenoss.EditorWithButtons);


Zenoss.EditableField = Ext.extend(Zenoss.DisplayField, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            cls: 'editable-field',
            overCls: 'editable-field-over',
            editEvent: 'click'
        });
        config.editor = Ext.applyIf(config.editor||{}, {
            xtype: 'btneditor',
            updateEl: !Ext.isIE,
            ignoreNoChange: true,
            autoSize: 'width',
            field: {
                xtype: 'textfield',
                selectOnFocus: true
            }
        });
        Zenoss.EditableField.superclass.constructor.call(this, config);
        this.initEditor();
        this.on('render', function(){
            this.getEl().on(this.editEvent, this.startEdit, this);
        }, this);
    },
    initEditor: function() {
        if (!(this.editor instanceof Ext.Editor)) {
            this.editor = Ext.create(this.editor);
        }
        var ed = this.editor;
        ed.on('beforecomplete', this.onBeforeComplete, this);
        if (Ext.isIE) {
            ed.field.origSetValue = ed.field.setValue;
            ed.field.setValue = function(v){
                v = v.replace(/\<BR\>/g, '\n');
                return this.origSetValue(v);
            }.createDelegate(ed.field);
        }
    },
    getForm: function() {
        var ownerCt = this.ownerCt;
        while (ownerCt && !(ownerCt instanceof Ext.FormPanel)) {
            ownerCt = ownerCt.ownerCt;
        }
        return ownerCt;
    },
    onBeforeComplete: function(t, val, startVal) {
        var opts = Ext.apply({}, this.getForm().baseParams||{});
        opts[this.name] = val;
        this.getForm().api.submit(opts, function(result) {
            if (result.success) {
                this.setValue(result[this.name] || val);
            } else {
                this.setValue(startVal);
            }
        }, this);
    },
    startEdit: function(e, t) {
        if (!this.disabled) {
            this.editor.startEdit(t);
        }
    }
});

Ext.reg('editable', Zenoss.EditableField);


Zenoss.EditableTextarea = Ext.extend(Zenoss.EditableField, {
    constructor: function(config) {
        config.editor = config.editor || {};
        config.editor.field = Ext.applyIf(config.editor.field||{}, {
            xtype: 'textarea',
            grow: true
        });
        Zenoss.EditableTextarea.superclass.constructor.call(this, config);
    }
});

Ext.reg('editabletextarea', Zenoss.EditableTextarea);


})();
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

/* package level */
(function() {

Ext.ns('Zenoss.form');

Zenoss.form.FieldRow = Ext.extend(Ext.Container, {
     constructor: function(config) {
         Ext.apply(config, {
             layout: 'hbox',
             align: 'top'
         });

         var items = config.items;
         config.items = [];

         Ext.each(items, function(item) {
             var cfg = {
                xtype: 'container',
                autoHeight: true,
                minWidth: 10,
                width: item.width,
                labelAlign: item.labelAlign || 'top',
                layout: 'form',
                items: [item]
             };
             if (!item.width) {
                 cfg.flex = item.flex || 1;
             }
             config.items.push(cfg);
         });

         Zenoss.form.FieldRow.superclass.constructor.call(this, config);

     }
 });
 Ext.reg('fieldrow', Zenoss.form.FieldRow);

})();
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

(function() {

Ext.ns('Zenoss.form');

Zenoss.form.InheritField = Ext.extend(Zenoss.form.FieldRow, {
    constructor: function(config) {
        var item;

        // Take the first member of items
        if (Ext.isObject(config.items)) {
            item = config.items;
        } else {
            item = config.items[0];
        }

        // Add the checkbox
        this.inheritbox = new Ext.form.Checkbox({
            fieldLabel: _t('Inherit?'),
            width: 60,
            handler: function(me) {
                console.log(me.checked ? 'checked' : 'unchecked');
            }
        });
        config.items = [this.inheritbox, item];
        Zenoss.form.InheritField.superclass.constructor.call(this, config);
        this.field = this.items.items[1].items.items[0];
    },
    inherit: function() {
        return this.inheritbox.checked;
    },
    getValue: function() {
        return {
            inherited: this.inherit(),
            value: this.field.getValue()
        };
    },
    setValue: function(value) {
        if (Ext.isObject(value)) {
            this.inheritbox.setValue(
                Ext.isDefined(value.inherited) ? value.inherited : false
            );
            value = value.value;
        } else {
            this.inheritbox.setValue(false);
        }
        this.field.setValue(value);
    }
 });
 Ext.reg('inheritfield', Zenoss.form.InheritField);

})();
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

/*
A widget that handles the fact that zProperties can be acquired from parents
in the dmd object hierarchy. The strategy is to use a field set. The title of
the field set is a friendly name for the zProperty (not the z* name). Inside
the field set are two lines. The user selects a line as being the active
setting by clicking on the radio button at the beginning of the row. The top
line has a label that reads 'Set Local Value:', and an appropriate widget for
the type of the zProp (checkbox, textfield, numberfield, or combobox). The 2nd
line states 'Inherit Value' and then indicates the acquired value and the path
of the ancestor from which the value is acquired.

Example usage:

var zMonitor = Ext.create({
    xtype: 'zprop',
    name: 'zMonitor',
    title: _t('Is Monitoring Enabled?'),
    localField: {
        xtype: 'select',
        model: 'local',
        store: [[true, 'Yes'], [false, 'No']],
        value: true
    }
});

zMonitor.setValues({
    isAcquired: false,
    localValue: false,
    acquiredValue: 'No',
    ancestor: '/Processes/Apache'
});

*/

(function() {

Ext.ns('Zenoss.form');


/* A radio button that allows for the boxLabel to be updated.
 */
Zenoss.form.Radio = Ext.extend(Ext.form.Radio, {
    
    setBoxLabel: function(boxLabel) {
        if (this.rendered) {
            this.wrap.down('.x-form-cb-label').update(boxLabel);
        } else {
            this.boxLabel = boxLabel;
        }
    }
    
});

Ext.reg('zradio', Zenoss.form.Radio);


/* A hidden field used internally by the ZProperty fieldset. This hidden field
   overrides getValue to return an object with isAcquired and localValue.
   Introduces the zpropFieldSet config prop which is a reference to the
   ZProperty field set.
 */
Zenoss.form.ZPropHidden = Ext.extend(Ext.form.Hidden, {

    getValue: function() {
        return {
            isAcquired: this.zpropFieldSet.acquiredRadio.getValue(),
            localValue: this.zpropFieldSet.localField.getValue(),
            
            // acquiredValue and ancestor aren't needed by the server, but it
            // is needed by reset which is called when the form is submitted
            acquiredValue: this.zpropFieldSet.acquiredValue,
            ancestor: this.zpropFieldSet.ancestor
        };
    },
    
    setValue: function(values) {
        this.zpropFieldSet.setValues(values);
    },
    
    isDirty: function() {
        return this.zpropFieldSet.acquiredRadio.isDirty() || this.zpropFieldSet.localField.isDirty();
    }

});

Ext.reg('zprophidden', Zenoss.form.ZPropHidden);


/*
A field set that represents a zProperty.

The config parameter passed into the constructor must have a ref. The refOwner
must be the FormPanel.

Additional config keys:
    localField - config for the Ext.form.Field used to input a local setting
    name - string that that is submited to the server as the name
    
New public method:
    setValues - upon context change in the client code, set all the values
                of this composite widget
 */
Zenoss.form.ZProperty = Ext.extend(Ext.form.FieldSet, {

    constructor: function(config) {
        Ext.applyIf(config, {
            hideLabels: true,
            hideBorders: true,
            items: [
                this.getLocalRadioConfig(config.localField),
                this.getAcquiredRadioConfig(),
                this.getHiddenFieldConfig(config.name)
            ]
        });
        Zenoss.form.ZProperty.superclass.constructor.call(this, config);
    },

    setValues: function(values) {
        // values has isAcquired, localValue, acquiredValue, and ancestor
        // localValue is the appropriate type
        // acquiredValue is always a string
        
        // setting the values this away marks the form clean and disables the
        // submit and cancel buttons
        var basicForm = this.refOwner.getForm();
        basicForm.setValues([
            {id: this.localRadio.getName(), value: !values.isAcquired},
            {id: this.localField.getName(), value: values.localValue},
            {id: this.acquiredRadio.getName(), value: values.isAcquired}
        ]);

        // update the boxLabel with the acquiredValue and ancestor
        var boxLabel;
        if ( values.acquiredValue !== null && values.ancestor !== null ) {
            boxLabel = String.format('Inherit Value "{0}" from {1}', values.acquiredValue, values.ancestor);
            this.acquiredRadio.enable();
        } else {
            boxLabel = String.format('Inherit Value');
            this.acquiredRadio.disable();
        }
        this.acquiredRadio.setBoxLabel(boxLabel);
        this.acquiredValue = values.acquiredValue;
        this.ancestor = values.ancestor;
    },

    // private
    getHiddenFieldConfig: function(name) {
        return {
            xtype: 'zprophidden',
            name: name,
            zpropFieldSet: this
        };
    },

    // private
    getAcquiredRadioConfig: function() {
        return {
            xtype: 'zradio',
            ref: 'acquiredRadio',
            boxLabel: 'Inherit Value',
            scope: this,
            handler: function(acquiredRadio, checked) {
                this.localRadio.setValue(!checked);
            }
        };
    },

    //private
    getLocalRadioConfig: function(localField) {
        return {
            xtype: 'panel',
            layout: 'column',
            hideBorders: true,
            defaults: {
                xtype: 'panel',
                layout: 'form',
                hideLabels: true
            },
            items: [{
                width: 130,
                items: [{
                    xtype: 'radio',
                    ref: '../../localRadio',
                    boxLabel: _t('Set Local Value:'),
                    checked: true,
                    scope: this,
                    handler: function(localRadio, checked) {
                        this.acquiredRadio.setValue(!checked);
                    }
                }]
            }, {
                columnWidth: 0.94,
                items: [
                    // Set submitValue to false in case localField has a name  
                    Ext.apply(localField, {
                        ref: '../../localField',
                        submitValue: false,
                        listeners: {
                            scope: this,
                            focus: function() {
                                this.localRadio.setValue(true);
                            }
                        }
                    })
                ]
            }]
        };
    }

});

Ext.reg('zprop', Zenoss.form.ZProperty);


// A simple ComboBox that behaves like an HTML select tag
Zenoss.form.Select = Ext.extend(Ext.form.ComboBox, {
    
    constructor: function(config){
        Ext.applyIf(config, {
            allowBlank: false,
            triggerAction: 'all',
            typeAhead: false,
            forceSelection: true
        });
        Zenoss.form.Select.superclass.constructor.call(this, config);
    }
    
});

Ext.reg('select', Zenoss.form.Select);


})();
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

(function() {

var ZF = Ext.ns('Zenoss.form');

ZF.IDField = Ext.extend(Ext.form.TextField, {
    /* 
    * Context on which to check for id validity. Defaults to
    * Zenoss.env.PARENT_CONTEXT.
    */
    context: null,
    /* 
    * Limit characters to those accepted by ObjectManager 
    */
    maskRe: /[a-zA-Z0-9-_~,.$\(\)# @]/,
    /*
    * Validator function that makes a request to the parent context and calls
    * the checkValidId method.
    */
    validator: function(value) {
        var context = this.context || Zenoss.env.PARENT_CONTEXT;

        // Don't bother with empty values
        if (Ext.isEmpty(value)) {
            return true;
        }

        // if the value has not changed do not send an ajax request
        if (this._previousValue !== undefined) {
            if (value === this._previousValue) {
                return this.reportResponse(this._previousResponseText);
            }
        }
        this._previousValue = value;
        
        if (this.vtransaction) {
            Ext.lib.Ajax.abort(this.vtransaction);
        }
        this.vtransaction = Ext.lib.Ajax.request(
            'GET',
            context + '/checkValidId?id='+value,
            {
                success: function(response) {
                    this._previousResponseText = response.responseText;
                    return this.reportResponse(response.responseText);
                },
                failure: function(response) {
                    this.markInvalid(
                        _t('That name is invalid or is already in use.')
                    );
                },
                scope: this
            }
        );
        return true;
    },
    /**
    * Interprets a response from the server to determine if this field is valid.
    **/
    reportResponse: function(responseText) {
        if (responseText === "True") {
            return true;
        }
        
        // the server responds with a string of why it is invalid 
        this.markInvalid(
            _t('That name is invalid: ') + ' ' + responseText
        );
        return false;
    }
});

Ext.reg('idfield', ZF.IDField);

})();
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

Ext.ns('Zenoss', 'Zenoss.dialog');

/**
 * @class BaseDialog
 * @extends Ext.Window
 * A modal dialog with Zenoss styling. Subclasses should specify a layout.
 * @constructor
 */
var BaseDialog = Ext.extend(Ext.Window, {
    constructor: function(config) {
        Ext.applyIf(config, {
            autoHeight: true,
            width: 310,
            closeAction: 'hide',
            plain: true,
            buttonAlign: 'left',
            padding: 10,
            modal: true
        });
        BaseDialog.superclass.constructor.call(this, config);
    }
});

function destroyWindow(button) {
    var container = button.ownerCt.ownerCt;
    if (container.ownerCt !== undefined) container.ownerCt.destroy();
    else container.destroy();
}

/**
 * @class Zenoss.dialog.HideDialogButton
 * @extends Ext.Button
 * A button that destroys it's window.
 * @constructor
 */
Zenoss.dialog.DialogButton = Ext.extend(Ext.Button, {
    constructor: function(config) {
        var h = config.handler;
        config.handler = h ? h.createSequence(destroyWindow) : destroyWindow;
        Zenoss.dialog.DialogButton.superclass.constructor.call(this, config);
    },
    setHandler: function(handler, scope) {
        var h = handler ? handler.createSequence(destroyWindow) : destroyWindow;
        Zenoss.dialog.DialogButton.superclass.setHandler.call(this, h, scope);
    }
});

Ext.reg('DialogButton', Zenoss.dialog.DialogButton);

function hideWindow(button){
    button.ownerCt.ownerCt.hide();
}

/**
 * @class Zenoss.dialog.HideDialogButton
 * @extends Ext.Button
 * A button that hides it's window.
 * @constructor
 */
Zenoss.dialog.HideDialogButton = Ext.extend(Ext.Button, {
    constructor: function(config) {
        var h = config.handler;
        config.handler = h ? h.createSequence(hideWindow) : hideWindow;
        Zenoss.dialog.HideDialogButton.superclass.constructor.call(this, config);
    },
    setHandler: function(handler, scope) {
        var h = handler ? handler.createSequence(hideWindow) : hideWindow;
        Zenoss.dialog.HideDialogButton.superclass.setHandler.call(this, h, scope);
    }

});

Ext.reg('HideDialogButton', Zenoss.dialog.HideDialogButton);

Zenoss.dialog.CANCEL = {
    xtype: 'DialogButton',
    text: _t('Cancel')
};

/**
 * @class Zenoss.MessageDialog
 * @extends BaseDialog
 * A modal dialog window with Zenoss styling and a fit layout.  This window
 * meant to be instantiated once per page, and hidden each time the user
 * closes it.  Includes an OK and Cancel button.
 * @constructor
 */
Zenoss.MessageDialog = Ext.extend(BaseDialog, {
    constructor: function(config) {
        Ext.applyIf(config, {
            layout: 'fit',
            border: false,
            items: [ {
                xtype: 'label',
                id: 'message',
                text: config.message,
                ref: 'messagelabel'
                } ],
            buttons: [
                {
                    xtype: 'HideDialogButton',
                    text: _t('OK'),
                    handler: config.okHandler
                }, {
                    xtype: 'HideDialogButton',
                    text: _t('Cancel'),
                    handler: config.cancelHandler
                }
            ]
        });
        Zenoss.MessageDialog.superclass.constructor.call(this, config);
    },
    setText: function(text) {
        this.messagelabel.setText(text);
    }
});

/**
 * @class Zenoss.dialog.SimpleMessageDialog
 * @extends BaseDialog
 * A modal dialog window with Zenoss styling and a fit layout. No buttons are
 * included
 * @constructor
 */
Zenoss.dialog.SimpleMessageDialog = Ext.extend(BaseDialog, {
    /**
     * message to be displayed on dialog
     * @param {Object} config
     */
    message: null,
    constructor: function(config) {
        Ext.applyIf(config, {
            layout: 'fit',
            items: {
                border: false,
                html: config.message
            },
            closeAction: 'close'
        });
        Zenoss.MessageDialog.superclass.constructor.call(this, config);
    }
});

/**
 * @class Zenoss.FormDialog
 * @extends Ext.Window
 * A modal dialog window with Zenoss styling and a form layout.  This window
 * meant to be instantiated multiple times per page, and destroyed each time
 * the user closes it.
 * @constructor
 */
Zenoss.FormDialog = Ext.extend(Ext.Window, {
    constructor: function(config) {
        var form = new Ext.form.FormPanel({
            border: false,
            id: config.formId,
            minWidth: 300,
            ref: 'editForm',
            labelAlign: 'top',
            autoScroll: true,
            defaults: {
                xtype: 'textfield',
                anchor: '85%',
                border: false
            },
            items: config.items,
            html: config.html,
            monitorValid: true,
            listeners: config.formListeners || {},
            paramsAsHash: true,
            api: config.formApi || {}
        });

        // Remove config properties that don't pertain to the window
        Ext.destroyMembers(config, 'items', 'formApi', 'formId', 'html');

        Ext.applyIf(config, {
            // ie renders window correctly on when layout is set to form
            // this may change in future ext/ie version
            layout: (Ext.isIE) ? 'form': 'fit',
            plain: true,
            border: false,
            buttonAlign: 'left',
            autoScroll: true,
            width: 375,
            modal: true,
            padding: 10
        });

        Zenoss.FormDialog.superclass.constructor.call(this, config);

        this.add(form);
    },

    getForm: function() {
        return this.editForm;
    }
});


Zenoss.dialog.CloseDialog = new Ext.extend(Ext.Window, {
    constructor: function(config) {
        Ext.applyIf(config, {
            width: 310,
            plain: true,
            buttonAlign: 'left',
            padding: 10,
            modal: true
        });
        Zenoss.dialog.CloseDialog.superclass.constructor.call(this, config);
    }
});


/**
 * @class Zenoss.HideFormDialog
 * @extends BaseDialog
 * A modal dialog window with Zenoss styling and a form layout.  This window
 * meant to be instantiated once per page, and hidden each time the user
 * closes it.
 * @constructor
 */
Zenoss.HideFormDialog = Ext.extend(BaseDialog, {
    constructor: function(config) {
        Ext.applyIf(config, {
            layout: 'form',
            labelAlign: 'top'
        });
        Zenoss.HideFormDialog.superclass.constructor.call(this, config);
    }
});

/**
 * @class Zenoss.SmartFormDialog
 * @extends FormDialog
 * A modal dialog window with Zenoss styling and a form layout.  This window
 * meant to be instantiated once and then thrown away after use.
 *
 * It smartly cleans up it's own form items when "hidden" and provides a better
 * handler mechanism for the callback, returning an object with properties of
 * all form values.
 * @constructor
 */
Zenoss.SmartFormDialog = Ext.extend(Zenoss.FormDialog, {
    message: '',
    submitHandler: null,
    constructor: function(config) {
        var handleEnterKey;

        handleEnterKey = function() {
            var first = this.buttons[0];
            if (first) {
                first.handler(first);
            }
        }.createDelegate(this);

        this.listeners = config.listeners;

        Ext.applyIf(config, {
            buttons: [{
                xtype: 'DialogButton',
                text: _t('Submit'),
                type: 'submit',
                ref: '../buttonSubmit'
             }, {
                xtype: 'DialogButton',
                ref: '../buttonCancel',
                text: _t('Cancel')
            }],
            keys: {
                key: Ext.EventObject.ENTER,
                handler: handleEnterKey,
                scope: this
            },
            modal: true,
            closeAction: 'close'
        });

        Zenoss.SmartFormDialog.superclass.constructor.call(this, config);

        if ( config.message || this.message ) {
            this.insert(0, {
                xtype: 'label',
                ref: 'label',
                text: config.message || this.message
            });
        }
    },
    setSubmitHandler: function(callbackFunction) {
        if (callbackFunction === null) {
            this.buttonSubmit.setHandler(null);
        }
        else {
            this.buttonSubmit.setHandler(function() {
                var values = this.getForm().getForm().getFieldValues();
                return callbackFunction(values);
            }.createDelegate(this));
        }
    },
    initComponent: function() {
        Zenoss.FormDialog.superclass.initComponent.call(this);
        if (this.submitHandler) {
            this.setSubmitHandler(this.submitHandler);
            delete this.submitHandler;
        }
    }
});


/**
 * @class Zenoss.HideFitDialog
 * @extends Ext.Window
 * A modal dialog window with Zenoss styling and a fit layout.
 * @constructor
 */
Zenoss.HideFitDialog = Ext.extend(Ext.Window, {
    constructor: function(config) {
        Ext.applyIf(config, {
            layout: 'fit',
            width: 600,
            height: 300,
            closeAction: 'hide',
            plain: true,
            buttonAlign: 'left',
            padding: 10,
            modal: true
        });
        Zenoss.HideFitDialog.superclass.constructor.call(this, config);
    }
});

/**
 * Works in conjunction with Zenoss.dialog.DialogFormPanel. Loads a formPanel
 * with ID diynamic-dialog-panel and submits the form on the panel when the
 * submit button on this dialog is pressed.
 */
Zenoss.dialog.DynamicDialog = Ext.extend(BaseDialog, {
    initEvents: function(){
        Zenoss.dialog.DynamicDialog.superclass.initEvents.call(this);
        this.body.getUpdater().on('failure', function(el, response) {
            el.update("Failed to load dialog");
        });
    },
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            layout: 'fit',
            modal: true,
            buttons: [{
                xtype: 'DialogButton',
                text: _t('Submit'),
                handler: this.submitHandler.createDelegate(this)
            }, Zenoss.dialog.CANCEL]
        });
        Ext.apply(config, {
            id: 'dynamic-dialog'
        });
        Zenoss.dialog.DynamicDialog.superclass.constructor.call(this, config);
    },
    submitHandler: function(b, event){
        var formPanel = Ext.getCmp('dynamic-dialog-panel');
        var form = formPanel.getForm();
        var params = {};
        if (Ext.isDefined(formPanel.submitName) && formPanel.submitName !== null){
            params[formPanel.submitName] = 'OK';
        }
        form.submit({
            params: params,
            success: function(form, action){
                Zenoss.message.success(_t('{0} finished successfully'), this.title);
            }.createDelegate(this),
            failure: function(form, action){                
                Zenoss.message.error(_t('{0} had errors'), this.title);
            }
        });
    }
});

/**
 * Used to create dialogs that will be added dynamically
 */
Zenoss.dialog.DialogFormPanel = Ext.extend(Ext.form.FormPanel, {
    /**
     * whether or not the result of submitting the form associated with this
     * panel will return a json result. Default true
     */
    jsonResult: true,
    /**
     * extra parameter to send when submitted
     */
    submitName: null,
    /**
     * name of an existing from to be submitted; if not defined a new form
     * will be created.  Primarily used for backwards compatibility so existing
     * dialog forms don't have to be entirely rewritten.
     */
    existingFormId: null,
    constructor: function(config) {
        config = config || {};
        Ext.apply(config, {
            border: false,
            id: 'dynamic-dialog-panel'
        });
        Zenoss.dialog.DialogFormPanel.superclass.constructor.call(this, config);
    },
    /**
     * private; override from base class so that a basic form can be created
     * to point at an existing from if configured and also set a different
     * response reader if expected result from submit is not JSON
     */
    createForm: function(){
        var config = Ext.applyIf({listeners: {}}, this.initialConfig);
        if (!this.jsonResult) {
            config.errorReader = {
                read: function(xhr) {
                    var success = true;
                    //TODO scan result for exceptions/errors
                    if (xhr.status != 200) {
                        success = false;
                    }
                    return {
                        records: [],
                        success: success
                    };
                }
            };
        }
        var formId = null;
        if (this.existingFormId !== null){
            formId = this.existingFormId;
        }
        return new Ext.form.BasicForm(formId, config);
    }
});

Ext.reg('DialogFormPanel', Zenoss.dialog.DialogFormPanel);

})();
/*
  ###########################################################################
  #
  # This program is part of Zenoss Core, an open source monitoring platform.
  # Copyright (C) 2010, Zenoss Inc.
  #
  # This program is free software; you can redistribute it and/or modify it
  # under the terms of the GNU General Public License version 2 as published by
  # the Free Software Foundation.
  #
  # For complete information please visit: http://www.zenoss.com/oss/
  #
  ###########################################################################
*/

/* package level */
(function() {
    Ext.namespace('Zenoss.form');
    /**
     * Config Options:
     * - record (required) - the record we are editing, it is attached to each item
     * - items - received from the Schema declaration
     * - directFn - The server call when we press Save
     * - saveHandler - the router callback from after we save
     **/
    Zenoss.form.DataSourceEditDialog = Ext.extend(Ext.Window, {
        constructor: function(config) {
            config = config || {};
            
            // verify we received the record we are editing
            var record = config.record,
            items = config.items,
            i,
            autoHeight = true,
            that = this; // used in the save handler closure
            this.itemCount = 0;
            // make sure the record was passed in
            if (!record) {
                throw "EditDialog did not recieve a record to edit (config.record is undefined)";
            }
            
            if (!config.singleColumn) {
                items = this.sortItems(items, record);      
            }else{
                // the items come in the form of a single fieldset
                // and we can not have a fieldset because it looks ugly in a dialog
                items = items.items[0].items;
                for(i=0; i<items.length; i+=1) {
                    items[i].record = record;
                    this.itemCount += 1;
                }
            }

            // Since ext has no maxheight property, we need to only use autoheight if we don't
            // have a lot of items. Otherwise the dialog expands off the screen.
            if (this.itemCount > 10) {
                autoHeight = false;
            }
            
            Ext.apply(config, {
                layout: (Ext.isIE) ? 'form': 'fit',
                plain: true,
                border: false,
                buttonAlign: 'left',
                autoScroll: true,
                constrain: true,
                modal: true,
                padding: 10,
                height: 500,
                autoHeight: autoHeight,
                items: [{
                    xtype:'form',
                    border: false,
                    minWidth: 300,
                    ref: 'editForm',
                    labelAlign: 'top',
                    autoScroll:true,
                    monitorValid: true,
                    defaults: {
                        xtype: 'textfield',
                        anchor: '85%',
                        border: false
                    },
                    listeners: {
                        /**
                         * Sets the windows submit button to be disabled when the form is not valid
                         **/
                        clientvalidation: function(formPanel, valid){
                            var dialogWindow;
                            dialogWindow = formPanel.refOwner;
                            // check security first
                            if (Zenoss.Security.hasPermission('Manage DMD')) {
                                dialogWindow.submitButton.setDisabled(!valid);    
                            }
                        }
                    },
                    items: items
                    
                }],
                buttons: [{
                    xtype: 'DialogButton',
                    ref: '../submitButton',
                    formBind: true,
                    disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                    text: _t('Save'),
                    handler: function () {
                        var form = that.editForm.form,
                            dirtyOnly = true,
                            values = form.getFieldValues(dirtyOnly);
                        values.uid = record.uid;
                        config.directFn(values, config.saveHandler);
                    }
                },
                    Zenoss.dialog.CANCEL
                ]
                
            });
            Zenoss.form.DataSourceEditDialog.superclass.constructor.apply(this, arguments);
        },

        /**
         * This function returns a two column layout with the fields auto divided between the
         * column according to the order they are defined.
         * If you specify a group in your datasource schema, they will appear in that "panel" on this dialog
         **/
        sortItems: function(fieldsets, record) {
            var panel = [], i, j, currentPanel,
            item, fieldset, tmp, header, textareas;
            
            // items comes back from the server in the form
            // fieldsets.items[0] = fieldset
            fieldsets = fieldsets.items;
            
            // The datasources have a convention to where the first items are
            // ungrouped. 
            // This section makes sure the non-titled one is first.
            // (It swaps the titled first fieldset with the untitled fieldset)
            if (fieldsets[0].title) {
                tmp = fieldsets[0];
                for (i=0; i < fieldsets.length; i += 1) {
                    if (!fieldsets[i].title) {
                        fieldsets[0] = fieldsets[i];
                        fieldsets[i] = tmp;
                        break;
                    }
                }
            }
            
            // this creates a new panel for each group of items that come back from the server
            for (i =0; i < fieldsets.length; i += 1) {
                fieldset = fieldsets[i];
                
                // format the title a little funny
                if (fieldset.title) {
                    header = {
                        xtype: 'panel',
                        layout: 'form',
                        html: '<br /><br /><h1>' + fieldset.title + '</h1>'
                    };
                }else {
                    header = null;    
                }

                textareas = [];
                
                currentPanel = {
                    xtype:'panel',
                    layout: 'column',
                    border: false,
                    items: [{
                        layout:'form',
                        border:false,
                        columnWidth: 0.5,
                        items: []
                    },{
                        layout:'form',
                        border:false,
                        columnWidth: 0.5,
                        items: []
                    }]
                };

                // alternate which column this item belongs too
                for(j = 0; j < fieldset.items.length; j += 1) {
                    item = fieldset.items[j];
                    this.itemCount += 1;
                    item.record = record;
                    if (!Ext.ComponentMgr.isRegistered(item.xtype)) {
                        throw item.xtype + " is not a valid xtype, please register it.";
                    }
                    // we want to keep text areas to put them in a single column panel
                    if (item.xtype.search(/textarea/) >= 0) {
                        textareas.push(item);   
                    }else{
                        currentPanel.items[j%2].items.push(item);   
                    }
                }
                
                // if we have a header set display it
                if (header) {
                    panel.push(header);
                }
                
                // add the non-textarea fields
                panel.push(currentPanel);

                // put text areas in a single column layout
                if (textareas.length) {
                    for (j=0; j<textareas.length; j += 1) {
                        panel.push({
                            xtype: 'panel',
                            layout: 'form',
                                       
                            border: false,
                            items: textareas[j]
                        });
                    }
                }
                
            }// fieldsets
            
            return panel;
        }
    });
    Ext.reg('datasourceeditdialog', Zenoss.form.DataSourceEditDialog);

}());/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2009, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function () {

Ext.ns('Zenoss');

/**
 * @class Zenoss.HierarchyTreePanel
 * @extends Ext.tree.TreePanel
 * The primary way of navigating one or more hierarchical structures. A
 * more advanced Subsections Tree Panel. Configurable as a drop target.
 * Accepts array containing one or more trees (as nested arrays). In at
 * least one case data needs to be asynchronous. Used on screens:
 *   Device Classification Setup Screen
 *   Devices
 *   Device
 *   Event Classification
 *   Templates
 *   Manufacturers
 *   Processes
 *   Services
 *   Report List
 * @constructor
 */

Zenoss.HierarchyTreeNodeUI = Ext.extend(Ext.tree.TreeNodeUI, {
    renderElements: function(n, a, targetNode, bulkRender) {
        // if children was explicitly set use it, otherwise assume the tree to be
        // asynchronous
        if (Ext.isDefined(a.children)) {
            // Hack this in here because baseAttrs doesn't work on loader
            n.hasChildNodes = function() {
                return (a.children && a.children.length>0);
            }.createDelegate(n);
        }

        Zenoss.HierarchyTreeNodeUI.superclass.renderElements.apply(this, arguments);

        this.textNode = Ext.DomHelper.overwrite(this.textNode, {
            tag: 'span',
            children: [
                { tag: 'span', cls: 'node-text' },
                { tag: 'span', cls: 'node-extra' }
            ]
        }, true);
        this.textNodeBody = this.textNode.child('.node-text');
        this.textNodeExtra = this.textNode.child('.node-extra');

        if ( n.getDepth() === 1 ) {
            this.addClass('hierarchy-root');
        }

        this.onTextChange(this.node, a.text, null);
    },
    onTextChange: function(node, data, oldText) {
        if ( this.rendered ) {
            if ( !Ext.isObject(data) ) {
                data = { text: data, count: null };
            }

            var textOverride = this.node.getDepth() === 1 ? this.node.getOwnerTree().getRootNode().attributes.text : null;
            if ( textOverride ) {
                data.text = textOverride;
            }

            // Just update the existing elements instead of replacing them so that the dd drop targets
            // stay the same
            this.textNodeBody.update(data.text);
            if ( Ext.isDefined(data.count) && data.count !== null ) {
                this.textNodeExtra.update('(' + data.count + ')');
                this.textNodeExtra.show();
            }
            else {
                this.textNodeExtra.update('');
                this.textNodeExtra.hide();
            }
        }
    },
    getDDHandles : function(){
        // include the child span nodes of the text node as drop targets
        var ddHandles = Zenoss.HierarchyTreeNodeUI.superclass.getDDHandles.call(this);
        return ddHandles.concat([this.textNodeBody, this.textNodeExtra]);
    }

});

Zenoss.HierarchyTreePanel = Ext.extend(Ext.tree.TreePanel, {
    constructor: function(config) {
        Ext.applyIf(config, {
            cls: 'hierarchy-panel',
            useArrows: true,
            border: false,
            autoScroll: true,
            containerScroll: true,
            selectRootOnLoad: true,
            rootVisible: false,
            loadMask: true,
            allowOrganizerMove: true
        });

        if ( config.router ) {
            Ext.applyIf(config, {
                addNodeFn: config.router.addNode,
                deleteNodeFn: config.router.deleteNode
            });
        }
        else {
            Ext.applyIf(config, {
                addNodeFn: Ext.emptyFn,
                deleteNodeFn: Ext.emptyFn
            });
        }

        if (config.directFn && !config.loader) {
            config.loader = new Ext.tree.TreeLoader({
                directFn: config.directFn,
                uiProviders: {
                    'hierarchy': Zenoss.HierarchyTreeNodeUI
                },
                getParams: function(node) {
                    return [node.attributes.uid];
                },
                listeners: {
                    beforeload: function(){
                        this.on('afterlayout', function(){
                            this.showLoadMask(true);
                        }, this, {single:true});
                    }.createDelegate(this),
                    load: function(){
                        this.showLoadMask(false);
                    }.createDelegate(this)
                }
            });
            Ext.destroyMembers(config, 'directFn');
        }
        var root = config.root || {};
        Ext.applyIf(root, {
            nodeType: 'async',
            id: root.id,
            uid: root.uid,
            text: _t(root.text || root.id),
            // Use null so the root won't render
            uiProvider: null
        });
        config.loader.baseAttrs = {
            iconCls: 'severity-icon-small clear',
            uiProvider: 'hierarchy'
        };

        Zenoss.HierarchyTreePanel.superclass.constructor.apply(this,
            arguments);
    },
    showLoadMask: function(bool) {
        if (!this.loadMask) { return; } 
        var container = this.container;
        container._treeLoadMask = container._treeLoadMask || new Ext.LoadMask(this.container);
        var mask = container._treeLoadMask,
            _ = bool ? mask.show() : [mask.hide(), mask.disable()];
    },
    initEvents: function() {
        Zenoss.HierarchyTreePanel.superclass.initEvents.call(this);
        if (this.selectRootOnLoad && !Ext.History.getToken()) {
            this.getRootNode().on('expand', function() {
                // The first child is our real root
                if ( this.getRootNode().firstChild ) {
                    this.getRootNode().firstChild.select();
                }
            }, this);
        }
        this.addEvents('filter');
        this.on('click', this.addHistoryToken, this);
    },
    addHistoryToken: function(node) {
        Ext.History.add(this.id + Ext.History.DELIMITER + node.id);
    },
    update: function(data) {
        function doUpdate(root, data) {
            Ext.each(data, function(datum){
                var node = root.findChild('id', datum.id);
                if(node) {
                    node.attributes = datum;
                    node.setText(node.attributes.text);
                    doUpdate(node, datum.children);
                }
            });
        }
        doUpdate(this.getRootNode(), data);

    },
    selectByToken: function(nodeId) {
        nodeId = unescape(nodeId);

        var selNode = function () {
            var sel = this.getSelectionModel().getSelectedNode();
            if ( !(sel && nodeId === sel.id) ) {
                var path = this.getNodePathById(nodeId);

                this.selectPath(path);
            }
        }.createDelegate(this);

        if (!this.root.loaded) {
            this.getRootNode().on('load', selNode, this);
        } else {
            selNode();
        }
    },
    getNodePathById: function(nodeId) {
        var part,
            depth = this.root.attributes.uid.split('/').length - 1,
            parts = nodeId.split('.'),
            curpath = parts.slice(0, depth).join('.');

        parts = parts.slice(depth);

        var path = [this.root.getPath()];
        while ( part = parts.shift() ) {

            // this adjusts the path for things like "Service.Linux.devices.Dev1"
            // where "devices" is the relationshipIdentifier"
            if (this.relationshipIdentifier && part == this.relationshipIdentifier){
                curpath = [curpath, part, parts.shift()].join('.');
            }else{
                curpath = [curpath, part].join('.');
            }


            path.push(curpath);
        }

        return path.join(this.pathSeparator);
    },
    afterRender: function() {
        Zenoss.HierarchyTreePanel.superclass.afterRender.call(this);

        if (this.searchField) {
            this.searchField = this.add({
                xtype: 'searchfield',
                bodyStyle: {padding: 10},
                listeners: {
                    valid: this.filterTree,
                    scope: this
                }
            });
        }
        this.getRootNode().expand(false, true, function(node) {
            node.expandChildNodes();
        });
    },
    filterTree: function(e) {
        var text = e.getValue();
        this.fireEvent('filter', e);
        if (this.hiddenPkgs) {
            Ext.each(this.hiddenPkgs, function(n){n.ui.show();});
        }
        this.hiddenPkgs = [];
        if (!text) {
            return;
        }
        this.expandAll();
        var re = new RegExp(Ext.escapeRe(text), 'i');
        this.root.cascade(function(n){
            var attr = n.id.slice('.zport.dmd'.length);
            if (!n.isRoot) {
                if (re.test(attr)) {
                    var parentNode = n.parentNode;
                    while (parentNode) {
                        if (!parentNode.hidden) {
                            break;
                        }
                        parentNode.ui.show();
                        parentNode = parentNode.parentNode;
                    }
                    // the cascade is stopped on this branch
                    return false;
                } else {
                    n.ui.hide();
                    this.hiddenPkgs.push(n);
                }
            }
            // continue cascading down the tree from this node
            return true;
        }, this);
    },

    addNode: function(type, id) {
        this.addChildNode({type: type, id: id});
    },

    addChildNode: function(params) {
        var selectedNode = this.getSelectionModel().getSelectedNode();
        var parentNode;
        if (selectedNode.leaf) {
            parentNode = selectedNode.parentNode;
        } else {
            parentNode = selectedNode;
        }
        var contextUid = parentNode.attributes.uid;
        Ext.applyIf(params, {
            contextUid: contextUid
        });

        this.addTreeNode(params);
    },

    addTreeNode: function(params) {
        var callback = function (provider, response) {
            var result = response.result;
            if (result.success) {
                // look for another node on result and assume it's the new node, grab it's id
                // TODO would be best to normalize the names of result node
                var nodeId;
                Ext.iterate(result, function(key, value) {
                    if ( key != 'success' && Ext.isObject(value) && value.id ) {
                        nodeId = value.id;
                        return false;
                    }
                }, this);

                this.getRootNode().reload(function() {
                    if ( nodeId ) {
                        this.selectByToken(nodeId);
                    }
                }, this);
            }
            else {
                Ext.Msg.alert('Error', result.msg);
            }
        };

        this.addNodeFn(params, callback.createDelegate(this));
    },

    deleteSelectedNode: function() {
        var node = this.getSelectionModel().getSelectedNode();
        var parentNode = node.parentNode;
        var uid = node.attributes.uid;
        var params = {uid: uid};
        function callback(provider, response) {
            parentNode.select();
            parentNode.removeChild(node);
            node.destroy();
        }
        this.deleteNodeFn(params, callback);
    },

    canMoveOrganizer: function(organizerUid, targetUid) {
        var orgPieces = organizerUid.split('/'),
            targetPieces = targetUid.split('/');

        // make sure we can actually move organizers
        if (!this.allowOrganizerMove) {
            return false;
        }

        // Relying on a coincidence that the third item
        // is the top level organizer (e.g. Locations, Groups)
        return orgPieces[3] === targetPieces[3];
    }
}); // HierarchyTreePanel

Ext.reg('HierarchyTreePanel', Zenoss.HierarchyTreePanel);

})();
Ext.ns('Zenoss');

/**
 * @class Zenoss.SearchField
 * @extends Ext.form.TextField
 * @constructor
 */
Zenoss.SearchField = Ext.extend(Ext.form.TextField, {
    constructor: function(config){
        if (!('selectOnFocus' in config))
            config.selectOnFocus = true;
        Zenoss.SearchField.superclass.constructor.apply(this, arguments);
    },
    getClass: function(){
        var cls = this.altCls ? this.altCls : 'searchfield';
        return this.black ? cls + '-black' : cls;
    },
    onRender: function() {
        Zenoss.SearchField.superclass.onRender.apply(this, arguments);
        this.wrap = this.el.boxWrap(this.getClass());
        if (this.bodyStyle) {
            this.wrap.setStyle(this.bodyStyle);
        }
        this.resizeEl = this.positionEl = this.wrap;
        this.syncSize();
    },
    syncSize: function(){
        this.el.setBox(this.el.parent().getBox());
    }

}); // Ext.extend

Ext.reg('searchfield', Zenoss.SearchField);
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

/* package level */
(function() {

Ext.ns('Zenoss.form');

Zenoss.form.LinkField = Ext.extend(Ext.form.DisplayField, {
     getValue: function() {
        return this.rawValue;
     },
     setValue: function(value) {
        this.rawValue = value;
        if (Ext.isEmpty(value)) {
            value = _t('None');
        } else {
            if (Ext.isArray(value)){
                var items = [];
                Ext.each(value, function(v){
                    items.push(Zenoss.render.link(v));
                });
                value = items.join('<br/>');
            } else {
                value = Zenoss.render.link(value);
            }
        }
        Zenoss.form.LinkField.superclass.setValue.call(this, value);
     }
 });
 Ext.reg('linkfield', Zenoss.form.LinkField);

})();
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

(function() {

var ZF = Ext.ns('Zenoss.form');

Ext.form.TextArea.prototype.grow = true;
Ext.form.TextArea.prototype.growMin = 20;

function isField(c) {
    return !!c.setValue && !!c.getValue && !!c.markInvalid && !!c.clearInvalid;
}

ZF.createDirectSubmitFunction = function(router) {

    // called in the Ext.form.Action.DirectSubmit.run method
    return function(formElDom, successFunction, directSubmitAction) {

        var form = directSubmitAction.form;

        // copy field values to the info object for fields that
        //   * are dirty (param passed to getFieldValues above)
        //   * have submitValue true
        //   * have an id or name set on them
        var info = {};
        var dirtyFieldValues = form.getFieldValues(true);
        Ext.iterate(dirtyFieldValues, function(key, value) {
            var field = form.findField(key);
            if ( field.submitValue !== false && field.getName().indexOf("ext-comp-") !== 0 ) {
                info[key] = value;
            }
        });

        // add the forms baseParams to the info object (often includes uid)
        Ext.applyIf(info, directSubmitAction.getParams());

        // define a callback to run after server responds
        var callback = function() {
            form.clearInvalid();
            form.setValues(dirtyFieldValues); // isDirty() will return false now
            form.afterAction(directSubmitAction, true);
            form.reset();
        };

        // the remote call
        router.setInfo(info, callback, directSubmitAction);

    };
};

ZF.BaseDetailForm = Ext.extend(Ext.form.FormPanel, {
    contextUid: null,
    isLoadInProgress: false,
    constructor: function(config){
        // Router doesn't technically matter, since they all do getInfo, but
        // getForm is definitely defined on DeviceRouter
        var router = config.router || Zenoss.remote.DeviceRouter;
        config.baseParams = Ext.applyIf(config.baseParams||{
            uid: config.contextUid
        });
        config.listeners = Ext.applyIf(config.listeners||{}, {
            'add': function(me, field, index){
                if (isField(field)) {
                    this.onFieldAdd.call(this, field);
                }
            }
        });
        config = Ext.applyIf(config||{}, {
            paramsAsHash: true,
            autoScroll: 'y',
            cls: 'detail-form-panel',
            buttonAlign: 'left',
            labelAlign: 'top',
            trackResetOnLoad: true,
            permission: 'Manage Device',
            api: {
                submit: ZF.createDirectSubmitFunction(router),
                load: router.getInfo
            }
        });
        var hasPermission = function() {
            var perm = !Zenoss.Security.doesNotHavePermission(config.permission);
            if (Ext.isDefined(config.userCreated)) {
                return config.userCreated && perm;
            } else {
                return perm;
            }
        };
        if (hasPermission()) {
            Ext.apply(config, {
                buttons:  [{
                    xtype: 'button',
                    ref: '../savebtn',
                    text: _t('Save'),
                    disabled: true,
                    cls: 'detailform-submit-button',
                    handler: function(btn, e) {
                        this.refOwner.getForm().submit();
                    }
                },{
                    xtype: 'button',
                    ref: '../cancelbtn',
                    disabled: true,
                    text: _t('Cancel'),
                    cls: 'detailform-cancel-button',
                    handler: function(btn, e) {
                        this.refOwner.getForm().reset();
                    }
                }]
            });
        } 
        ZF.BaseDetailForm.superclass.constructor.call(this, config);
    },
    hasPermission: function() {
        var perm = !Zenoss.Security.doesNotHavePermission(this.permission);
        if (Ext.isDefined(this.userCreated)) {
            return this.userCreated && perm;
        } else {
            return perm;
        }
    },
    setButtonsDisabled: function(b) {
        if (this.hasPermission()) {
            this.savebtn.setDisabled(b);
            this.cancelbtn.setDisabled(b);
        }
    },
    doButtons: function() {
        this.setButtonsDisabled(!this.form.isDirty());
    },
    onFieldAdd: function(field) {
        if (!field.isXType('displayfield')) {
            this.mon(field, 'valid', this.doButtons, this);
        }
    },
    getFieldNames: function() {
        var keys = [];
        for (var k in this.getForm().getFieldValues(false)) {
            if (keys.indexOf(k)==-1) {
                keys.push(k);
            }
        }
        return keys;
    },
    load: function() {
        var o = Ext.apply({keys:this.getFieldNames()}, this.baseParams);
        this.form.load(o, function(result) {
            this.form.setValues(result.data);
            this.form.reset();
            this.doLayout();
        }, this);
    },
    setContext: function(uid) {
        this.contextUid = uid;
        this.baseParams.uid = uid;
        this.isLoadInProgress = true;
        this.load();
    }
});

Ext.reg('basedetailform', ZF.BaseDetailForm);

})();
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

/* package level */
(function() {

var ZF = Ext.ns('Zenoss.form');

ZF.AutoFormPanel = Ext.extend(ZF.BaseDetailForm, {});

Ext.reg('autoformpanel', ZF.AutoFormPanel);

/*
* Zenoss.form.getGeneratedForm 
* Accepts a uid and a callback.
* Asks the router for a form for the object represented by uid.
* Returns a config object that can be added to a container to render the form.
*/
ZF.getGeneratedForm = function(uid, callback, router) {
    // Router doesn't technically matter, since they all do getInfo, but
    // getForm is definitely defined on DeviceRouter
    router = router || Zenoss.remote.DeviceRouter;
    router.getForm({uid:uid}, function(response){
        callback(Ext.apply({
            xtype:'autoformpanel',
            contextUid: uid,
            border: false,
            layout: 'column',
            defaults: {
                layout: 'form',
                bodyStyle: 'padding:10px',
                labelAlign: 'top',
                columnWidth: 0.5
            }
        }, response.form));
    });
}

ZF.AutoFormCombo = Ext.extend(Ext.form.ComboBox, {
     constructor: function(config) {
         config = Ext.applyIf(config||{}, {
             editable: false,
             forceSelection: true,
             autoSelect: true,
             triggerAction: 'all',
             mode: 'local',
             store: config.values || []
         });
         Zenoss.form.AutoFormCombo.superclass.constructor.call(this, config);
     }

});
Ext.reg('autoformcombo', ZF.AutoFormCombo);

})();
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */
(function(){

Ext.ns('Zenoss.events');

/**
 * @class Zenoss.EventPanelSelectionModel
 * @extends Zenoss.ExtraHooksSelectionModel
 *
 */
Zenoss.EventPanelSelectionModel = Ext.extend(Zenoss.ExtraHooksSelectionModel, {
    selectState: null,
    badIds: {},
    initEvents: function(){
        Zenoss.EventPanelSelectionModel.superclass.initEvents.call(this);
        this.on('beforerowselect', this.handleBeforeRowSelect, this);
        this.on('rowselect', this.handleRowSelect, this);
        this.on('rowdeselect', this.handleRowDeSelect, this);

    },
    handleBeforeRowSelect: function(sm, index, keepExisting, record){
        if (!keepExisting) {
            this.selectNone();
        }
        return true;
    },
    handleRowSelect: function(sm, index, record){
        if (record) {
            delete this.badIds[record.id];
        }
    },
    handleRowDeSelect: function(sm, index, record){
        if (this.selectState && record) {
            this.badIds[record.id] = 1;
        }
    },
    selectEventState: function(state){
        var record,
            start = this.grid.store.bufferRange[0],
            end = this.grid.store.bufferRange[1];

        this.clearSelections(true);

        for (var i = start; i <= end; i++) {
            record = this.grid.store.getAt(i);
            if (state === 'All' || record.data.eventState == state) {
                this.selectRow(i, true);
            }
        }

        this.selectState = state;
    },
    selectAll: function(){

        this.clearSelections();
        this.selectEventState('All');
    },
    selectNone: function(){
        this.clearSelections();
    },
    selectAck: function(){
        this.clearSelections();
        this.selectEventState('Acknowledged');
    },
    selectNew: function(){
        this.clearSelections();
        this.selectEventState('New');
    },
    selectSuppressed: function(){
        this.clearSelections();
        this.selectEventState('Suppressed');
    },
    /**
     * Override handle mouse down method from "Ext.grid.RowSelectionModel"
     * to handle shift select more intelligently.
     * We need to disallow shift select when the selection range crosses
     * a buffer to prevent a user from taking action upon an event they they
     * may not have seen yet. See trac #6959
     **/
    handleMouseDown: function(g, rowIndex, e){
        if(e.button !== 0 || this.isLocked()){
            return;
        }
        var view = this.grid.getView();
        // handle shift select
        if(e.shiftKey && !this.singleSelect && this.last !== false){
            // last is the index of the previous row they selected
            var last = this.last;

            // bufferRange is the first and last item in our current view
            var startIndex = this.grid.store.bufferRange[0];
            var endIndex = this.grid.store.bufferRange[1];

            // only allow shift select if the range is in our current view
            if (last >= startIndex && last <= endIndex){
                this.selectRange(last, rowIndex, e.ctrlKey);
                this.last = last; // reset the last
                view.focusRow(rowIndex);
            }else{
                // unselect everything (in case they shift select, then jump around buffers and shift select again)
                this.clearSelections();
                this.doSingleSelect(rowIndex, e);
            }
        }else{
            this.doSingleSelect(rowIndex, e);
        }
    },
    /**
     * Used by handleMouseDown to handle a single selection
     **/
    doSingleSelect: function(rowIndex, e){
        var view = this.grid.getView();
        var isSelected = this.isSelected(rowIndex);
        if(e.ctrlKey && isSelected){
            this.deselectRow(rowIndex);
        }else if(!isSelected || this.getCount() > 1){
            this.selectRow(rowIndex, e.ctrlKey || e.shiftKey);
            view.focusRow(rowIndex);
        }
    },
    clearSelections: function(fast){
        var start, end, record;
        if (this.isLocked()) {
            return;
        }
        this.selectState = null;
        if(!fast){
            //make sure all rows are deselected so that UI renders properly
            //base class only deselects rows it knows are selected; so we need
            //to deselect rows that may have been selected via selectstate
            start = this.grid.store.bufferRange[0];
            end = this.grid.store.bufferRange[1];
            for (var i = start; i <= end; i++) {
                record = this.grid.store.getAt(i);
                this.deselectRow(i);
            }
        }
        this.badIds = {};
        Zenoss.EventPanelSelectionModel.superclass.clearSelections.apply(this, arguments);
    },
    onRefresh: function(){
        //override from base class to prevent reslect after sorting
        var ds = this.grid.store, index;
        var s = this.getSelections();
        this.clearSelections(false);
        if (s.length != this.selections.getCount()) {
            this.fireEvent('selectionchange', this);
        }
    },
    isSelected: function(index){
        var r = Ext.isNumber(index) ? this.grid.store.getAt(index) : index;
        var selected = (r && this.selections.key(r.id) ? true : false);
        var badId = false;
        if (r && this.badIds[r.id]) {
            selected = false;
        }
        else if (this.selectState == 'All') {
                selected = true;
        }
        else if (r && !selected && this.selectState) {
            if (this.selectState == 'Acknowledged') {
                selected = r.data.eventState == 'Acknowledged';
            }
            else if (this.selectState == 'Suppressed') {
                selected = r.data.eventState == 'Suppressed';
            }
            else if (this.selectState == 'New') {
                selected = r.data.eventState == 'New';
            }
        }
        return selected;

    }
});
// the column model for the device grid
Zenoss.EventStore = Ext.extend(Ext.ux.grid.livegrid.Store, {
    constructor: function(config){
        Ext.applyIf(config, {
            proxy: new Ext.data.DirectProxy({
                directFn: Zenoss.remote.EventsRouter.query
            }),
            bufferSize: 100,
        defaultSort: {field:'severity', direction:'DESC'},
        sortInfo: {field:'severity', direction:'DESC'},
            reader: new Ext.ux.grid.livegrid.JsonReader({
                root: 'events',
                totalProperty: 'totalCount'
                }, [
                // List all possible columns.
                // FIXME: This should come from the server.
                    'dedupid',
                    'evid',
                    'device',
                    'device_title',
                    'device_url',
                    'component',
                    'component_url',
                    'component_title',
                    'summary',
                    'eventState',
                    'eventClass',
                    'eventClass_url',
                    'eventKey',
                    'message',
                    'eventClassKey',
                    'eventGroup',
                    'prodState',
                    'suppid',
                    'manager',
                    'agent',
                    'DeviceClass',
                    'Location',
                    'Systems',
                    'DeviceGroups',
                    'ipAddress',
                    'facility',
                    'priority',
                    'ntevid',
                    'ownerid',
                    'clearid',
                    'DevicePriority',
                    'eventClassMapping',
                    'monitor',
                    'iprealm',
                    {name:'count', type:'int'},
                    {name:'severity', type:'int'},
                    {name:'firstTime', type:'date',
                        dateFormat:Zenoss.date.ISO8601Long},
                    {name:'lastTime', type:'date',
                        dateFormat:Zenoss.date.ISO8601Long},
                    {name:'stateChange', type:'date',
                        dateFormat:Zenoss.date.ISO8601Long}
                ]) // reader
        }); // Ext.applyIf
        Zenoss.EventStore.superclass.constructor.call(this, config);
    } // constructor
}); // Ext.extend
Ext.reg('EventStore', Zenoss.EventStore);


Zenoss.SimpleEventColumnModel = Ext.extend(Ext.grid.ColumnModel, {
    constructor: function(config){
        config = Ext.applyIf(config || {}, {
            defaults: {
                sortable: false,
                menuDisabled: true,
                width: 200
            },
            columns: [{
                dataIndex: 'severity',
                header: _t('Severity'),
                width: 60,
                id: 'severity',
                renderer: Zenoss.util.render_severity
            }, {
                dataIndex: 'device',
                header: _t('Device'),
                renderer: Zenoss.render.linkFromGrid
            }, {
                dataIndex: 'component',
                header: _t('Component'),
                renderer: Zenoss.render.linkFromGrid
            }, {
                dataIndex: 'eventClass',
                header: _t('Event Class'),
                renderer: Zenoss.render.linkFromGrid
            }, {
                dataIndex: 'summary',
                header: _t('Summary'),
                id: 'summary'
            }] // columns
        }); // Ext.applyIf
    Zenoss.SimpleEventColumnModel.superclass.constructor.call(
        this, config);
    } // constructor
}); // Ext.extend
Ext.reg('SimpleEventColumnModel', Zenoss.SimpleEventColumnModel);


Zenoss.FullEventColumnModel = Ext.extend(Ext.grid.ColumnModel, {
    constructor: function(config){
        // Zenoss.env.COLUMN_DEFINITIONS comes from the server, and depends on
        // the resultFields associated with the context.
        // FIXME: This shouldn't come from the server.
        config = Ext.applyIf(config || {}, {
            columns:Zenoss.env.COLUMN_DEFINITIONS
        });
        Zenoss.FullEventColumnModel.superclass.constructor.call(this, config);
    }
});
Ext.reg('FullEventColumnModel', Zenoss.FullEventColumnModel);


/**
 * @class Zenoss.SimpleEventGridPanel
 * @extends Ext.ux.grid.livegrid.GridPanel
 * Shows events in a grid panel similar to that on the event console.
 * Fixed columns.
 * @constructor
 */
Zenoss.SimpleEventGridPanel = Ext.extend(Ext.ux.grid.livegrid.GridPanel, {
    constructor: function(config){
        var store = {xtype:'EventStore'},
            cmConfig = {};
        if (Ext.isDefined(config.columns)) {
            cmConfig.columns = config.columns;
        }
        var cm = new Zenoss.SimpleEventColumnModel(cmConfig);
        if (!Ext.isEmpty(config.directFn)) {
            Ext.apply(store, {
                proxy: new Ext.data.DirectProxy({
                    directFn: config.directFn
                })
            });
        }
        var id = config.id || Ext.id();
        Ext.applyIf(config, {
            id: 'eventGrid' + id,
            stripeRows: true,
            stateId: Zenoss.env.EVENTSGRID_STATEID || 'default_eventsgrid',
            enableDragDrop: false,
            stateful: true,
            border: false,
            rowSelectorDepth: 5,
            store: store,
            cm: cm,
            sm: new Zenoss.EventPanelSelectionModel(),
            autoExpandColumn: Zenoss.env.EVENT_AUTO_EXPAND_COLUMN || '',
            view: new Ext.ux.grid.livegrid.GridView({
                nearLimit: 20,
                rowHeight: 10,
                emptyText: _t('No events'),
                loadMask: {msg: 'Loading. Please wait...'},
                listeners: {
                    beforeBuffer: function(view, ds, idx, len, total, opts){
                        opts.params.uid = view._context;
                    }
                },
                getRowClass: function(record, index) {
                    var stateclass = record.get('eventState')=='New' ?
                                        'unacknowledged':'acknowledged';
                    var sev = Zenoss.util.convertSeverity(record.get('severity'));
                    var rowcolors = Ext.state.Manager.get('rowcolor') ? 'rowcolor rowcolor-' : '';
                    var cls = rowcolors + sev + '-' + stateclass + ' ' + stateclass;
                    return cls;
                }
            })
        }); // Ext.applyIf
        Zenoss.SimpleEventGridPanel.superclass.constructor.call(this, config);
        this.on('rowdblclick', this.onRowDblClick, this);
    }, // constructor
    setContext: function(uid){
        this.view._context = uid;
        this.view.updateLiveRows(this.view.rowIndex, true, true);
    },
    onRowDblClick: function(grid, rowIndex, e) {
        var row = grid.getStore().getAt(rowIndex),
            evid = row.id,
             url = '/zport/dmd/Events/viewDetail?evid='+evid;
         window.open(url, evid.replace(/-/g,'_'),
             "status=1,width=600,height=500");
    }
}); // SimpleEventGridPanel
Ext.reg('SimpleEventGridPanel', Zenoss.SimpleEventGridPanel);

/**
 * Select Menu
 **/
Zenoss.events.SelectMenu = {
    text: _t('Select'),
    id: 'select-button',
    menu:{
        xtype: 'menu',
        items: [{
            text: 'All',
            handler: function(){
                var grid = Ext.getCmp('select-button').ownerCt.ownerCt,
                sm = grid.getSelectionModel();
                sm.selectAll();
            }
        },{
            text: 'None',
            handler: function(){
                var grid = Ext.getCmp('select-button').ownerCt.ownerCt,
                sm = grid.getSelectionModel();
                sm.selectNone();
            }
        },{
            text: 'New',
            iconCls: 'unacknowledge',
            handler: function(){
                // New == 0
                var grid = Ext.getCmp('select-button').ownerCt.ownerCt,
                sm = grid.getSelectionModel();
                sm.selectNew();
            }
        },{
            text: 'Acknowledged',
            iconCls: 'acknowledge',
            handler: function(){
                // Acknowledged == 1
                var grid = Ext.getCmp('select-button').ownerCt.ownerCt,
                sm = grid.getSelectionModel();
                sm.selectAck();
            }
        },{
            text: 'Suppressed',
            iconCls: 'suppress',
            handler: function(){
                // Suppressed == 2
                var grid = Ext.getCmp('select-button').ownerCt.ownerCt,
                sm = grid.getSelectionModel();
                sm.selectSuppressed();
            }
        }
               ]
    }
};

Zenoss.EventGridPanel = Ext.extend(Zenoss.SimpleEventGridPanel, {
    constructor: function(config) {
        var evtGrid = this,
            tbarItems = [
            {
                xtype: 'tbtext',
                text: _t('Event Console')
            },
                '-',
                Zenoss.events.SelectMenu,
               '-',

            {
                xtype: 'tbtext',
                text: _t('Display: ')
            },{
                xtype: 'combo',
                id: 'history_combo',
                name: 'event_display',
                mode: 'local',
                store: new Ext.data.SimpleStore({
                    fields: ['id', 'event_type'],
                    data: [[0,'Events'],[1,'Event History']]
                }),
                displayField: 'event_type',
                valueField: 'id',
                width: 120,
                value: 0,
                triggerAction: 'all',
                forceSelection: true,
                editable: false,
                listeners: {
                    select: function(selection) {
                        var getHistory = selection.value == 1,
                            params = {
                                uid: evtGrid.view._context,
                                history: getHistory
                            },
                            yesterday;
                        if (getHistory) {
                            yesterday = new Date();
                            yesterday.setDate(yesterday.getDate() - 1);
                            params['params'] = { lastTime: yesterday };
                        }
                        evtGrid.getStore().load({ params: params });
                        Zenoss.events.EventPanelToolbarActions.acknowledge.setHidden(getHistory);
                        Zenoss.events.EventPanelToolbarActions.close.setHidden(getHistory);
                    }
                }
            },
            '-',
            Zenoss.events.EventPanelToolbarActions.acknowledge,
            Zenoss.events.EventPanelToolbarActions.close,
            Zenoss.events.EventPanelToolbarActions.refresh
        ];
        if (config.newwindowBtn) {
            tbarItems.push('-');
            tbarItems.push(Zenoss.events.EventPanelToolbarActions.newwindow);
        }
        Ext.applyIf(config, {
            tbar: {
                xtype: 'largetoolbar',
                cls: 'largetoolbar consolebar',
                height: 35,
                items: tbarItems
            }
        });
        Zenoss.EventGridPanel.superclass.constructor.call(this, config);
    },
    onRowDblClick: function(grid, rowIndex, e) {
        var row = grid.getStore().getAt(rowIndex),
            evid = row.id,
            combo = Ext.getCmp('history_combo'),
            history = (combo.getValue() == '1') ? 'History' : '',
            url = '/zport/dmd/Events/view'+history+'Detail?evid='+evid;
        window.open(url, evid.replace(/-/g,'_'),
            "status=1,width=600,height=500");
    }
});
Ext.reg('EventGridPanel', Zenoss.EventGridPanel);


Zenoss.EventRainbow = Ext.extend(Ext.Toolbar.TextItem, {
    constructor: function(config) {
        var severityCounts = {
            critical: 0,
            error: 0,
            warning: 0,
            info: 0,
            debug: 0,
            clear: 0
        }
        config = Ext.applyIf(config || {}, {
            height: 45,
            directFn: Zenoss.remote.DeviceRouter.getInfo,
            text: Zenoss.render.events(severityCounts, config.count || 3)
        });
        Zenoss.EventRainbow.superclass.constructor.call(this, config);
    },
    setContext: function(uid){
        this.directFn({uid:uid}, function(result){
            this.updateRainbow(result.data.events);
        }, this);
    },
    updateRainbow: function(severityCounts) {
        this.setText(Zenoss.render.events(severityCounts, this.count));
    },
    onRender: function() {
        Zenoss.EventRainbow.superclass.onRender.apply(this, arguments);
        this.el.on("click", function(){
            window.location = "/zport/dmd/Events/evconsole?filter=default";
        });
    }
});

Ext.reg('eventrainbow', Zenoss.EventRainbow);

Zenoss.events.EventPanelToolbarActions = {
    acknowledge: new Zenoss.Action({
        //text: _t('Acknowledge'),
        iconCls: 'acknowledge',
        tooltip: _t('Acknowledge events'),
        permission: 'Manage Events',
        handler: function(btn) {
            var grid = btn.grid || this.ownerCt.ownerCt,
                sm = grid.getSelectionModel(),
                selected = sm.getSelections(),
                evids = Ext.pluck(selected, 'id');
            Zenoss.remote.EventsRouter.acknowledge(
                {evids:evids},
                function(provider, response){
                    var view = grid.getView();
                    view.updateLiveRows(view.rowIndex, true, true);
                }
            );
        }
    }),
    close: new Zenoss.Action({
        //text: _t('Close'),
        iconCls: 'close',
        permission: 'Manage Events',
        tooltip: _t('Close events'),
        handler: function(btn) {
            var grid = btn.grid || this.ownerCt.ownerCt,
                sm = grid.getSelectionModel(),
                selected = sm.getSelections(),
                evids = Ext.pluck(selected, 'id');
            Zenoss.remote.EventsRouter.close(
                {evids:evids},
                function(provider, response){
                    var view = grid.getView();
                    view.updateLiveRows(view.rowIndex, true, true);
                }
            );
        }
    }),
    newwindow: new Zenoss.Action({
        //text: _t('Open in new window'),
        iconCls: 'newwindow',
        tooltip: _t('Go to event console'),
        permission: 'View',
        handler: function(btn) {
            var grid = btn.grid || this.ownerCt.ownerCt,
                curState = Ext.state.Manager.get('evconsole') || {},
                filters = curState.filters || {},
                opts = filters.options || {},
                pat = /devices\/([^\/]+)(\/.*\/([^\/]+)$)?/,
                matches = grid.view._context.match(pat),
                st, url;
            // on the device page
            if (matches) {
                opts.device = matches[1];
                if (matches[3]) {
                    opts.component = matches[3];
                }
            }
            filters.options = opts;
            curState.filters = filters;
            st = Zenoss.util.base64.encode(Ext.encode(curState));
            url = '/zport/dmd/Events/evconsole?state=' + st;
            window.open(url, '_newtab', "");
        }
    }),
    refresh: new Zenoss.Action({
        //text: _t('refresh'),
        iconCls: 'refresh',
        permission: 'View',
        tooltip: _t('Refresh events'),
        handler: function(btn) {
            var grid = btn.grid || this.ownerCt.ownerCt,
                view = grid.getView();
            view.updateLiveRows(view.rowIndex, true, true);
        }
    })
};


})(); // end of function namespace scoping
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2009, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

Ext.ns('Zenoss');


Zenoss.DeviceColumnModel = Ext.extend(Ext.grid.ColumnModel, {
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            columns: [{
                dataIndex: 'name',
                header: _t('Device'),
                id: 'name',
                renderer: function(name, row, record) {
                    return Zenoss.render.Device(record.data.uid, name);
                }
            },{
                id: 'ipAddress',
                width: 100,
                dataIndex: 'ipAddress',
                header: _t('IP Address'),
                renderer: Zenoss.render.ipAddress
            },{
                dataIndex: 'uid',
                header: _t('Device Class'), 
                id: 'deviceClass',
                width: 120,
                renderer: Zenoss.render.DeviceClass
            },{
                id: 'productionState',
                dataIndex: 'productionState',
                width: 100,
                filter: {
                    xtype: 'multiselect-prodstate'
                },
                header: _t('Production State')
            },{
                id: 'events',
                sortable: false,
                filter: false,
                dataIndex: 'events',
                header: _t('Events'),
                renderer: function(ev, ignored, record) {
                    var table = Zenoss.render.events(ev),
                        url = record.data.uid + '/devicedetail?filter=default#deviceDetailNav:device_events';
                    table = table.replace('table', 'table onclick="location.href=\''+url+'\';"');
                    return table;
                }
            }] // columns
        }); // Ext.applyIf
        config.defaults = Ext.applyIf(config.defaults || {}, {
            sortable: false,
            menuDisabled: true,
            width: 200
        });
        Zenoss.DeviceColumnModel.superclass.constructor.call(this, config);
    } // constructor
});
Ext.reg('DeviceColumnModel', Zenoss.DeviceColumnModel);


/**
 * Device data store definition
 * @constructor
 */
Zenoss.DeviceStore = Ext.extend(Ext.ux.grid.livegrid.Store, {

    constructor: function(config) {
        config = config || {};
        Ext.applyIf(config, {
            autoLoad: true,
            bufferSize: 50,
            defaultSort: {field: 'name', direction:'ASC'},
            sortInfo: {field: 'name', direction:'ASC'},
            proxy: new Ext.data.DirectProxy({
                directFn: Zenoss.remote.DeviceRouter.getDevices
            }),
            reader: new Ext.ux.grid.livegrid.JsonReader({
                root: 'devices',
                totalProperty: 'totalCount',
                idProperty: 'uid'
            },[
                  {name: 'uid', type: 'string'},
                  {name: 'name', type: 'string'},
                  {name: 'ipAddress', type: 'int'},
                  {name: 'productionState', type: 'string'},
                  {name: 'events', type: 'auto'},
                  {name: 'availability', type: 'float'}
              ])
        });
        Zenoss.DeviceStore.superclass.constructor.call(this, config);
    },
    loadRanges: function(ranges) {
        // We actually just want to send the ranges themselves, so we'll
        // short-circuit this so it doesn't try to turn them into uids in yet
        // another server request
    }
});

Ext.reg('DeviceStore', Zenoss.DeviceStore);


Zenoss.SimpleDeviceGridPanel = Ext.extend(Ext.ux.grid.livegrid.GridPanel, {
    constructor: function(config) {
        var store = {xtype:'DeviceStore'};
        if (!Ext.isEmpty(config.directFn)) {
            Ext.apply(store, {
                proxy: new Ext.data.DirectProxy({
                    directFn: config.directFn
                })
            });
        }
        config = Ext.applyIf(config || {}, {
            cm: new Zenoss.DeviceColumnModel({
                menuDisabled: true
            }),
            sm: new Zenoss.ExtraHooksSelectionModel(),
            store: store,
            enableDragDrop: false,
            border:false,
            rowSelectorDepth: 5,
            autoExpandColumn: 'name',
            stripeRows: true
        });
        Zenoss.SimpleDeviceGridPanel.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        this.getStore().load({params:{uid:uid}});
    }
});
Ext.reg('SimpleDeviceGridPanel', Zenoss.SimpleDeviceGridPanel);


Zenoss.DeviceGridPanel = Ext.extend(Zenoss.FilterGridPanel,{
    lastHash: null,
    constructor: function(config) {
        var store = { xtype:'DeviceStore' };
        if (!Ext.isEmpty(config.directFn)) {
            Ext.apply(store, {
                proxy: new Ext.data.DirectProxy({
                    directFn: config.directFn
                })
            });
        }
        Ext.applyIf(config, {
            store: store,
            enableDragDrop: false,
            border: false,
            rowSelectorDepth: 5,
            view: new Zenoss.FilterGridView({
                rowHeight: 24,
                nearLimit: 10,
                loadMask: {msg: 'Loading. Please wait...'}
            }),
            autoExpandColumn: 'name',
            cm: new Zenoss.DeviceColumnModel({defaults:{sortable:true}}),
            sm: new Zenoss.ExtraHooksSelectionModel(),
            stripeRows: true
        });
        Zenoss.DeviceGridPanel.superclass.constructor.call(this, config);
        this.store.proxy.on('beforeload', function(){
            this.view._loadMaskAnchor = Ext.get('center_panel_container');
            Ext.apply(this.view.loadMask,{
                msgCls : 'x-mask-loading'
            });
            this.view._loadMaskAnchor.mask(this.view.loadMask.msg, this.view.loadMask.msgCls);
            this.view.showLoadMask(true);
        }, this, {single:true});
        this.store.proxy.on('load', function(){
            this.view.showLoadMask(false);
            this.view._loadMaskAnchor = Ext.get(this.view.mainBody.dom.parentNode.parentNode);
        }, this, {single:true});
        this.store.proxy.on('load', 
            function(proxy, o, options) {
                this.lastHash = o.result.hash || this.lastHash;
            },
            this);
        this.on('rowdblclick', this.onRowDblClick, this);
    }, // constructor
    onRowDblClick: function(grid, rowIndex, e) {
        window.location = grid.getStore().getAt(rowIndex).data.uid;
    }
});
Ext.reg('DeviceGridPanel', Zenoss.DeviceGridPanel);


})(); // end of function namespace scoping
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

Ext.ns('Zenoss');

var InstanceColumnModel = Ext.extend(Ext.grid.ColumnModel, {

    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            defaults: {
                menuDisabled: true
            },
            columns: [{
                    id: 'device',
                    dataIndex: 'device',
                    header: _t('Device'),
                    width: 200,
                    renderer: function(device, row, record){
                        return Zenoss.render.link(device.uid, undefined,
                                                  device.name);
                   }
                }, {
                    id: 'name',
                    dataIndex: config.nameDataIndex || 'name',
                    header: _t('Name'),
                    width: 400
                }, {
                    id: 'monitored',
                    dataIndex: 'monitored',
                    header: _t('Monitored'),
                    width: 70,
                    sortable: true
                }, {
                    id: 'status',
                    dataIndex: 'status',
                    header: _t('Status'),
                    renderer: Zenoss.render.pingStatus,
                    width: 60
                }
            ]
        });
        InstanceColumnModel.superclass.constructor.call(this, config);
    }

});

/**
 * Device data store definition
 * @constructor
 */
Zenoss.InstanceStore = Ext.extend(Ext.ux.grid.livegrid.Store, {

    constructor: function(config) {
        Ext.applyIf(config, {
            bufferSize: 50,
            proxy: new Ext.data.DirectProxy({
                directFn: config.directFn
            }),
            reader: new Ext.ux.grid.livegrid.JsonReader({
                root: 'data',
                idProperty: 'uid',
                totalProperty: 'totalCount',
                fields: [
                    {name: 'device'},
                    {name: config.nameDataIndex || 'name'},
                    {name: 'monitored'},
                    {name: 'status'}
                ]
            })
        });
        Zenoss.InstanceStore.superclass.constructor.call(this, config);
    }

});

Ext.reg('InstanceStore', Zenoss.InstanceStore);

Zenoss.SimpleInstanceGridPanel = Ext.extend(Ext.ux.grid.livegrid.GridPanel, {

    constructor: function(config) {
        Ext.applyIf(config, {
            autoExpandColumn: 'name',
            stripeRows: true,
            cm: config.cm || new InstanceColumnModel({
                nameDataIndex: config.nameDataIndex
            }),
            store: config.store || {
                xtype: 'InstanceStore',
                directFn: config.directFn,
                nameDataIndex: config.nameDataIndex
            },
            sm: config.sm || new Ext.ux.grid.livegrid.RowSelectionModel(),
            view: new Ext.ux.grid.livegrid.GridView({
                nearLimit: 20,
                loadMask: {msg: _t('Loading...'),
                          msgCls: 'x-mask-loading'}
            })
        });
        Zenoss.SimpleInstanceGridPanel.superclass.constructor.call(this, config);
        Zenoss.util.addLoadingMaskToGrid(this);
    },

    setContext: function(uid) {
        this.getStore().load({params:{uid:uid}});
    }

});

Ext.reg('SimpleInstanceGridPanel', Zenoss.SimpleInstanceGridPanel);


// supply the instances implementation in the config
Zenoss.SimpleCardPanel = Ext.extend(Ext.Panel, {

    constructor: function(config) {
        this.contextUid = null;
        Ext.applyIf(config, {
            layout: 'card',
            activeItem: 0,
            collapsed: true,
            height: Math.min(((Ext.getCmp('viewport').getHeight() - 75)/5)+30, 200),
            tbar: {
                xtype: 'consolebar',
                title: 'Display: ',
                leftItems: [{
                    xtype: 'select',
                    ref: '../displaySelect',
                    mode: 'local',
                    value: config.instancesTitle,
                    store: [config.instancesTitle, 'Configuration Properties'],
                    listeners: {
                        select: function(displaySelect, record, index) {
                            displaySelect.refOwner.getLayout().setActiveItem(index);
                        }
                    }
                }]
            },
            listeners:{
                scope: this,
                expand: function(panel){
                    this.loadInstances();
                }
            },
            items: [
                config.instances,
            {
                xtype: 'backcompat',
                ref: 'zPropertyEdit',
                viewName: 'zPropertyEdit',
                refreshOnContextChange: true,
                listeners: config.zPropertyEditListeners
            }]
        });
        Zenoss.SimpleCardPanel.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        // only reload our datastores if we are not collapsed
        this.contextUid = uid;
        if (!this.collapsed){
            this.loadInstances(uid);
        }
    },
    loadInstances: function(){
        if (!this.contextUid){
            return;
        }
        var contextUid = this.contextUid;
        // if we have not set our store since we last updated the
        // context uid update it now
        this.items.each(function(item) {
            item.setContext(contextUid);
        });
        this.contextUid = null;
    }
});

Ext.reg('simplecardpanel', Zenoss.SimpleCardPanel);

// has the instances used by the Processes page and the two Services pages
Zenoss.InstanceCardPanel = Ext.extend(Zenoss.SimpleCardPanel, {

    constructor: function(config) {
        Ext.applyIf(config, {
            instances: [{
                xtype: 'SimpleInstanceGridPanel',
                ref: 'instancesGrid',
                directFn: config.router.getInstances,
                nameDataIndex: config.nameDataIndex || "name",
                cm: config.cm,
                sm: config.sm,
                store: config.store
            }]
        });
        Zenoss.InstanceCardPanel.superclass.constructor.call(this, config);
    }

});

Ext.reg('instancecardpanel', Zenoss.InstanceCardPanel);


})();
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

Ext.ns('Zenoss');

/**
 * @class Zenoss.ViewButton
 * @extends Ext.Button
 * A button that toggles between cards in a panel with a card layout.
 * @constructor
 */
Zenoss.ViewButton = Ext.extend(Ext.Button, {

    constructor: function(userConfig) {

        var baseConfig = {
            enableToggle: true,
            toggleGroup: 'CardButtonPanel',
            allowDepress: false
        };

        var config = Ext.apply(baseConfig, userConfig);
        Zenoss.ViewButton.superclass.constructor.call(this, config);
    }

});

Ext.reg('ViewButton', Zenoss.ViewButton);

/**
 * @class Zenoss.CardButtonPanel
 * @extends Ext.Button
 * A Panel with a card layout and toolbar buttons for switching between the
 * cards.
 * @constructor
 */
Zenoss.CardButtonPanel = Ext.extend(Ext.Panel, {

    constructor: function(config) {
        // Inner secret closure function to create the handler
        function createToggleHandler(cardPanel, panel) {
            return function(button, pressed) {
                if (pressed) {
                    cardPanel.fireEvent('cardchange', panel);
                    cardPanel.getLayout().setActiveItem(panel.id);
                }
            };
        }

        function syncButtons(me) {
            var tb = me.getTopToolbar();
            for (var idx=0; idx < me.items.getCount(); ++idx) {
                var newComponent = me.items.get(idx);

                if (newComponent instanceof Ext.Panel) {
                    tb.add({
                        xtype: 'ViewButton',
                        id: 'button_' + newComponent.id,
                        text: Ext.clean(newComponent.buttonTitle,
                                        newComponent.title, 'Undefined'),
                        pressed: (newComponent == me.layout.activeItem),
                        iconCls: newComponent.iconCls,
                        toggleHandler: createToggleHandler(me, newComponent)
                    });
                }
            }
        }

        function addButtons(me, newComponent, index) {
        }

        Ext.applyIf(config, {
            id: 'cardPanel',
            layout: 'card',
            activeItem: 0
        });

        Ext.apply(config, {
            header: false,
            tbar: [{
                xtype: 'tbtext',
                text: _t('View: ')
            }]
        });

        this.addEvents('cardchange');
        this.on('afterrender', syncButtons, this);
        this.listeners = config.listeners;
        Zenoss.CardButtonPanel.superclass.constructor.call(this, config);
    }
});

Ext.reg('CardButtonPanel', Zenoss.CardButtonPanel);

})();
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

Ext.ns('Zenoss');

/**
 * @class Zenoss.ContextCardButtonPanel
 * @extends Zenoss.CardButtonPanel
 * Support context-driven loading
 * @constructor
 */
Zenoss.ContextCardButtonPanel = Ext.extend(Zenoss.CardButtonPanel, {
    contextUid: null,
    initEvents: function() {
        this.on('cardchange', this.cardChangeHandler, this);
        Zenoss.CardButtonPanel.superclass.initEvents.call(this);
    },
    setContext: function(uid) {
        var panel;
        if (this.contextUid!=uid) {
            this.contextUid = uid;
            panel = this.layout.activeItem;
            if (panel.setContext) {
                panel.setContext(uid);
            }
        }
    },
    cardChangeHandler: function(panel) {
        if (panel.setContext) {
            panel.setContext(this.contextUid);
        }
    }
});

Ext.reg('ContextCardButtonPanel', Zenoss.ContextCardButtonPanel);

var oldActiveItem = Ext.layout.CardLayout.prototype.setActiveItem;
var oldInitEvents = Ext.layout.CardLayout.prototype.initEvents;

Ext.override(Ext.layout.CardLayout, {
    initEvents: function() {
        oldInitEvents.apply(this, arguments);
        this.container.addEvents('cardchange');
    },
    setActiveItem: function(item) {
        oldActiveItem.apply(this, arguments);
        this.container.fireEvent('cardchange', this.container, item);
    }
});

Zenoss.ContextCardPanel = Ext.extend(Ext.Panel, {
    contextUid: null,
    constructor: function(config) {
        Ext.applyIf(config, {
            layout: 'card'
        });
        Zenoss.ContextCardPanel.superclass.constructor.call(this, config);
        this.on('cardchange', this.cardChangeHandler, this);
    },
    setContext: function(uid) {
        this.contextUid = uid;
        this.cardChangeHandler(this.layout.activeItem);
    },
    cardChangeHandler: function(panel) {
        if (panel!=this && panel.setContext) {
            panel.setContext(this.contextUid);
        }
    }

});
Ext.reg('contextcardpanel', Zenoss.ContextCardPanel);

})();


/*!
 * Ext JS Library 3.1.1
 * Copyright(c) 2006-2010 Ext JS, LLC
 * licensing@extjs.com
 * http://www.extjs.com/license
 */
Ext.ns('Ext.ux.form');

/**
 * @class Ext.ux.form.MultiSelect
 * @extends Ext.form.Field
 * A control that allows selection and form submission of multiple list items.
 *
 *  @history
 *    2008-06-19 bpm Original code contributed by Toby Stuart (with contributions from Robert Williams)
 *    2008-06-19 bpm Docs and demo code clean up
 *
 * @constructor
 * Create a new MultiSelect
 * @param {Object} config Configuration options
 * @xtype multiselect
 */
Ext.ux.form.MultiSelect = Ext.extend(Ext.form.Field,  {
    /**
     * @cfg {String} legend Wraps the object with a fieldset and specified legend.
     */
    /**
     * @cfg {Ext.ListView} view The {@link Ext.ListView} used to render the multiselect list.
     */
    /**
     * @cfg {String/Array} dragGroup The ddgroup name(s) for the MultiSelect DragZone (defaults to undefined).
     */
    /**
     * @cfg {String/Array} dropGroup The ddgroup name(s) for the MultiSelect DropZone (defaults to undefined).
     */
    /**
     * @cfg {Boolean} ddReorder Whether the items in the MultiSelect list are drag/drop reorderable (defaults to false).
     */
    ddReorder:false,
    /**
     * @cfg {Object/Array} tbar The top toolbar of the control. This can be a {@link Ext.Toolbar} object, a
     * toolbar config, or an array of buttons/button configs to be added to the toolbar.
     */
    /**
     * @cfg {String} appendOnly True if the list should only allow append drops when drag/drop is enabled
     * (use for lists which are sorted, defaults to false).
     */
    appendOnly:false,
    /**
     * @cfg {Number} width Width in pixels of the control (defaults to 100).
     */
    width:100,
    /**
     * @cfg {Number} height Height in pixels of the control (defaults to 100).
     */
    height:100,
    /**
     * @cfg {String/Number} displayField Name/Index of the desired display field in the dataset (defaults to 0).
     */
    displayField:0,
    /**
     * @cfg {String/Number} valueField Name/Index of the desired value field in the dataset (defaults to 1).
     */
    valueField:1,
    /**
     * @cfg {Boolean} allowBlank False to require at least one item in the list to be selected, true to allow no
     * selection (defaults to true).
     */
    allowBlank:true,
    /**
     * @cfg {Number} minSelections Minimum number of selections allowed (defaults to 0).
     */
    minSelections:0,
    /**
     * @cfg {Number} maxSelections Maximum number of selections allowed (defaults to Number.MAX_VALUE).
     */
    maxSelections:Number.MAX_VALUE,
    /**
     * @cfg {String} blankText Default text displayed when the control contains no items (defaults to the same value as
     * {@link Ext.form.TextField#blankText}.
     */
    blankText:Ext.form.TextField.prototype.blankText,
    /**
     * @cfg {String} minSelectionsText Validation message displayed when {@link #minSelections} is not met (defaults to 'Minimum {0}
     * item(s) required').  The {0} token will be replaced by the value of {@link #minSelections}.
     */
    minSelectionsText:'Minimum {0} item(s) required',
    /**
     * @cfg {String} maxSelectionsText Validation message displayed when {@link #maxSelections} is not met (defaults to 'Maximum {0}
     * item(s) allowed').  The {0} token will be replaced by the value of {@link #maxSelections}.
     */
    maxSelectionsText:'Maximum {0} item(s) allowed',
    /**
     * @cfg {String} delimiter The string used to delimit between items when set or returned as a string of values
     * (defaults to ',').
     */
    delimiter:',',
    /**
     * @cfg {Ext.data.Store/Array} store The data source to which this MultiSelect is bound (defaults to <tt>undefined</tt>).
     * Acceptable values for this property are:
     * <div class="mdetail-params"><ul>
     * <li><b>any {@link Ext.data.Store Store} subclass</b></li>
     * <li><b>an Array</b> : Arrays will be converted to a {@link Ext.data.ArrayStore} internally.
     * <div class="mdetail-params"><ul>
     * <li><b>1-dimensional array</b> : (e.g., <tt>['Foo','Bar']</tt>)<div class="sub-desc">
     * A 1-dimensional array will automatically be expanded (each array item will be the combo
     * {@link #valueField value} and {@link #displayField text})</div></li>
     * <li><b>2-dimensional array</b> : (e.g., <tt>[['f','Foo'],['b','Bar']]</tt>)<div class="sub-desc">
     * For a multi-dimensional array, the value in index 0 of each item will be assumed to be the combo
     * {@link #valueField value}, while the value at index 1 is assumed to be the combo {@link #displayField text}.
     * </div></li></ul></div></li></ul></div>
     */

    // private
    defaultAutoCreate : {tag: "div"},

    // private
    initComponent: function(){
        Ext.ux.form.MultiSelect.superclass.initComponent.call(this);

        if(Ext.isArray(this.store)){
            if (Ext.isArray(this.store[0])){
                this.store = new Ext.data.ArrayStore({
                    fields: ['value','text'],
                    data: this.store
                });
                this.valueField = 'value';
            }else{
                this.store = new Ext.data.ArrayStore({
                    fields: ['text'],
                    data: this.store,
                    expandData: true
                });
                this.valueField = 'text';
            }
            this.displayField = 'text';
        } else {
            this.store = Ext.StoreMgr.lookup(this.store);
        }

        this.addEvents({
            'dblclick' : true,
            'click' : true,
            'change' : true,
            'drop' : true
        });
    },

    // private
    onRender: function(ct, position){
        Ext.ux.form.MultiSelect.superclass.onRender.call(this, ct, position);

        var fs = this.fs = new Ext.form.FieldSet({
            renderTo: this.el,
            title: this.legend,
            height: this.height,
            width: this.width,
            style: "padding:0;",
            tbar: this.tbar
        });
        fs.body.addClass('ux-mselect');

        this.view = new Ext.ListView({
            multiSelect: true,
            store: this.store,
            columns: [{ header: 'Value', width: 1, dataIndex: this.displayField }],
            hideHeaders: true
        });

        fs.add(this.view);

        this.view.on('click', this.onViewClick, this);
        this.view.on('beforeclick', this.onViewBeforeClick, this);
        this.view.on('dblclick', this.onViewDblClick, this);

        this.hiddenName = this.name || Ext.id();
        var hiddenTag = { tag: "input", type: "hidden", value: "", name: this.hiddenName };
        this.hiddenField = this.el.createChild(hiddenTag);
        this.hiddenField.dom.disabled = this.hiddenName != this.name;
        fs.doLayout();
    },

    // private
    afterRender: function(){
        Ext.ux.form.MultiSelect.superclass.afterRender.call(this);

        if (this.ddReorder && !this.dragGroup && !this.dropGroup){
            this.dragGroup = this.dropGroup = 'MultiselectDD-' + Ext.id();
        }

        if (this.draggable || this.dragGroup){
            this.dragZone = new Ext.ux.form.MultiSelect.DragZone(this, {
                ddGroup: this.dragGroup
            });
        }
        if (this.droppable || this.dropGroup){
            this.dropZone = new Ext.ux.form.MultiSelect.DropZone(this, {
                ddGroup: this.dropGroup
            });
        }
    },

    // private
    onViewClick: function(vw, index, node, e) {
        this.fireEvent('change', this, this.getValue(), this.hiddenField.dom.value);
        this.hiddenField.dom.value = this.getValue();
        this.fireEvent('click', this, e);
        this.validate();
    },

    // private
    onViewBeforeClick: function(vw, index, node, e) {
        if (this.disabled) {return false;}
    },

    // private
    onViewDblClick : function(vw, index, node, e) {
        return this.fireEvent('dblclick', vw, index, node, e);
    },

    /**
     * Returns an array of data values for the selected items in the list. The values will be separated
     * by {@link #delimiter}.
     * @return {Array} value An array of string data values
     */
    getValue: function(valueField){
        var returnArray = [];
        var selectionsArray = this.view.getSelectedIndexes();
        if (selectionsArray.length == 0) {return '';}
        for (var i=0; i<selectionsArray.length; i++) {
            returnArray.push(this.store.getAt(selectionsArray[i]).get((valueField != null) ? valueField : this.valueField));
        }
        return returnArray.join(this.delimiter);
    },

    /**
     * Sets a delimited string (using {@link #delimiter}) or array of data values into the list.
     * @param {String/Array} values The values to set
     */
    setValue: function(values) {
        var index;
        var selections = [];
        this.view.clearSelections();
        this.hiddenField.dom.value = '';

        if (!values || (values == '')) { return; }

        if (!Ext.isArray(values)) { values = values.split(this.delimiter); }
        for (var i=0; i<values.length; i++) {
            index = this.view.store.indexOf(this.view.store.query(this.valueField,
                new RegExp('^' + values[i] + '$', "i")).itemAt(0));
            selections.push(index);
        }
        this.view.select(selections);
        this.hiddenField.dom.value = this.getValue();
        this.validate();
    },

    // inherit docs
    reset : function() {
        this.setValue('');
    },

    // inherit docs
    getRawValue: function(valueField) {
        var tmp = this.getValue(valueField);
        if (tmp.length) {
            tmp = tmp.split(this.delimiter);
        }
        else {
            tmp = [];
        }
        return tmp;
    },

    // inherit docs
    setRawValue: function(values){
        setValue(values);
    },

    // inherit docs
    validateValue : function(value){
        if (value.length < 1) { // if it has no value
             if (this.allowBlank) {
                 this.clearInvalid();
                 return true;
             } else {
                 this.markInvalid(this.blankText);
                 return false;
             }
        }
        if (value.length < this.minSelections) {
            this.markInvalid(String.format(this.minSelectionsText, this.minSelections));
            return false;
        }
        if (value.length > this.maxSelections) {
            this.markInvalid(String.format(this.maxSelectionsText, this.maxSelections));
            return false;
        }
        return true;
    },

    // inherit docs
    disable: function(){
        this.disabled = true;
        this.hiddenField.dom.disabled = true;
        this.fs.disable();
    },

    // inherit docs
    enable: function(){
        this.disabled = false;
        this.hiddenField.dom.disabled = false;
        this.fs.enable();
    },

    // inherit docs
    destroy: function(){
        Ext.destroy(this.fs, this.dragZone, this.dropZone);
        Ext.ux.form.MultiSelect.superclass.destroy.call(this);
    }
});


Ext.reg('multiselect', Ext.ux.form.MultiSelect);

//backwards compat
Ext.ux.Multiselect = Ext.ux.form.MultiSelect;


Ext.ux.form.MultiSelect.DragZone = function(ms, config){
    this.ms = ms;
    this.view = ms.view;
    var ddGroup = config.ddGroup || 'MultiselectDD';
    var dd;
    if (Ext.isArray(ddGroup)){
        dd = ddGroup.shift();
    } else {
        dd = ddGroup;
        ddGroup = null;
    }
    Ext.ux.form.MultiSelect.DragZone.superclass.constructor.call(this, this.ms.fs.body, { containerScroll: true, ddGroup: dd });
    this.setDraggable(ddGroup);
};

Ext.extend(Ext.ux.form.MultiSelect.DragZone, Ext.dd.DragZone, {
    onInitDrag : function(x, y){
        var el = Ext.get(this.dragData.ddel.cloneNode(true));
        this.proxy.update(el.dom);
        el.setWidth(el.child('em').getWidth());
        this.onStartDrag(x, y);
        return true;
    },

    // private
    collectSelection: function(data) {
        data.repairXY = Ext.fly(this.view.getSelectedNodes()[0]).getXY();
        var i = 0;
        this.view.store.each(function(rec){
            if (this.view.isSelected(i)) {
                var n = this.view.getNode(i);
                var dragNode = n.cloneNode(true);
                dragNode.id = Ext.id();
                data.ddel.appendChild(dragNode);
                data.records.push(this.view.store.getAt(i));
                data.viewNodes.push(n);
            }
            i++;
        }, this);
    },

    // override
    onEndDrag: function(data, e) {
        var d = Ext.get(this.dragData.ddel);
        if (d && d.hasClass("multi-proxy")) {
            d.remove();
        }
    },

    // override
    getDragData: function(e){
        var target = this.view.findItemFromChild(e.getTarget());
        if(target) {
            if (!this.view.isSelected(target) && !e.ctrlKey && !e.shiftKey) {
                this.view.select(target);
                this.ms.setValue(this.ms.getValue());
            }
            if (this.view.getSelectionCount() == 0 || e.ctrlKey || e.shiftKey) return false;
            var dragData = {
                sourceView: this.view,
                viewNodes: [],
                records: []
            };
            if (this.view.getSelectionCount() == 1) {
                var i = this.view.getSelectedIndexes()[0];
                var n = this.view.getNode(i);
                dragData.viewNodes.push(dragData.ddel = n);
                dragData.records.push(this.view.store.getAt(i));
                dragData.repairXY = Ext.fly(n).getXY();
            } else {
                dragData.ddel = document.createElement('div');
                dragData.ddel.className = 'multi-proxy';
                this.collectSelection(dragData);
            }
            return dragData;
        }
        return false;
    },

    // override the default repairXY.
    getRepairXY : function(e){
        return this.dragData.repairXY;
    },

    // private
    setDraggable: function(ddGroup){
        if (!ddGroup) return;
        if (Ext.isArray(ddGroup)) {
            Ext.each(ddGroup, this.setDraggable, this);
            return;
        }
        this.addToGroup(ddGroup);
    }
});

Ext.ux.form.MultiSelect.DropZone = function(ms, config){
    this.ms = ms;
    this.view = ms.view;
    var ddGroup = config.ddGroup || 'MultiselectDD';
    var dd;
    if (Ext.isArray(ddGroup)){
        dd = ddGroup.shift();
    } else {
        dd = ddGroup;
        ddGroup = null;
    }
    Ext.ux.form.MultiSelect.DropZone.superclass.constructor.call(this, this.ms.fs.body, { containerScroll: true, ddGroup: dd });
    this.setDroppable(ddGroup);
};

Ext.extend(Ext.ux.form.MultiSelect.DropZone, Ext.dd.DropZone, {
    /**
     * Part of the Ext.dd.DropZone interface. If no target node is found, the
     * whole Element becomes the target, and this causes the drop gesture to append.
     */
    getTargetFromEvent : function(e) {
        var target = e.getTarget();
        return target;
    },

    // private
    getDropPoint : function(e, n, dd){
        if (n == this.ms.fs.body.dom) { return "below"; }
        var t = Ext.lib.Dom.getY(n), b = t + n.offsetHeight;
        var c = t + (b - t) / 2;
        var y = Ext.lib.Event.getPageY(e);
        if(y <= c) {
            return "above";
        }else{
            return "below";
        }
    },

    // private
    isValidDropPoint: function(pt, n, data) {
        if (!data.viewNodes || (data.viewNodes.length != 1)) {
            return true;
        }
        var d = data.viewNodes[0];
        if (d == n) {
            return false;
        }
        if ((pt == "below") && (n.nextSibling == d)) {
            return false;
        }
        if ((pt == "above") && (n.previousSibling == d)) {
            return false;
        }
        return true;
    },

    // override
    onNodeEnter : function(n, dd, e, data){
        return false;
    },

    // override
    onNodeOver : function(n, dd, e, data){
        var dragElClass = this.dropNotAllowed;
        var pt = this.getDropPoint(e, n, dd);
        if (this.isValidDropPoint(pt, n, data)) {
            if (this.ms.appendOnly) {
                return "x-tree-drop-ok-below";
            }

            // set the insert point style on the target node
            if (pt) {
                var targetElClass;
                if (pt == "above"){
                    dragElClass = n.previousSibling ? "x-tree-drop-ok-between" : "x-tree-drop-ok-above";
                    targetElClass = "x-view-drag-insert-above";
                } else {
                    dragElClass = n.nextSibling ? "x-tree-drop-ok-between" : "x-tree-drop-ok-below";
                    targetElClass = "x-view-drag-insert-below";
                }
                if (this.lastInsertClass != targetElClass){
                    Ext.fly(n).replaceClass(this.lastInsertClass, targetElClass);
                    this.lastInsertClass = targetElClass;
                }
            }
        }
        return dragElClass;
    },

    // private
    onNodeOut : function(n, dd, e, data){
        this.removeDropIndicators(n);
    },

    // private
    onNodeDrop : function(n, dd, e, data){
        if (this.ms.fireEvent("drop", this, n, dd, e, data) === false) {
            return false;
        }
        var pt = this.getDropPoint(e, n, dd);
        if (n != this.ms.fs.body.dom)
            n = this.view.findItemFromChild(n);

        if(this.ms.appendOnly) {
            insertAt = this.view.store.getCount();
        } else {
            insertAt = n == this.ms.fs.body.dom ? this.view.store.getCount() - 1 : this.view.indexOf(n);
            if (pt == "below") {
                insertAt++;
            }
        }

        var dir = false;

        // Validate if dragging within the same MultiSelect
        if (data.sourceView == this.view) {
            // If the first element to be inserted below is the target node, remove it
            if (pt == "below") {
                if (data.viewNodes[0] == n) {
                    data.viewNodes.shift();
                }
            } else {  // If the last element to be inserted above is the target node, remove it
                if (data.viewNodes[data.viewNodes.length - 1] == n) {
                    data.viewNodes.pop();
                }
            }

            // Nothing to drop...
            if (!data.viewNodes.length) {
                return false;
            }

            // If we are moving DOWN, then because a store.remove() takes place first,
            // the insertAt must be decremented.
            if (insertAt > this.view.store.indexOf(data.records[0])) {
                dir = 'down';
                insertAt--;
            }
        }

        for (var i = 0; i < data.records.length; i++) {
            var r = data.records[i];
            if (data.sourceView) {
                data.sourceView.store.remove(r);
            }
            this.view.store.insert(dir == 'down' ? insertAt : insertAt++, r);
            var si = this.view.store.sortInfo;
            if(si){
                this.view.store.sort(si.field, si.direction);
            }
        }
        return true;
    },

    // private
    removeDropIndicators : function(n){
        if(n){
            Ext.fly(n).removeClass([
                "x-view-drag-insert-above",
                "x-view-drag-insert-left",
                "x-view-drag-insert-right",
                "x-view-drag-insert-below"]);
            this.lastInsertClass = "_noclass";
        }
    },

    // private
    setDroppable: function(ddGroup){
        if (!ddGroup) return;
        if (Ext.isArray(ddGroup)) {
            Ext.each(ddGroup, this.setDroppable, this);
            return;
        }
        this.addToGroup(ddGroup);
    }
});



/*!
 * Ext JS Library 3.1.1
 * Copyright(c) 2006-2010 Ext JS, LLC
 * licensing@extjs.com
 * http://www.extjs.com/license
 */
/*
 * Note that this control will most likely remain as an example, and not as a core Ext form
 * control.  However, the API will be changing in a future release and so should not yet be
 * treated as a final, stable API at this time.
 */

/**
 * @class Ext.ux.form.ItemSelector
 * @extends Ext.form.Field
 * A control that allows selection of between two Ext.ux.form.MultiSelect controls.
 *
 *  @history
 *    2008-06-19 bpm Original code contributed by Toby Stuart (with contributions from Robert Williams)
 *
 * @constructor
 * Create a new ItemSelector
 * @param {Object} config Configuration options
 * @xtype itemselector 
 */
Ext.ux.form.ItemSelector = Ext.extend(Ext.form.Field,  {
    hideNavIcons:false,
    imagePath:"",
    iconUp:"up2.gif",
    iconDown:"down2.gif",
    iconLeft:"left2.gif",
    iconRight:"right2.gif",
    iconTop:"top2.gif",
    iconBottom:"bottom2.gif",
    drawUpIcon:true,
    drawDownIcon:true,
    drawLeftIcon:true,
    drawRightIcon:true,
    drawTopIcon:true,
    drawBotIcon:true,
    delimiter:',',
    bodyStyle:null,
    border:false,
    defaultAutoCreate:{tag: "div"},
    /**
     * @cfg {Array} multiselects An array of {@link Ext.ux.form.MultiSelect} config objects, with at least all required parameters (e.g., store)
     */
    multiselects:null,

    initComponent: function(){
        Ext.ux.form.ItemSelector.superclass.initComponent.call(this);
        this.addEvents({
            'rowdblclick' : true,
            'change' : true
        });
    },

    onRender: function(ct, position){
        Ext.ux.form.ItemSelector.superclass.onRender.call(this, ct, position);

        // Internal default configuration for both multiselects
        var msConfig = [{
            legend: 'Available',
            draggable: true,
            droppable: true,
            width: 100,
            height: 100
        },{
            legend: 'Selected',
            droppable: true,
            draggable: true,
            width: 100,
            height: 100
        }];

        this.fromMultiselect = new Ext.ux.form.MultiSelect(Ext.applyIf(this.multiselects[0], msConfig[0]));
        this.fromMultiselect.on('dblclick', this.onRowDblClick, this);

        this.toMultiselect = new Ext.ux.form.MultiSelect(Ext.applyIf(this.multiselects[1], msConfig[1]));
        this.toMultiselect.on('dblclick', this.onRowDblClick, this);

        var p = new Ext.Panel({
            bodyStyle:this.bodyStyle,
            border:this.border,
            layout:"table",
            layoutConfig:{columns:3}
        });

        p.add(this.fromMultiselect);
        var icons = new Ext.Panel({header:false});
        p.add(icons);
        p.add(this.toMultiselect);
        p.render(this.el);
        icons.el.down('.'+icons.bwrapCls).remove();

        // ICON HELL!!!
        if (this.imagePath!="" && this.imagePath.charAt(this.imagePath.length-1)!="/")
            this.imagePath+="/";
        this.iconUp = this.imagePath + (this.iconUp || 'up2.gif');
        this.iconDown = this.imagePath + (this.iconDown || 'down2.gif');
        this.iconLeft = this.imagePath + (this.iconLeft || 'left2.gif');
        this.iconRight = this.imagePath + (this.iconRight || 'right2.gif');
        this.iconTop = this.imagePath + (this.iconTop || 'top2.gif');
        this.iconBottom = this.imagePath + (this.iconBottom || 'bottom2.gif');
        var el=icons.getEl();
        this.toTopIcon = el.createChild({tag:'img', src:this.iconTop, style:{cursor:'pointer', margin:'2px'}});
        el.createChild({tag: 'br'});
        this.upIcon = el.createChild({tag:'img', src:this.iconUp, style:{cursor:'pointer', margin:'2px'}});
        el.createChild({tag: 'br'});
        this.addIcon = el.createChild({tag:'img', src:this.iconRight, style:{cursor:'pointer', margin:'2px'}});
        el.createChild({tag: 'br'});
        this.removeIcon = el.createChild({tag:'img', src:this.iconLeft, style:{cursor:'pointer', margin:'2px'}});
        el.createChild({tag: 'br'});
        this.downIcon = el.createChild({tag:'img', src:this.iconDown, style:{cursor:'pointer', margin:'2px'}});
        el.createChild({tag: 'br'});
        this.toBottomIcon = el.createChild({tag:'img', src:this.iconBottom, style:{cursor:'pointer', margin:'2px'}});
        this.toTopIcon.on('click', this.toTop, this);
        this.upIcon.on('click', this.up, this);
        this.downIcon.on('click', this.down, this);
        this.toBottomIcon.on('click', this.toBottom, this);
        this.addIcon.on('click', this.fromTo, this);
        this.removeIcon.on('click', this.toFrom, this);
        if (!this.drawUpIcon || this.hideNavIcons) { this.upIcon.dom.style.display='none'; }
        if (!this.drawDownIcon || this.hideNavIcons) { this.downIcon.dom.style.display='none'; }
        if (!this.drawLeftIcon || this.hideNavIcons) { this.addIcon.dom.style.display='none'; }
        if (!this.drawRightIcon || this.hideNavIcons) { this.removeIcon.dom.style.display='none'; }
        if (!this.drawTopIcon || this.hideNavIcons) { this.toTopIcon.dom.style.display='none'; }
        if (!this.drawBotIcon || this.hideNavIcons) { this.toBottomIcon.dom.style.display='none'; }

        var tb = p.body.first();
        this.el.setWidth(p.body.first().getWidth());
        p.body.removeClass();

        this.hiddenName = this.name;
        var hiddenTag = {tag: "input", type: "hidden", value: "", name: this.name};
        this.hiddenField = this.el.createChild(hiddenTag);
    },
    
    doLayout: function(){
        if(this.rendered){
            this.fromMultiselect.fs.doLayout();
            this.toMultiselect.fs.doLayout();
        }
    },

    afterRender: function(){
        Ext.ux.form.ItemSelector.superclass.afterRender.call(this);

        this.toStore = this.toMultiselect.store;
        this.toStore.on('add', this.valueChanged, this);
        this.toStore.on('remove', this.valueChanged, this);
        this.toStore.on('load', this.valueChanged, this);
        this.valueChanged(this.toStore);
    },

    toTop : function() {
        var selectionsArray = this.toMultiselect.view.getSelectedIndexes();
        var records = [];
        if (selectionsArray.length > 0) {
            selectionsArray.sort();
            for (var i=0; i<selectionsArray.length; i++) {
                record = this.toMultiselect.view.store.getAt(selectionsArray[i]);
                records.push(record);
            }
            selectionsArray = [];
            for (var i=records.length-1; i>-1; i--) {
                record = records[i];
                this.toMultiselect.view.store.remove(record);
                this.toMultiselect.view.store.insert(0, record);
                selectionsArray.push(((records.length - 1) - i));
            }
        }
        this.toMultiselect.view.refresh();
        this.toMultiselect.view.select(selectionsArray);
    },

    toBottom : function() {
        var selectionsArray = this.toMultiselect.view.getSelectedIndexes();
        var records = [];
        if (selectionsArray.length > 0) {
            selectionsArray.sort();
            for (var i=0; i<selectionsArray.length; i++) {
                record = this.toMultiselect.view.store.getAt(selectionsArray[i]);
                records.push(record);
            }
            selectionsArray = [];
            for (var i=0; i<records.length; i++) {
                record = records[i];
                this.toMultiselect.view.store.remove(record);
                this.toMultiselect.view.store.add(record);
                selectionsArray.push((this.toMultiselect.view.store.getCount()) - (records.length - i));
            }
        }
        this.toMultiselect.view.refresh();
        this.toMultiselect.view.select(selectionsArray);
    },

    up : function() {
        var record = null;
        var selectionsArray = this.toMultiselect.view.getSelectedIndexes();
        selectionsArray.sort();
        var newSelectionsArray = [];
        if (selectionsArray.length > 0) {
            for (var i=0; i<selectionsArray.length; i++) {
                record = this.toMultiselect.view.store.getAt(selectionsArray[i]);
                if ((selectionsArray[i] - 1) >= 0) {
                    this.toMultiselect.view.store.remove(record);
                    this.toMultiselect.view.store.insert(selectionsArray[i] - 1, record);
                    newSelectionsArray.push(selectionsArray[i] - 1);
                }
            }
            this.toMultiselect.view.refresh();
            this.toMultiselect.view.select(newSelectionsArray);
        }
    },

    down : function() {
        var record = null;
        var selectionsArray = this.toMultiselect.view.getSelectedIndexes();
        selectionsArray.sort();
        selectionsArray.reverse();
        var newSelectionsArray = [];
        if (selectionsArray.length > 0) {
            for (var i=0; i<selectionsArray.length; i++) {
                record = this.toMultiselect.view.store.getAt(selectionsArray[i]);
                if ((selectionsArray[i] + 1) < this.toMultiselect.view.store.getCount()) {
                    this.toMultiselect.view.store.remove(record);
                    this.toMultiselect.view.store.insert(selectionsArray[i] + 1, record);
                    newSelectionsArray.push(selectionsArray[i] + 1);
                }
            }
            this.toMultiselect.view.refresh();
            this.toMultiselect.view.select(newSelectionsArray);
        }
    },

    fromTo : function() {
        var selectionsArray = this.fromMultiselect.view.getSelectedIndexes();
        var records = [];
        if (selectionsArray.length > 0) {
            for (var i=0; i<selectionsArray.length; i++) {
                record = this.fromMultiselect.view.store.getAt(selectionsArray[i]);
                records.push(record);
            }
            if(!this.allowDup)selectionsArray = [];
            for (var i=0; i<records.length; i++) {
                record = records[i];
                if(this.allowDup){
                    var x=new Ext.data.Record();
                    record.id=x.id;
                    delete x;
                    this.toMultiselect.view.store.add(record);
                }else{
                    this.fromMultiselect.view.store.remove(record);
                    this.toMultiselect.view.store.add(record);
                    selectionsArray.push((this.toMultiselect.view.store.getCount() - 1));
                }
            }
        }
        this.toMultiselect.view.refresh();
        this.fromMultiselect.view.refresh();
        var si = this.toMultiselect.store.sortInfo;
        if(si){
            this.toMultiselect.store.sort(si.field, si.direction);
        }
        this.toMultiselect.view.select(selectionsArray);
    },

    toFrom : function() {
        var selectionsArray = this.toMultiselect.view.getSelectedIndexes();
        var records = [];
        if (selectionsArray.length > 0) {
            for (var i=0; i<selectionsArray.length; i++) {
                record = this.toMultiselect.view.store.getAt(selectionsArray[i]);
                records.push(record);
            }
            selectionsArray = [];
            for (var i=0; i<records.length; i++) {
                record = records[i];
                this.toMultiselect.view.store.remove(record);
                if(!this.allowDup){
                    this.fromMultiselect.view.store.add(record);
                    selectionsArray.push((this.fromMultiselect.view.store.getCount() - 1));
                }
            }
        }
        this.fromMultiselect.view.refresh();
        this.toMultiselect.view.refresh();
        var si = this.fromMultiselect.store.sortInfo;
        if (si){
            this.fromMultiselect.store.sort(si.field, si.direction);
        }
        this.fromMultiselect.view.select(selectionsArray);
    },

    valueChanged: function(store) {
        var record = null;
        var values = [];
        for (var i=0; i<store.getCount(); i++) {
            record = store.getAt(i);
            values.push(record.get(this.toMultiselect.valueField));
        }
        this.hiddenField.dom.value = values.join(this.delimiter);
        this.fireEvent('change', this, this.getValue(), this.hiddenField.dom.value);
    },

    getValue : function() {
        return this.hiddenField.dom.value;
    },

    onRowDblClick : function(vw, index, node, e) {
        if (vw == this.toMultiselect.view){
            this.toFrom();
        } else if (vw == this.fromMultiselect.view) {
            this.fromTo();
        }
        return this.fireEvent('rowdblclick', vw, index, node, e);
    },

    reset: function(){
        range = this.toMultiselect.store.getRange();
        this.toMultiselect.store.removeAll();
        this.fromMultiselect.store.add(range);
        var si = this.fromMultiselect.store.sortInfo;
        if (si){
            this.fromMultiselect.store.sort(si.field, si.direction);
        }
        this.valueChanged(this.toMultiselect.store);
    }
});

Ext.reg('itemselector', Ext.ux.form.ItemSelector);

//backwards compat
Ext.ux.ItemSelector = Ext.ux.form.ItemSelector;
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

var router, treeId, initTreeDialogs;

router = Zenoss.remote.TemplateRouter;
treeId = 'templateTree';



/**
 * The two default views
 **/
Ext.ns('Zenoss', 'Zenoss.templates');
Zenoss.templates.templateView = 'template';
Zenoss.templates.deviceClassView = 'deviceClass';


initTreeDialogs = function(tree) {

    new Zenoss.HideFormDialog({
        id: 'addTemplateDialog',
        title: _t('Add Monitoring Template'),
        items: {
            xtype: 'textfield',
            id: 'idTextfield',
            fieldLabel: _t('Name'),
            allowBlank: false
        },
        listeners: {
            'hide': function(treeDialog) {
                Ext.getCmp('idTextfield').setValue('');
            }
        },
        buttons: [
            {
                xtype: 'HideDialogButton',
                text: _t('Submit'),
                handler: function(button, event) {
                    var id = Ext.getCmp('idTextfield').getValue();
                    tree.addTemplate(id);
                }
            }, {
                xtype: 'HideDialogButton',
                text: _t('Cancel')
            }
        ]
    });

    new Zenoss.MessageDialog({
        id: 'deleteNodeDialog',
        title: _t('Delete Tree Node'),
        message: _t('The selected node will be deleted.'),
        okHandler: function(){
            tree.deleteTemplate();
        }
    });

};

Ext.ns('Zenoss');

/**
 * @class Zenoss.TemplateTreePanel
 * @extends Ext.tree.TreePanel
 * @constructor
 */
Zenoss.TemplateTreePanel = Ext.extend(Ext.tree.TreePanel, {

    constructor: function(config) {
        var view = config.view,
            directFn = router.getTemplates;
        if (view == Zenoss.templates.deviceClassView) {
            directFn = router.asyncGetTree;
        }

        Ext.applyIf(config, {
            id: treeId,
            rootVisible: false,
            border: false,
            autoScroll: true,
            containerScroll: true,
            useArrows: true,
            loadMask: true,
            cls: 'x-tree-noicon',
            loader: {
                directFn: directFn,
                baseAttrs: {singleClickExpand: true},
                getParams: function(node) {
                    return [node.attributes.uid];
                },
                listeners: {
                    beforeload: function(){
                        this.showLoadMask(true);
                    }.createDelegate(this),
                    load: function(){
                        this.showLoadMask(false);
                    }.createDelegate(this)
                }
            },
            root: {
                nodeType: 'async',
                id: 'root',
                uid: '/zport/dmd/Devices'
            },
            listeners: {
                scope: this,
                expandnode: this.onExpandnode
            }
        });
        Zenoss.TemplateTreePanel.superclass.constructor.call(this, config);
        initTreeDialogs(this);
        this.on('buttonClick', this.buttonClickHandler, this);
    },
    showLoadMask: function(bool) {
        if (!this.loadMask) { return; } 
        var container = this.container;
        container._treeLoadMask = container._treeLoadMask || new Ext.LoadMask(this.container);
        var mask = container._treeLoadMask,
            _ = bool ? mask.show() : [mask.hide(), mask.disable()];
    },
    buttonClickHandler: function(buttonId) {
        switch(buttonId) {
            case 'addButton':
                Ext.getCmp('addTemplateDialog').show();
                break;
            case 'deleteButton':
                Ext.getCmp('deleteNodeDialog').show();
                break;
            default:
                break;
        }
    },

    addTemplate: function(id) {
        var rootNode, contextUid, params, tree, type;
        rootNode = this.getRootNode();
        contextUid = rootNode.attributes.uid;
        params = {contextUid: contextUid, id: id};
        tree = this;
        function callback(provider, response) {
            var result, nodeConfig, node, leaf;
            result = response.result;
            if (result.success) {
                nodeConfig = response.result.nodeConfig;
                node = tree.getLoader().createNode(nodeConfig);
                rootNode.appendChild(node);
                node.expand();
                leaf = node.childNodes[0];
                leaf.select();
            } else {
                Ext.Msg.alert('Error', result.msg);
            }
        }
        router.addTemplate(params, callback);
    },

    deleteTemplate: function() {
        var node, params, me;
        node = this.getSelectionModel().getSelectedNode();
        params = {uid: node.attributes.uid};
        me = this;
        function callback(provider, response) {
            me.getRootNode().reload();
        }
        router.deleteTemplate(params, callback);
    },
    afterRender: function() {
        Zenoss.TemplateTreePanel.superclass.afterRender.call(this);

        // add the search text box
        this.add({
            xtype: 'searchfield',
            ref: 'searchField',
            bodyStyle: {padding: 10},
            listeners: {
                valid: this.filterTree,
                scope: this
            }
        });
    },

    clearFilter: function() {
        // use set raw value to not trigger listeners
        this.searchField.setRawValue('');
        this.hiddenPkgs = [];
    },

    filterTree: function(e) {
        var re,
            text = e.getValue();

        // show all of our hidden nodes
        if (this.hiddenPkgs) {
            Ext.each(this.hiddenPkgs, function(node){node.ui.show();});
        }

        // de-select the selected node
        if (this.getSelectionModel().getSelectedNode()){
            this.getSelectionModel().getSelectedNode().unselect();
        }

        this.hiddenPkgs = [];
        if (!text) {
            return;
        }
        this.expandAll();

        // test every node against the Regular expression
        re = new RegExp(Ext.escapeRe(text), 'i');
        this.root.cascade(function(node){
            var attr = node.id, parentNode;
            if (!node.isRoot) {
                if (re.test(attr)) {
                    // if regex passes show our node and our parent
                    parentNode = node.parentNode;
                    while (parentNode) {
                        if (!parentNode.hidden) {
                            break;
                        }
                        parentNode.ui.show();
                        parentNode = parentNode.parentNode;
                    }
                    // the cascade is stopped on this branch
                    return false;
                } else {
                    node.ui.hide();
                    this.hiddenPkgs.push(node);
                }
            }
            // continue cascading down the tree from this node
            return true;
        }, this);
    },

    onExpandnode: function(node) {
        // select the first template when the base URL is accessed without a
        // history token and without a filter value
        if ( ! this.searchField.getValue() && ! Ext.History.getToken() ) {
            if ( node === this.getRootNode() ) {
                node.childNodes[0].expand();
            } else {
                node.childNodes[0].select();
            }
        }
    },
    selectByToken: function(uid) {
        if (this.view == Zenoss.templates.deviceClassView){
            this.selectPath(unescape(uid));
        }else{
            this.templateViewSelectByToken(uid);
        }
    },
    templateViewSelectByToken: function(uid) {
        // called on Ext.History change event (see HistoryManager.js)
        // convert uid to path and select the path
        // example uid: '/zport/dmd/Devices/Power/UPS/APC/rrdTemplates/Device'
        // example path: '/root/Device/Device..Power.UPS.APC'
        var templateSplit, pathParts, nameParts,
            templateName, dmdPath, path, deviceName;

        if (uid.search('/rrdTemplates/') != -1) {
            templateSplit = unescape(uid).split('/rrdTemplates/');
            pathParts = templateSplit[0].split('/');
            nameParts = templateSplit[1].split('/');
            templateName = nameParts[0];
        }else{
            // it is a template on a device
            pathParts = uid.replace('/devices/', '/').split('/');
            templateName = pathParts.pop();
        }

        if ( pathParts.length === 4 ) {
            // Defined at devices, special case, include 'Devices'
            dmdPath = 'Devices';
        } else {
            // all the DeviceClass names under Devices separated by dots
            dmdPath = pathParts.slice(4).join('.');
        }
        path = String.format('/root/{0}/{0}..{1}', templateName, dmdPath);
        this.selectPath(path);
    }

});

Ext.reg('TemplateTreePanel', Zenoss.TemplateTreePanel);

})();
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

/* package level */
(function() {

    var ZD = Ext.ns('Zenoss.devices');

    ZD.PriorityCombo = Ext.extend(Ext.form.ComboBox, {
        constructor: function(config) {
            config = Ext.apply(config || {}, {
                fieldLabel: _t('Priority'),
                store: new Ext.data.DirectStore({
                    directFn: Zenoss.remote.DeviceRouter.getPriorities,
                    root: 'data',
                    fields: ['name', 'value']
                }),
                valueField: 'value',
                displayField: 'name',
                forceSelection: true,
                editable: false,
                autoSelect: true,
                triggerAction: 'all'
            });
            ZD.PriorityCombo.superclass.constructor.call(this, config);
        }
    });

    Ext.reg('PriorityCombo', ZD.PriorityCombo);


    ZD.ProductionStateCombo = Ext.extend(Ext.form.ComboBox, {
        constructor: function(config) {
            config = Ext.apply(config || {}, {
                fieldLabel: _t('Production State'),
                store: new Ext.data.DirectStore({
                    directFn: Zenoss.remote.DeviceRouter.getProductionStates,
                    root: 'data',
                    fields: ['name', 'value']
                }),
                valueField: 'value',
                displayField: 'name',
                forceSelection: true,
                editable: false,
                autoSelect: true,
                triggerAction: 'all'
            });
            ZD.ProductionStateCombo.superclass.constructor.call(this, config);

        }
    });

    Ext.reg('ProductionStateCombo', ZD.ProductionStateCombo);

    ZD.ProductionStateMultiselectMenu = Ext.extend(Zenoss.MultiselectMenu, {
        constructor: function(config) {
            config = Ext.apply(config || {}, {
                text:'...',
                store: new Ext.data.DirectStore({
                    directFn: Zenoss.remote.DeviceRouter.getProductionStates,
                    root: 'data',
                    fields: ['name', 'value']
                }),
                defaultValues: ['1000']
            });
            ZD.ProductionStateMultiselectMenu.superclass.constructor.call(this, config);
        }
    });

    Ext.reg('multiselect-prodstate', ZD.ProductionStateMultiselectMenu);

    ZD.ManufacturerDataStore = Ext.extend(Ext.data.DirectStore, {
        constructor: function(config) {
            config = config || {};
            var router = config.router || Zenoss.remote.DeviceRouter;
            Ext.applyIf(config, {
                root: 'manufacturers',
                totalProperty: 'totalCount',
                fields: ['name'],
                directFn: router.getManufacturerNames
            });
            ZD.ManufacturerDataStore.superclass.constructor.call(this, config);
        }
    });

    ZD.OSProductDataStore = Ext.extend(Ext.data.DirectStore, {
        constructor: function(config) {
            config = config || {};
            var router = config.router || Zenoss.remote.DeviceRouter;
            Ext.applyIf(config, {
                root: 'productNames',
                totalProperty: 'totalCount',
                fields: ['name'],
                directFn: router.getOSProductNames
            });
            ZD.ManufacturerDataStore.superclass.constructor.call(this, config);
        }
    });

    ZD.HWProductDataStore = Ext.extend(Ext.data.DirectStore, {
        constructor: function(config) {
            config = config || {};
            var router = config.router || Zenoss.remote.DeviceRouter;
            Ext.applyIf(config, {
                root: 'productNames',
                totalProperty: 'totalCount',
                fields: ['name'],
                directFn: router.getHardwareProductNames
            });
            ZD.ManufacturerDataStore.superclass.constructor.call(this, config);
        }
    });

    ZD.ManufacturerCombo = Ext.extend(Ext.form.ComboBox, {
        constructor: function(config) {
            var store = (config||{}).store || new ZD.ManufacturerDataStore();
            config = Ext.applyIf(config||{}, {
                store: store,
                triggerAction: 'all',
                selectOnFocus: true,
                valueField: 'name',
                displayField: 'name',
                forceSelection: true,
                editable: false,
                width: 160
            });
            ZD.ManufacturerCombo.superclass.constructor.call(this, config);
        }
    });
    Ext.reg('manufacturercombo', ZD.ManufacturerCombo);

    ZD.ProductCombo = Ext.extend(Ext.form.ComboBox, {
        constructor: function(config) {
            var prodType = config.prodType || 'OS',
                store = (config||{}).store || 
                    prodType=='OS' ? new ZD.OSProductDataStore() : new ZD.HWProductDataStore();
            config = Ext.applyIf(config||{}, {
                store: store,
                triggerAction: 'all',
                selectOnFocus: true,
                valueField: 'name',
                displayField: 'name',
                forceSelection: true,
                editable: false,
                width: 160
            });
            ZD.ProductCombo.superclass.constructor.call(this, config);
        }
    });
    Ext.reg('productcombo', ZD.ProductCombo);


}());
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

var router, dataSourcesId, graphsId, resetCombo,  
    addMetricToGraph, showAddToGraphDialog, editDataSourcesId, treeId,
    dataSourceMenu, editingReSelectId;

Ext.ns('Zenoss');

router = Zenoss.remote.TemplateRouter;
dataSourcesId = 'dataSourceTreeGrid';
graphsId = 'graphGrid';
editDataSourcesId = "editDataSource";
     
// NOTE: this must match the tree id from the template.js file
treeId = 'templateTree';

/**
 *@returns the currently selected Data Source or Data Point, or none if nothing is selected
 **/
function getSelectedDataSourceOrPoint() {
    return Ext.getCmp(dataSourcesId).getSelectionModel().getSelectedNode();   
}
     
resetCombo = function(combo, uid) {
    combo.clearValue();
    combo.getStore().setBaseParam('uid', uid);
    delete combo.lastQuery;
    combo.doQuery(combo.allQuery, true);
};

addMetricToGraph = function(dataPointUid, graphUid) {
    var params, callback;
    params = {dataPointUid: dataPointUid, graphUid: graphUid};
    callback = function(provider, response) {
        Ext.getCmp(graphsId).getStore().reload();
    };
    router.addDataPointToGraph(params, callback);
};

Zenoss.GraphStore = Ext.extend(Ext.data.DirectStore, {
    constructor: function(config) {
        Ext.applyIf(config, {
            xtype: 'directstore',
            directFn: router.getGraphs,
            idProperty: 'uid',
            fields: ['uid', 'name', 'graphPoints', 'units', 'height', 'width',
                     'sequence']
        });
        Zenoss.GraphStore.superclass.constructor.call(this, config);
    }
});
Ext.reg('graphstore', Zenoss.GraphStore);

new Zenoss.HideFormDialog({
    id: 'addToGraphDialog',
    title: _t('Add Data Point to Graph'),
    items: [
    {
        xtype: 'panel',
        id: 'addToGraphMetricPanel',
        border: false
    }, {
        xtype: 'combo',
        id: 'graphCombo',
        fieldLabel: _t('Graph'),
        displayField: 'name',
        valueField: 'uid',
        forceSelection: true,
        minChars: 999, // only do an all query
        triggerAction: 'all',
        emptyText: 'Select a graph...',
        selectOnFocus: true,
        store: {xtype: 'graphstore'},
        listeners: {select: function(){
            Ext.getCmp('addToGraphDialog').submit.enable();
        }}
    }],
    buttons: [
    {
        xtype: 'HideDialogButton',
        ref: '../submit',
        text: _t('Submit'),
        disabled: true,
        handler: function(button, event) {
            var node, datapointUid, graphUid;
            node = Ext.getCmp(dataSourcesId).getSelectionModel().getSelectedNode();
            datapointUid = node.attributes.uid;
            graphUid = Ext.getCmp('graphCombo').getValue();
            addMetricToGraph(datapointUid, graphUid);
        }
    }, {
        xtype: 'HideDialogButton',
        text: _t('Cancel')
    }]
});

showAddToGraphDialog = function() {
    var smTemplate, templateUid, smDataSource, 
        nodeDataSource, metricName, html, combo;
    smTemplate = Ext.getCmp('templateTree').getSelectionModel();
    templateUid = smTemplate.getSelectedNode().attributes.uid;
    smDataSource = Ext.getCmp(dataSourcesId).getSelectionModel();
    nodeDataSource = smDataSource.getSelectedNode();
    if ( nodeDataSource && nodeDataSource.isLeaf() ) {
        metricName = nodeDataSource.attributes.name;
        html = '<div>Data Point</div>';
        html += '<div>' + metricName + '</div><br/>';
        Ext.getCmp('addToGraphDialog').show();
        Ext.getCmp('addToGraphMetricPanel').body.update(html);
        combo = Ext.getCmp('graphCombo');
        resetCombo(combo, templateUid);
        Ext.getCmp('addToGraphDialog').submit.disable();
    } else {
        Ext.Msg.alert('Error', 'You must select a Data Point.');
    }
};
     
     
/**********************************************************************
 *
 * Add Data Point
 *
 **/
     
/**
 * Causes the DataSources Grid to refresh from the server
 * 
 **/
function refreshDataSourceGrid(selectedId) {
    var grid = Ext.getCmp(dataSourcesId);
    Ext.getCmp('addDataPointDialog').hide();
    Ext.getCmp('addDataSourceDialog').hide();
    if (selectedId) {
        grid.getRootNode().reload(function(){
            grid.getRootNode().cascade(function(node){
                if (node.attributes.id == selectedId) {
                    node.expand();
                    node.select();
                }
            });                                     
        });
    }else{
        grid.getRootNode().reload();
    }
}

/**
 * Gets the DataPoint name from the dialog and sends it to the server
 **/
function saveDataPoint() {
    var grid = Ext.getCmp(dataSourcesId),
        selectedNode = grid.getSelectionModel().getSelectedNode(),
        parameters, selectedId;

    // if we have a datapoint, find the datasource associated with it
    if (selectedNode.attributes.leaf) {
        selectedNode = selectedNode.parentNode;
    }
    
    parameters = {
        name: Ext.getCmp('metricName').getValue(),
        dataSourceUid: selectedNode.attributes.uid
    };
    selectedId = selectedNode.attributes.id;
    // get selected datasource, and reopen the grid to that point
    function callback() {
        refreshDataSourceGrid(selectedId);
    }
    return router.addDataPoint(parameters, callback);
                              
}
     
/**
 * Add Data Point Dialog Configuration
 **/
new Ext.Window({
        id: 'addDataPointDialog',
        title: _t('Add Data Point'),
        height: 160,
        width: 310,
        plain: true,
        modal: true,
        
        closeAction: 'hide',
        listeners: {
            hide: function() {
                Ext.getCmp('metricName').setValue(null);
                Ext.getCmp('metricName').clearInvalid();
            }
        },
        items:{
            xtype: 'form',
            border: false,
            buttonAlign: 'left',
            monitorValid: true,
            items: [{                       
                xtype: 'idfield',
                id: 'metricName',
                fieldLabel: _t('Name'),
                allowBlank: false,
                blankText: _t('Name is a required field')
                   }],
            buttons: [{
                    xtype: 'button',
                    text: _t('Submit'),
                    formBind: true,
                    handler: saveDataPoint
                }, {
                    xtype: 'button',
                    text: _t('Cancel'),
                    handler: function() {
                        Ext.getCmp('addDataPointDialog').hide();
                    }
                }]
            
        }
            
    }
);
     
/**
 * Displays the Add Data Point dialog and saves the inputted infomation
 * back to the server
 **/
function showAddDataPointDialog() {
    var grid = Ext.getCmp(dataSourcesId),
        selectedNode = grid.getSelectionModel().getSelectedNode();
    
    // make sure they selected a node
    if (!selectedNode) {
        Ext.Msg.alert(_t('Error'), _t('You must select data source'));
        return; 
    }
    
    // display the name dialog
    Ext.getCmp('addDataPointDialog').show();
}
     
/**********************************************************************
 *
 * Add Data Source
 *
 */
     
/**
 * Gets the info from the Add Datasource dialog and sends it to the server
 **/
function saveDataSource() {
    var grid = Ext.getCmp(treeId),
        selectedNode = grid.getSelectionModel().getSelectedNode(),    
        parameters = {
            name: Ext.getCmp('dataSourceName').getValue(),
            type: Ext.getCmp('dataSourceTypeCombo').getValue(),
            templateUid: selectedNode.attributes.uid
        };
    return router.addDataSource(parameters, refreshDataSourceGrid);
}
     
new Ext.Window({
        id: 'addDataSourceDialog',
        title: _t('Add Data Source'),
        height: 160,
        width: 310,
        modal: true,
        plain: true,
        closeAction: 'hide',
        buttonAlight: 'left',
        listeners: {
            hide: function() {
                Ext.getCmp('dataSourceTypeCombo').setValue('SNMP');
                Ext.getCmp('dataSourceName').setValue('');
                Ext.getCmp('dataSourceName').clearInvalid();
            }
        },
        items:{
            xtype:'form',
            border: false,
            buttonAlign: 'left',
            monitorValid: true,
            items:[{
            xtype: 'idfield',
            id: 'dataSourceName',
            fieldLabel: _t('Name'),
            allowBlank: false,
            blankText: _t('Name is a required field')           
          },
          {
            xtype: 'combo',
            id: 'dataSourceTypeCombo',
            displayField: 'type',
            fieldLabel: _t('Type'),
            editable: false,
            forceSelection: true,
            autoSelect: true,
            value: 'SNMP',
            selectOnFocus: true,
            triggerAction: 'all',
            store:  new Ext.data.DirectStore({
                fields: ['type'],
                root: 'data',
                directFn: router.getDataSourceTypes
            })
        }],
        buttons:[{
            xtype: 'button',
            text: _t('Submit'),
            formBind: true,
            handler: saveDataSource
        },{
            xtype: 'button',
            text: _t('Cancel'),
            handler: function () {
                Ext.getCmp('addDataSourceDialog').hide();
            }
        }]} 
});
     
/**
 * Shows the Add Data Source dialog and saves the inputted information
 * back to the server
 **/
function showAddDataSourceDialog() {
    var cmp = Ext.getCmp(treeId),
        selectedNode = cmp.getSelectionModel().getSelectedNode();
    
    // make sure they selected a node
    if (!selectedNode) {
        Ext.Msg.alert(_t('Error'), _t('You must select a template'));
        return; 
    }
    // clear the entries (all of our forms are blank when you load them)
    Ext.getCmp('addDataSourceDialog').show();    
}

/**********************************************************************
 *
 * Delete DataSource
 *
 */

/**
 * Creates the dynamic delete message and shows the dialog
 **/
function showDeleteDataSourceDialog() {
    var msg, name, html, dialog;
    if (getSelectedDataSourceOrPoint()) {
        // set up the custom delete message 
        msg = _t("Are you sure you want to remove {0}? There is no undo.");
        name = getSelectedDataSourceOrPoint().attributes.name;
        html = String.format(msg, name);

        // show the dialog
        dialog = Ext.getCmp('deleteDataSourceDialog');
        dialog.show();
        dialog.getComponent('message').update(html);
    }else{
        Ext.Msg.alert(_t('Error'), _t('You must select a Data Source or Data Point.'));
    }
}

new Zenoss.MessageDialog({
    id: 'deleteDataSourceDialog',
    title: _t('Delete'),
    // msg is generated dynamically
    okHandler: function(){
        var params, node = getSelectedDataSourceOrPoint(),
        selectedId;
        params = {
            uid: getSelectedDataSourceOrPoint().id
        };
        
        // data points are always leafs
        if (getSelectedDataSourceOrPoint().leaf) {
            selectedId = node.parentNode.attributes.id;
            function callback() {
                refreshDataSourceGrid(selectedId);
            }
            router.deleteDataPoint(params, callback);  
        }else {
            router.deleteDataSource(params, refreshDataSourceGrid); 
        }
    }
});
     
/**********************************************************************
 *
 * Edit DataSource/DataPoint 
 *
 */
     
/**
 * Closes the edit dialog and updates the store of the datasources. 
 * This is called after the router request to save the edit dialog
 **/
function closeEditDialog(response) {
    var dialog = Ext.getCmp('editDataSources');
    refreshDataSourceGrid(editingReSelectId);
        
    // hide the dialog
    if (dialog) {
        dialog.hide();
    }
}

/**
 * Event handler for when a user wants to test a datasource
 * against a specific device.
 **/
function testDataSource() {
    var cmp = Ext.getCmp(editDataSourcesId),
        values = cmp.editForm.form.getValues(),
        win, testDevice, data;
    
    testDevice = values.testDevice;
            
    win = new Zenoss.CommandWindow({
        uids: testDevice,
        title: _t('Test Data Source'),
        data: values,
        target: values.uid + '/test_datasource'
    });
        
    win.show();
}

/**
 * Used when we save the data grid, it needs to 
 * explicitly get the "Alias" value and turn it into a
 * list before going back to the server
 **/
function submitDataPointForm (values, callback) {
    // will always have only one alias form
    var aliases = Ext.getCmp(editDataSourcesId).findByType('alias'),
        alias;

    // assert that we have one exactly one alias form
    if (aliases.length < 1) {
        throw "The DataPoint form does not have an alias field, it should have only one";
    }
    
    alias = aliases[0];
    values.aliases = alias.getValue();
    router.setInfo(values, callback);
}
     
/**
 * Event handler for editing a specific datasource or
 * datapoint. 
 **/
function editDataSourceOrPoint() {
    var cmp = Ext.getCmp(dataSourcesId),
        selectedNode = cmp.getSelectionModel().getSelectedNode(),
        attributes,
        isDataPoint = false,
        params, reselectId;
    
    // make sure they selected something
    if (!selectedNode) {
        Ext.Msg.alert(_t('Error'), _t('You must select a data source or data point.'));
        return;
    }
    attributes = selectedNode.attributes;
    
    // find out if we are editing a datasource or a datapoint
    if (attributes.leaf) {
        isDataPoint = true;
        editingReSelectId = selectedNode.parentNode.attributes.id;
    }else{
        editingReSelectId = attributes.id;
    }
    
    // parameters for the router call    
    params = {
        uid: attributes.uid  
    };
    
    // callback for the router request
    function displayEditDialog(response) {
        var win,
        config = {};
        
        config.record = response.record;
        config.items = response.form;
        config.xtype = "datasourceeditdialog";
        config.id = editDataSourcesId;
        config.isDataPoint = isDataPoint;
        config.title = _t('Edit Data Source');
        config.directFn = router.setInfo;
        config.width = 800;
        if (isDataPoint) {
            config.title = _t('Edit Data Point');
            config.directFn = submitDataPointForm;
            config.singleColumn = true;
        }else if (config.record.testable &&
                  Zenoss.Security.hasPermission('Change Device')){
            // add the test against device panel
            config.items.items.push({
                xtype:'panel',
                columnWidth: 0.5,
                baseCls: 'test-against-device',
                hidden: Zenoss.Security.doesNotHavePermission('Run Commands'),
                title: _t('Test Against a Device'),
                items:[{
                    xtype: 'textfield',
                    fieldLabel: _t('Device Name'),
                    id: 'testDevice',
                    name: 'testDevice'
                },{
                    xtype: 'hidden',
                    name: 'uid',
                    value: response.record.id
                },{
                    xtype: 'button',
                    text: _t('Test'),
                    handler: testDataSource
                }]});
         
        }
        
        config.saveHandler = closeEditDialog;
        win = Ext.create(config);
        win.show();
    }
    
    // get the details
    if (isDataPoint) {
        router.getDataPointDetails(params, displayEditDialog);
    }else{
        router.getDataSourceDetails(params, displayEditDialog);
    }
}

dataSourceMenu = new Ext.menu.Menu({
    id: 'dataSourceMenu',
    items: [{
        xtype: 'menuitem',
        text: _t('Add Data Point To Graph'),
        disable: Zenoss.Security.doesNotHavePermission('Manage DMD'),
        handler: showAddToGraphDialog
    },{
        xtype: 'menuitem',
        text: _t('Add Data Point'),
        disable: Zenoss.Security.doesNotHavePermission('Manage DMD'),
        handler: showAddDataPointDialog
    },{
        xtype: 'menuitem',
        text: _t('View and Edit Details'),
        disable: Zenoss.Security.doesNotHavePermission('Manage DMD'),
        handler: editDataSourceOrPoint
    }]
});
     
/**
 * @class Zenoss.DataSourceTreeGrid
 * @extends Ext.ux.tree.TreeGrid
 * @constructor
 */
Zenoss.DataSourceTreeGrid = Ext.extend(Ext.ux.tree.TreeGrid, {

    constructor: function(config) {
        Ext.applyIf(config, {
            border: false,
            useArrows: true,
            cls: 'x-tree-noicon',
            id: dataSourcesId,
            title: _t('Data Sources'),
            listeners: {
                // when they doubleclick we will open up the tree and
                // display the dialog
                beforedblclick: editDataSourceOrPoint                
            },
            loader: new Ext.ux.tree.TreeGridLoader({
                directFn: router.getDataSources
            }),
            tbar: [{
                    xtype: 'button',
                    iconCls: 'add',
                    id:'datasourceAddButton',
                    ref: '../addButton',
                    disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                    handler: showAddDataSourceDialog,
                    listeners: {
                        render: function() {
                            Zenoss.registerTooltipFor('datasourceAddButton');
                        }
                    }
            }, {
                xtype: 'button',
                iconCls: 'delete',
                ref: '../deleteButton',
                id: 'datasourceDeleteButton',
                disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('datasourceDeleteButton');
                    }
                },
                handler: showDeleteDataSourceDialog
            },{
                xtype: 'button',
                id: 'datasourceEditButton',
                iconCls: 'customize',
                ref: '../customizeButton',
                disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('datasourceEditButton');
                    }
                },
                menu: 'dataSourceMenu'
            }],
            columns: [
                {
                    id: 'name',
                    dataIndex: 'name',
                    header: 'Data Points by Data Source',
                    width: 250
                }, {
                    dataIndex: 'source',
                    header: 'Source',
                    width: 250
                }, {
                    dataIndex: 'enabled',
                    header: 'Enabled',
                    width: 40
                }, {
                    dataIndex: 'type',
                    header: 'Type',
                    width: 90
                }
            ]
        });
        Zenoss.DataSourceTreeGrid.superclass.constructor.call(this, config);
        
    },
    disableToolBarButtons: function(bool) {
        this.addButton.setDisabled(bool && Zenoss.Security.hasPermission('Manage DMD'));
        this.deleteButton.setDisabled(bool && Zenoss.Security.hasPermission('Manage DMD'));
        this.customizeButton.setDisabled(bool && Zenoss.Security.hasPermission('Manage DMD'));
    }
    
});

Ext.reg('DataSourceTreeGrid', Zenoss.DataSourceTreeGrid);

})();
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

/* package level */
(function() {
    Ext.namespace('Zenoss.templates');
    /**********************************************************************
     *
     * Variable Declarations
     *
     */
    var thresholdSelectionModel, thresholdDeleteButton, addThreshold,
        router, treeId, thresholdEditButton, MinMaxThresholdDialog, dataSourcesId, addThresholdDialog;
     
    Zenoss.templates.thresholdsId = 'thresholdGrid';
    thresholdDeleteButton = 'thresholdDeleteButton';
    thresholdEditButton = 'thresholdEditButton';
    dataSourcesId = 'dataSourceTreeGrid';
    router = Zenoss.remote.TemplateRouter;
     
    // The id of the tree on the left hand side of the screen
    treeId = 'templateTree';
     
    /**********************************************************************
     *
     * Add Threshold
     *
     */
     
    addThreshold = function(data){
        var uid, node, dataPoints, params, callback;
        uid = Ext.getCmp(treeId).getSelectionModel().getSelectedNode().attributes.uid;
        node = Ext.getCmp(dataSourcesId).getSelectionModel().getSelectedNode();
        if ( node && node.isLeaf() ) {
            dataPoints = [node.attributes.uid];
        } else {
            dataPoints = [];
        }
        params = {
            uid: uid, 
            thresholdType: data.thresholdTypeField,
            thresholdId: data.thresholdIdField,
            dataPoints: dataPoints
        };
        callback = function(provider, response) {
            Ext.getCmp(Zenoss.templates.thresholdsId).getStore().reload();
        };
        Zenoss.remote.TemplateRouter.addThreshold(params, callback);
    };
    
    addThresholdDialog = new Ext.create({
        xtype: 'window',
        id: 'addThresholdDialog',
        title: _t('Add Threshold'),
        message: _t('Allow the user to add a threshold.'),
        closeAction: 'hide',
        buttonAlign: 'left',
        autoScroll: true,
        plain: true,
        width: 375,
        autoHeight: true,
        modal: true,
        padding: 10,
        listeners:{
            show: function() {
                this.formPanel.getForm().reset();
            }
        },
        
        buttons: [{ 
            ref: '../submitButton',
            text: _t('Add'), 
            handler: function(submitButton) {
                var dialogWindow, basicForm; 
                dialogWindow = submitButton.refOwner; 
                basicForm = dialogWindow.formPanel.getForm();
                basicForm.api.submit(basicForm.getValues()); 
                dialogWindow.hide(); 
            } },
                  { 
                      ref: '../cancelButton', 
                      text: _t('Cancel'), 
                      handler: function(cancelButton) { 
                          var dialogWindow = cancelButton.refOwner; 
                          dialogWindow.hide();
                      } 
                  }],
        items: {
            xtype: 'form',
            ref: 'formPanel',
            leftAlign: 'top',
            monitorValid: true,
            border: false,
            paramsAsHash: true,
            api: { submit: addThreshold },
            listeners: {
                clientValidation: function(formPanel, valid) {
                    var dialogWindow;
                    dialogWindow = formPanel.refOwner;
                    dialogWindow.submitButton.setDisabled( !valid );
                }
            },
            items: [{
                name: 'thresholdTypeField',
                xtype: 'combo',
                fieldLabel: _t('Type'),
                displayField: 'type',
                forceSelection: true,
                triggerAction: 'all',
                emptyText: _t('Select a type...'),
                selectOnFocus: true,
                allowBlank: false,
                store: new Ext.data.DirectStore({
                    fields: ['type'],
                    root: 'data',
                    directFn: Zenoss.remote.TemplateRouter.getThresholdTypes
                })
            }, {
                name: 'thresholdIdField',
                xtype: 'idfield',
                fieldLabel: _t('Name'),
                allowBlank: false
            }
                   ]
        }});
    /**********************************************************************
     *
     * Delete Thresholds
     *
     */
     
    /**
     * Calls the router to delete the selected
     * threshold
     **/
    function deleteThreshold() {
        var grid, selectedRow, callback,
            opts;
        grid = Ext.getCmp(Zenoss.templates.thresholdsId);
        selectedRow = grid.getSelectionModel().getSelected();

        // delete threshold callback
        callback = function (response) {
            // only if we were successful. If there
            // was an exception we should see a dialog
            if (!response.success) {
                return;
            }

            // reload the thresholds grid
            grid.getStore().reload();

            // disable the delete button (they have to select another one to enable it)
            Ext.getCmp(thresholdDeleteButton).disable();
            Ext.getCmp(thresholdEditButton).disable();
        };
        
        // parameters for the router request
        opts = {
            'uid': selectedRow.id
        };
        
        router.removeThreshold(opts, callback);
    }

    /**
     * Configuration Options for the Delete Threshold Dialog
     **/
    function thresholdDeleteConfig(){
        // assuming there is always a selected node in the tree and the
        // threshold datagrid at this point
        var grid = Ext.getCmp(Zenoss.templates.thresholdsId),
            name = grid.getSelectionModel().getSelected().data.name,
            tree = Ext.getCmp(treeId),
            msg = String.format(_t("Are you sure you want to remove {0} from the template {1}? There is no undo."),
                name,
                tree.getSelectionModel().getSelectedNode().id.replace('./',''));
        
        return {
            title: _t('Remove Threshold'),
            modal: true,
            width: 310,
            height: 130,
            items: [{
                xtype: 'panel',
                bodyStyle: 'font-weight: bold; text-align:center',
                html: msg
            }],
            buttons: [{
                xtype: 'DialogButton',
                text: _t('Remove'),
                handler: deleteThreshold
            }, 
                Zenoss.dialog.CANCEL
            ]      
        };
    }
     
     
     /**********************************************************************
     *
     * Edit Thresholds
     *
     */

     
    /**
     * Call back from the router call to save
     * the threshold (from the edit dialog),
     * just reloads the threshold dialg
     **/
    function closeThresholdDialog(response) {
        // update the datasource of the threshold
        var grid = Ext.getCmp(Zenoss.templates.thresholdsId),
            dialog = Ext.getCmp('editThresholdDialog');
        grid.getStore().reload();
        // hide the form
        if (dialog) {
            dialog.hide();
        }
    }
     
    /**
     *@returns Zenoss.FormDialog Ext Dialog type associated with the
     *          selected threshold type
     **/
    function thresholdEdit() {
        var grid = Ext.getCmp(Zenoss.templates.thresholdsId),
            record = grid.getSelectionModel().getSelected(),
            config = {};
                            
        function displayEditDialog(response) {
            var win = Ext.create( {
                record: response.record,
                items: response.form,
                singleColumn: true,
                width: 650,
                xtype: 'datasourceeditdialog',
                title: _t('Edit Threshold'),
                directFn: router.setInfo,
                id: 'editThresholdDialog',
                saveHandler: closeThresholdDialog                 
            });
                        
            win.show();
        }

        // send the request for all of the threshold's info to the server
        router.getThresholdDetails({uid: record.id}, displayEditDialog);
    }
     
     
     /**********************************************************************
     *
     * Threshold Data Grid
     *
     */
    
    /**
     * Threshold DataGrid Selection Model
     * Handles when a user clicks on a specific threshold.
     **/
    thresholdSelectionModel = new Ext.grid.RowSelectionModel ({
        singleSelect: true,
        listeners : {
            /**
             * If they have permission and they select a row, show the
             * edit and delete buttons
             **/
            rowselect: function (selectionModel, rowIndex, record ) {
                // enable the "Delete Threshold" button
                if (Zenoss.Security.hasPermission('Manage DMD')) {
                    Ext.getCmp(thresholdDeleteButton).enable();
                    Ext.getCmp(thresholdEditButton).enable();
                }
            },
            
            /**
             * When they deselect don't allow them to press the buttons
             **/
            rowdeselect: function(selectionModel, rowIndex, record) {
                Ext.getCmp(thresholdDeleteButton).disable();
                Ext.getCmp(thresholdEditButton).disable();
            }
        }
    });

    /**
     * Definition for the Thresholds datagrid. This is used in 
     * templates.js in the updateThresholds function.
     **/
    Zenoss.templates.thresholdDataGrid = Ext.extend(Ext.grid.GridPanel, {
        constructor: function(config) {
            var listeners = {};
            
            listeners = Ext.apply(listeners, { rowdblclick: thresholdEdit});
            
            config = config || {};
            Ext.apply(config, {
                id: Zenoss.templates.thresholdsId,
                selModel: thresholdSelectionModel,
                title: _t('Thresholds'),
                border: false,
                store: {
                    xtype: 'directstore',
                    directFn: router.getThresholds,
                    fields: ['name', 'type', 'dataPoints', 'severity', 'enabled']
                },
                listeners: listeners,
                tbar: [{
                    xtype: 'button',
                    iconCls: 'add',
                    id: 'thresholdAddButton',
                    ref: '../addButton',
                    disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                    handler: function() {
                        Ext.getCmp('addThresholdDialog').show();
                    },
                    listeners: {
                        render: function() {
                            Zenoss.registerTooltipFor('thresholdAddButton');
                        }
                    }
                },
                    {
                    id: thresholdDeleteButton,
                    xtype: 'button',
                    iconCls: 'delete',
                    disabled: true,
                    handler: function() {
                        // when they press delete show the Confirmation
                        var win = new Zenoss.FormDialog(thresholdDeleteConfig());
                        win.show();
                    },
                    listeners: {
                        render: function() {
                            Zenoss.registerTooltipFor(thresholdDeleteButton);
                        }
                    }
                                       
                }, {
                    id: thresholdEditButton,
                    xtype: 'button',
                    iconCls: 'customize',
                    disabled: true,
                    handler: thresholdEdit,
                    listeners: {
                        render: function() {
                            Zenoss.registerTooltipFor(thresholdEditButton);
                        }
                    }
                }],
                colModel: new Ext.grid.ColumnModel({
                    columns: [
                        {dataIndex: 'name', header: _t('Name'), width:400}
                    ]
                })
            });
            
            Zenoss.templates.thresholdDataGrid.superclass.constructor.apply(
                this, arguments);
        }        
    });
    Ext.reg('thresholddatagrid', Zenoss.templates.thresholdDataGrid);
}());/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

var router, getSelectedTemplate, getSelectedGraphDefinition,
    addGraphDefinition, deleteGraphDefinition, addThresholdToGraph;


Ext.ns('Zenoss', 'Zenoss.templates');

router = Zenoss.remote.TemplateRouter;

getSelectedTemplate = function() {
    return Ext.getCmp('templateTree').getSelectionModel().getSelectedNode();
};

getSelectedGraphDefinition = function() {
    return Ext.getCmp('graphGrid').getSelectionModel().getSelected();
};

function getSelectedGraphPoint() {
    var cmp = Ext.getCmp('graphPointGrid');
    if (cmp) {
        return cmp.getSelectionModel().getSelected();
    }
    return null;
}

addGraphDefinition = function(){
    var params, callback;
    params = {
        templateUid: getSelectedTemplate().attributes.uid,
        graphDefinitionId: Ext.getCmp('graphDefinitionIdTextfield').getValue()
    };
    callback = function(provider, response) {
        Ext.getCmp('graphGrid').getStore().reload();
    };
    router.addGraphDefinition(params, callback);
};

new Zenoss.HideFormDialog({
    id: 'addGraphDefinitionDialog',
    title: _t('Add Graph Definition'),
    items: [
        {
            xtype: 'idfield',
            id: 'graphDefinitionIdTextfield',
            fieldLabel: _t('Name'),
            allowBlank: false,
            listeners: {
                invalid: function(){
                    Ext.getCmp('addGraphDefinitionSubmit').disable();
                },
                valid: function(){
                    Ext.getCmp('addGraphDefinitionSubmit').enable();
                }
            }
        }
    ],
    listeners: {
        hide: function() {
            Ext.getCmp('graphDefinitionIdTextfield').reset();
            Ext.getCmp('addGraphDefinitionSubmit').disable();
        }
    },
    buttons: [
    {
        xtype: 'HideDialogButton',
        id: 'addGraphDefinitionSubmit',
        text: _t('Submit'),
        disabled: true,
        handler: function(button, event) {
            addGraphDefinition();
        }
    }, {
        xtype: 'HideDialogButton',
        text: _t('Cancel')
    }]
});

deleteGraphDefinition = function() {
    var params, callback;
    params = {
        uid: getSelectedGraphDefinition().id
    };
    callback = function(provider, response) {
        Ext.getCmp('deleteGraphDefinitionButton').disable();
        Ext.getCmp('graphDefinitionMenuButton').disable();
        Ext.getCmp('graphGrid').getStore().reload();
    };
    router.deleteGraphDefinition(params, callback);
};


new Zenoss.MessageDialog({
    id: 'deleteGraphDefinitionDialog',
    title: _t('Delete Graph Definition'),
    // the message is generated dynamically
    okHandler: function(){
        deleteGraphDefinition();
    }
});

/**
 * Deletes the selected graph point
 **/
function deleteGraphPoint() {
    var params, callback;
    params = {
        uid: getSelectedGraphPoint().id
    };
    callback = function(provider, response) {
        Ext.getCmp('deleteGraphPointButton').disable();
        Ext.getCmp('editGraphPointButton').disable();
        Ext.getCmp('graphPointGrid').getStore().reload();
    };
    router.deleteGraphPoint(params, callback);
}

new Zenoss.MessageDialog({
    id: 'deleteGraphPointDialog',
    title: _t('Delete Graph Point'),
    // the message is generated dynamically
    okHandler: deleteGraphPoint
});

/**
 * Adds the selected datapoint as a graph point to our
 * graph definition we are managing
 **/
function addDataPointToGraph() {
    var dataPointUid = Ext.getCmp('addDataPointToGraphDialog').comboBox.getValue(),
        graphUid = getSelectedGraphDefinition().id,
        includeThresholds = Ext.getCmp('addDataPointToGraphDialog').includeRelatedThresholds.getValue(),
        params = {
            dataPointUid: dataPointUid,
            graphUid: graphUid,
            includeThresholds: includeThresholds
        },
        callback = function() {
            Ext.getCmp('graphPointGrid').getStore().reload();
        };
    router.addDataPointToGraph(params, callback);
}

new Zenoss.HideFormDialog({
    id: 'addDataPointToGraphDialog',
    title: _t('Add Data Point'),
    items:[{
        xtype: 'combo',
        ref: 'comboBox',
        fieldLabel: _t('Data Point'),
        valueField: 'uid',
        displayField: 'name',
        triggerAction: 'all',
        selectOnFocus: true,
        forceSelection: true,
        editable: false,
        allowBlank: false,
        listeners: {
            invalid: function(){
                Ext.getCmp('addDataPointToGraphDialog').submit.disable();
            },
            valid: function(){
                Ext.getCmp('addDataPointToGraphDialog').submit.enable();
            }
        },
        store: {
            xtype: 'directstore',
            directFn: router.getDataPoints,
            fields: ['uid', 'name'],
            root: 'data'
        }
    },{
        xtype: 'checkbox',
        name: 'include_related_thresholds',
        ref: 'includeRelatedThresholds',
        fieldLabel: _t('Include Related Thresholds'),
        checked: true
    }],
    listeners: {
        show: function() {
            var combo, uid;
            combo = Ext.getCmp('addDataPointToGraphDialog').comboBox;
            combo.reset();
            Ext.getCmp('addDataPointToGraphDialog').submit.disable();
            uid = getSelectedTemplate().attributes.uid;
            combo.getStore().setBaseParam('query', uid);
            combo.getStore().setBaseParam('uid', uid);
            combo.getStore().load();
        }
    },
    buttons: [
    {
        xtype: 'HideDialogButton',
        ref: '../submit',
        text: _t('Submit'),
        disabled: true,
        handler: function(button, event) {
            addDataPointToGraph();
        }
    }, {
        xtype: 'HideDialogButton',
        text: _t('Cancel')
    }]

});

addThresholdToGraph = function() {
    var params, callback;
    params = {
        graphUid: getSelectedGraphDefinition().id,
        thresholdUid: Ext.getCmp('addThresholdToGraphCombo').getValue()
    };
    callback = function() {
        Ext.getCmp('graphPointGrid').getStore().reload();
    };
    router.addThresholdToGraph(params, callback);
};

new Zenoss.HideFormDialog({
    id: 'addThresholdToGraphDialog',
    title: _t('Add Threshold'),
    items: {
        xtype: 'combo',
        id: 'addThresholdToGraphCombo',
        fieldLabel: _t('Threshold'),
        valueField: 'uid',
        displayField: 'name',
        triggerAction: 'all',
        selectOnFocus: true,
        forceSelection: true,
        editable: false,
        allowBlank: false,
        listeners: {
            invalid: function(){
                Ext.getCmp('addThresholdToGraphSubmit').disable();
            },
            valid: function(){
                Ext.getCmp('addThresholdToGraphSubmit').enable();
            }
        },
        store: {
            xtype: 'directstore',
            directFn: router.getThresholds,
            fields: ['uid', 'name']
        }
    },
    listeners: {
        show: function() {
            var combo, uid;
            combo = Ext.getCmp('addThresholdToGraphCombo');
            combo.reset();
            Ext.getCmp('addThresholdToGraphSubmit').disable();
            uid = getSelectedTemplate().attributes.uid;
            combo.getStore().setBaseParam('uid', uid);
            delete combo.lastQuery;
            combo.doQuery(combo.allQuery, true);
        }
    },
    buttons: [
    {
        xtype: 'HideDialogButton',
        id: 'addThresholdToGraphSubmit',
        text: _t('Submit'),
        disabled: true,
        handler: function(button, event) {
            addThresholdToGraph();
        }
    }, {
        xtype: 'HideDialogButton',
        text: _t('Cancel')
    }]

});

new Zenoss.HideFormDialog({
    id: 'addCustomToGraphDialog',
    title: _t('Add Custom Graph Point'),
    listeners: {
        show: function(dialog) {
            dialog.idField.reset();
            dialog.typeCombo.reset();
            dialog._addButton.disable();
            delete dialog.typeCombo.lastQuery;
            dialog.typeCombo.doQuery(dialog.typeCombo.allQuery, true);
        }
    },
    items: [{
        xtype: 'idfield',
        ref: 'idField',
        fieldLabel: _t('Name'),
        allowBlank: false,
        listeners: {
            invalid: function(idField){
                idField.refOwner._addButton.disable();
            },
            valid: function(idField){
                if ( idField.refOwner.typeCombo.isValid(true) ) {
                    idField.refOwner._addButton.enable();
                }
            }
        }
    }, {
        xtype: 'combo',
        ref: 'typeCombo',
        fieldLabel: _t('Instruction Type'),
        valueField: 'pythonClassName',
        displayField: 'label',
        triggerAction: 'all',
        selectOnFocus: true,
        forceSelection: true,
        editable: false,
        allowBlank: false,
        listeners: {
            invalid: function(typeCombo){
                typeCombo.refOwner._addButton.disable();
            },
            valid: function(typeCombo){
                if ( typeCombo.refOwner.idField.isValid(true) ) {
                    typeCombo.refOwner._addButton.enable();
                }
            }
        },
        store: {
            xtype: 'directstore',
            directFn: router.getGraphInstructionTypes,
            fields: ['pythonClassName', 'label'],
            root: 'data'
        }
    }],
    buttons: [{
        xtype: 'HideDialogButton',
        ref: '../_addButton',
        text: _t('Add'),
        disabled: true,
        handler: function(addButton) {
            var params, callback;
            params = {
                graphUid: getSelectedGraphDefinition().id,
                customId: addButton.refOwner.idField.getValue(),
                customType: addButton.refOwner.typeCombo.getValue()
            };
            callback = function() {
                Ext.getCmp('graphPointGrid').getStore().reload();
            };
            router.addCustomToGraph(params, callback);
        }
    }, {
        xtype: 'HideDialogButton',
        ref: '../cancelButton',
        text: _t('Cancel')
    }]
});

/**********************************************************************
 *
 * Graph Custom Definition
 *
 */
Ext.create({
    xtype: 'window',
    title: _t('Graph Custom Definition'),
    id: 'graphCustomDefinitionDialog',
    closeAction: 'hide',
    buttonAlign: 'left',
    autoScroll: true,
    height: 500,
    width: 400,
    modal: true,
    plain: true,
    padding: 10,
    items: [{
        xtype:'form',
        ref: 'formPanel',
        labelAlign: 'top',
        border: false,
        monitorValid: true,
        paramsAsHash: true,
        api: {
            load: router.getGraphDefinition,
            submit: router.setInfo
        },
        items:[{
            xtype: 'label',
            fieldLabel: _t('Name'),
            name:'id',
            ref: 'nameLabel'
        },{
            xtype: 'textarea',
            fieldLabel: _t('Custom'),
            width: 300,
            height: 300,
            name: 'custom',
            ref: 'custom'
        },{
            xtype: 'label',
            fieldLabel: _t('Available RRD Variables'),
            border: false,
            ref: 'rrdVariables'
        }]
    }],
    buttons: [{
        xtype: 'HideDialogButton',
        text: _t('Submit'),
        handler: function(button, event) {
            var cmp = Ext.getCmp('graphCustomDefinitionDialog'),
                routerCallback,
                data = cmp.record,
                params = {};

            // we just need to update custom
            params.uid = data.uid;
            params.custom = cmp.formPanel.custom.getValue();

            router.setInfo(params);
        }
    }, {
        xtype: 'HideDialogButton',
        text: _t('Cancel')
    }],
    loadAndShow: function(uid) {
        this.uid = uid;
        this.formPanel.getForm().load({
            params: {uid:uid},
            success: function(btn, response) {
                var data = response.result.data;
                this.record = data;
                // populate the form
                this.formPanel.nameLabel.setText(data.id);
                this.formPanel.custom.setValue(data.custom);
                this.formPanel.rrdVariables.setText(data.rrdVariables.join('<br />'), false);

                this.show();
            },
            scope: this
        });
    }

});

Zenoss.GraphPointStore = Ext.extend(Ext.data.DirectStore, {
    constructor: function(config){
        Ext.applyIf(config, {
            directFn: router.getGraphPoints,
            idProperty: 'uid',
            fields: ['uid', 'name', 'type', 'description'],
            root: 'data'
        });
        Zenoss.GraphPointStore.superclass.constructor.call(this, config);
    }
});
Ext.reg('graphpointstore', Zenoss.GraphPointStore);

new Ext.menu.Menu({
    id: 'graphPointMenu',
    items: [{
        xtype: 'menuitem',
        text: _t('Data Point'),
        handler: function(){
            Ext.getCmp('addDataPointToGraphDialog').show();
        }
    }, {
        xtype: 'menuitem',
        text: _t('Threshold'),
        handler: function(){
            Ext.getCmp('addThresholdToGraphDialog').show();
        }
    }, {
        xtype: 'menuitem',
        text: _t('Custom Graph Point'),
        handler: function(){
            Ext.getCmp('addCustomToGraphDialog').show();
        }
    }]
});

Zenoss.BaseSequenceGrid = Ext.extend(Ext.grid.GridPanel, {
    constructor: function(config){
        Ext.applyIf(config, {
            enableDragDrop: Zenoss.Security.hasPermission('Manage DMD'),
            ddGroup: 'sequenceDDGroup'
        });
        this.addEvents({'resequence': true});
        Zenoss.BaseSequenceGrid.superclass.constructor.call(this, config);
    },
    onRender: function() {
        Zenoss.BaseSequenceGrid.superclass.onRender.apply(this, arguments);
        var grid, store;
        grid = this;
        store = this.getStore();
        this.dropZone = new Ext.dd.DropZone(this.view.scroller.dom, {
            ddGroup: 'sequenceDDGroup',
            onContainerOver: function(source, event, data) {
                return this.dropAllowed;
            },
            notifyDrop: function(source, event, data) {
                var sm, rows, cindex, i, rowData;
                sm = grid.getSelectionModel();
                rows = sm.getSelections();
                cindex = source.getDragData(event).rowIndex;
                if (typeof cindex != "undefined") {
                    for (i = 0; i < rows.length; i++) {
                        rowData = store.getById(rows[i].id);
                        store.remove(store.getById(rows[i].id));
                        store.insert(cindex, rowData);
                    }
                    sm.selectRecords(rows);
                    grid.fireEvent('resequence');
                }
            }
        });
    }
});

Zenoss.GraphPointGrid = Ext.extend(Zenoss.BaseSequenceGrid, {
    constructor: function(config){
        Ext.applyIf(config, {
            stripeRows: true,
            autoScroll: true,
            border: false,
            listeners: {
                /**
                 * The selection model was being ignored at this point so I used the
                 * row click.
                 **/
                click: function() {
                    var record = getSelectedGraphPoint();
                    if (record) {
                        Ext.getCmp('deleteGraphPointButton').enable();
                        Ext.getCmp('editGraphPointButton').enable();
                    }else{
                        Ext.getCmp('deleteGraphPointButton').disable();
                        Ext.getCmp('editGraphPointButton').disable();
                    }
                },
                rowdblclick: displayGraphPointForm
            },
            autoExpandColumn: 'description',
            store: {xtype: 'graphpointstore'},
            columns: [
                {dataIndex: 'name', header: _t('Name'), width: 150},
                {dataIndex: 'type', header: _t('Type'), width: 150},
                {dataIndex: 'description', header: _t('Description'), id: 'description'}
            ],
            tbar: [{
                xtype: 'button',
                id: 'addGraphPointButton',
                iconCls: 'add',
                menu: 'graphPointMenu',
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('addGraphPointButton');
                    }
                }
            }, {
                xtype: 'button',
                id: 'deleteGraphPointButton',
                iconCls: 'delete',
                disabled: true,
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('deleteGraphPointButton');
                    }
                },
                handler: function() {
                    var html, dialog;
                    // format the confimation message
                    html = _t("Are you sure you want to remove the graph point, {0}?") + "<br />" +
                        _t("There is no undo.");
                    html = String.format(html, getSelectedGraphPoint().data.name);

                    // show the dialog
                    dialog = Ext.getCmp('deleteGraphPointDialog');
                    dialog.show();
                    dialog.getComponent('message').update(html);
                }
            }, {
                xtype: 'button',
                id: 'editGraphPointButton',
                iconCls: 'customize',
                disabled: true,
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('editGraphPointButton');
                    }
                },
                handler: displayGraphPointForm
            }]
        });
        Zenoss.GraphPointGrid.superclass.constructor.call(this, config);
    }
});
Ext.reg('graphpointgrid', Zenoss.GraphPointGrid);

/**********************************************************************
 *
 * Graph Point Edit Dialog/Grid
 *
 */

function reloadGraphPoints() {
    var grid = Ext.getCmp('graphPointGrid');
    grid.getStore().reload();
}

/**
 * Call back function from when a user selects a graph point.
 * This shows yet another dialog for editing a graph point
 **/
function displayGraphPointForm() {
    var record = getSelectedGraphPoint();

    function displayEditDialog(response) {
        var win = Ext.create( {
            record: response.data,
            items: response.form,
            singleColumn: true,
            width: 400,
            xtype: 'datasourceeditdialog',
            title: _t('Edit Graph Point'),
            directFn: router.setInfo,
            id: 'editGraphPointDialog',
            saveHandler: reloadGraphPoints
        });

        win.show();
    }

    // remote call to get the object details
    router.getInfo({uid: record.id}, displayEditDialog);
}

new Zenoss.HideFitDialog({
    id: 'manageGraphPointsDialog',
    title: _t('Manage Graph Points'),
    items: [{
        xtype: 'graphpointgrid',
        id: 'graphPointGrid',
        ref: 'graphGrid'
    }],
    buttons: [
    {
        xtype: 'HideDialogButton',
        text: _t('Save'),
        handler: function(){
            if (Zenoss.Security.hasPermission('Manage DMD')) {
                var records, uids;
                records = Ext.getCmp('graphPointGrid').getStore().getRange();
                uids = Ext.pluck(records, 'id');
                router.setGraphPointSequence({'uids': uids});
            }
        }
    }, {
        xtype: 'HideDialogButton',
        text: _t('Cancel')
    }]
});

Ext.create({
    xtype: 'window',
    layout: (Ext.isIE) ? 'form': 'fit',
    id: 'viewGraphDefinitionDialog',
    title: _t('View and Edit Graph Definition'),
    closeAction: 'hide',
    buttonAlign: 'left',
    autoScroll: true,
    plain: true,
    modal: true,
    width: 400,
    height: 500,
    padding: 10,
    buttons: [{
        ref: '../submitButton',
        text: _t('Submit'),
        disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
        handler: function(submitButton){
            var dialogWindow, basicForm, params;
            dialogWindow = submitButton.refOwner;
            basicForm = dialogWindow.formPanel.getForm();
            params = Ext.apply(basicForm.getFieldValues(), {
                uid: dialogWindow.uid
            });            
            basicForm.api.submit(params, function() {
                Ext.getCmp('graphGrid').getStore().reload();
            });
            dialogWindow.hide();
        }
    },{
        ref: '../cancelButton',
        text: 'Cancel',
        handler: function(cancelButton){
            var dialogWindow = cancelButton.refOwner;
            dialogWindow.hide();
        }
    }],
    items: {
        xtype: 'form',
        ref: 'formPanel',
        autoScroll: true,
        labelAlign: 'top',
        monitorValid: true,
        border: false,
        paramsAsHash: true,
        api: {
            load: router.getGraphDefinition,
            submit: router.setGraphDefinition
        },
        listeners: {
            clientvalidation: function(formPanel, valid){
                var dialogWindow;
                dialogWindow = formPanel.refOwner;
                if (Zenoss.Security.hasPermission('Manage DMD')) {
                    dialogWindow.submitButton.setDisabled( ! valid );
                }
            },
            show: function(formPanel){
                formPanel.getForm().load();
            }
        },
        items: [{
            xtype: 'idfield',
            fieldLabel: _t('Name'),
            name: 'newId',
            allowBlank: false
        },{
            xtype: 'numberfield',
            fieldLabel: _t('Height'),
            name: 'height',
            minValue: 0
        },{
            xtype: 'numberfield',
            fieldLabel: _t('Width'),
            name: 'width',
            minValue: 0
        },{
            xtype: 'textfield',
            fieldLabel: _t('Units'),
            name: 'units'
        },{
            xtype: 'checkbox',
            fieldLabel: _t('Logarithmic Scale'),
            name: 'log'
        },{
            xtype: 'checkbox',
            fieldLabel: _t('Base 1024'),
            name: 'base'
        },{
            xtype: 'numberfield',
            fieldLabel: _t('Min Y'),
            name: 'miny'
        },{
            xtype: 'numberfield',
            fieldLabel: _t('Max Y'),
            name: 'maxy'
        },{
            xtype: 'checkbox',
            fieldLabel: _t('Has Summary'),
            name: 'hasSummary'
        }]
    },
    loadAndShow: function(uid) {
        this.uid = uid;
        this.formPanel.getForm().load({
            params: {uid:uid},
            success: function() {
                this.show();
            },
            scope: this
        });
    }
});

new Ext.menu.Menu({
    id: 'graphDefinitionMenu',
    items: [{
        xtype: 'menuitem',
        text: _t('Manage Graph Points'),
        handler: function(){
            var uid, store;
            uid = getSelectedGraphDefinition().id;
            Ext.getCmp('manageGraphPointsDialog').show();
            store = Ext.getCmp('graphPointGrid').getStore();
            store.setBaseParam('uid', uid);
            store.load();
        }
    }, {
        xtype: 'menuitem',
        text: _t('View and Edit Details'),
        handler: function(){
            var dialogWindow, uid;
            dialogWindow = Ext.getCmp('viewGraphDefinitionDialog');
            uid = getSelectedGraphDefinition().id;
            dialogWindow.loadAndShow(uid);
        }
    },{
        xtype: 'menuitem',
        text: _t('Custom Graph Definition'),
        handler: function () {
            var win = Ext.getCmp('graphCustomDefinitionDialog'),
                uid = getSelectedGraphDefinition().id;
            win.loadAndShow(uid);
        }
    },{
        xtype: 'menuitem',
        text: _t('Graph Commands'),
        handler: function () {
            var params = {
                uid: getSelectedGraphDefinition().id
            };

            router.getGraphDefinition(params, function (response) {
                Ext.MessageBox.show({
                    title: _t('Graph Commands'),
                    minWidth: 700,
                    msg: String.format('<pre>{0}</pre>', response.data.fakeGraphCommands),
                    buttons: Ext.MessageBox.OK
                });
            });
        }
    }]
});

Zenoss.templates.GraphGrid = Ext.extend(Zenoss.BaseSequenceGrid, {
    constructor: function(config) {
        Ext.applyIf(config, {
            title: _t('Graph Definitions'),
            store: {xtype: 'graphstore'},
            listeners: {
                /**
                 * Double click to edit a graph definition
                 **/
                rowdblclick: function()  {
                    var dialogWindow, uid;
                    dialogWindow = Ext.getCmp('viewGraphDefinitionDialog');
                    uid = getSelectedGraphDefinition().id;
                    dialogWindow.loadAndShow(uid);
                },
                resequence: function() {
                    var records, uids;
                    records = this.store.getRange();
                    uids = Ext.pluck(records, 'id');
                    router.setGraphDefinitionSequence({'uids': uids});
                }
            },
            selModel: new Ext.grid.RowSelectionModel({
                singleSelect: true,
                listeners: {
                    rowdeselect: function() {
                        Ext.getCmp('deleteGraphDefinitionButton').disable();
                        Ext.getCmp('graphDefinitionMenuButton').disable();
                    },
                    rowselect: function() {
                        if (Zenoss.Security.hasPermission('Manage DMD')){
                            Ext.getCmp('deleteGraphDefinitionButton').enable();
                            Ext.getCmp('graphDefinitionMenuButton').enable();
                        }
                    }
                }
            }),
            colModel: new Ext.grid.ColumnModel({
                columns: [{dataIndex: 'name', header: _t('Name'), width: 400}]
            }),
            tbar: [{
                id: 'addGraphDefinitionButton',
                xtype: 'button',
                iconCls: 'add',
                ref: '../addButton',
                disabled: Zenoss.Security.doesNotHavePermission('Manage DMD'),
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('addGraphDefinitionButton');
                    }
                },
                handler: function() {
                    Ext.getCmp('addGraphDefinitionDialog').show();
                }
            }, {
                id: 'deleteGraphDefinitionButton',
                xtype: 'button',
                iconCls: 'delete',
                disabled: true,
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('deleteGraphDefinitionButton');
                    }
                },
                handler: function() {
                    var msg, name, html, dialog;
                    msg = _t("Are you sure you want to remove {0}? There is no undo.");
                    name = getSelectedGraphDefinition().data.name;
                    html = String.format(msg, name);
                    dialog = Ext.getCmp('deleteGraphDefinitionDialog');
                    dialog.show();
                    dialog.getComponent('message').update(html);
                }
            }, {
                id: 'graphDefinitionMenuButton',
                xtype: 'button',
                listeners: {
                    render: function() {
                        Zenoss.registerTooltipFor('graphDefinitionMenuButton');
                    }
                },
                iconCls: 'customize',
                menu: 'graphDefinitionMenu',
                disabled: true
            }]
        });
        Zenoss.templates.GraphGrid.superclass.constructor.call(this, config);
    }
});
Ext.reg('graphgrid', Zenoss.templates.GraphGrid);

})();
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

Ext.ns('Zenoss', 'Zenoss.templates');

var REMOTE = Zenoss.remote.DeviceRouter;

/**
 * Updates the data store for the template tree
 **/

function refreshTemplateTree() {
    var cmp = Ext.getCmp('templateTree');
    if (cmp && cmp.isVisible()) {
        cmp.getRootNode().reload();
    }
}

Zenoss.templates.Container = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        Ext.applyIf(config, {
            layout: 'border',
            border: false,
            defaults: {
                border: false,
                split: true
            },
            items: [{
                xtype: 'DataSourceTreeGrid',
                id: 'dataSourceTreeGrid',
                region: 'center',
                ref: 'dataSourceTreeGrid'
            }, {
                xtype: 'panel',
                layout: 'border',
                region: 'east',
                width: '35%',
                defaults: {
                    border: false,
                    split: true
                },
                items: [{
                    xtype: 'thresholddatagrid',
                    id: 'thresholdGrid',
                    ref: '../thresholdGrid',
                    region: 'north',
                    height: 300
                }, {
                    xtype: 'graphgrid',
                    id: 'graphGrid',
                    ref: '../graphGrid',
                    region: 'center'
                }]
            }]
        });
        Zenoss.templates.Container.superclass.constructor.call(this, config);
    },
    setContext: function(uid){
        this.updateTreeGrid(this.dataSourceTreeGrid, uid);
        this.updateGrid(this.thresholdGrid, uid);
        this.updateGrid(this.graphGrid, uid);
    },
    updateTreeGrid: function(treeGrid, uid){
        treeGrid.getRootNode().setId(uid);
        treeGrid.getRootNode().reload();
    },
    updateGrid: function(grid, uid) {
        grid.getStore().load({
            params: {uid: uid}
        });
    }
});
Ext.reg('templatecontainer', Zenoss.templates.Container);

Zenoss.BubblingSelectionModel = Ext.extend(Ext.tree.DefaultSelectionModel, {
    constructor: function(config) {
        Zenoss.BubblingSelectionModel.superclass.constructor.call(this, config);
        this.enableBubble('selectionchange');
        this.bubbleTarget = config.bubbleTarget;
    },
    getBubbleTarget: function() {
        return this.bubbleTarget;
    }
});

Zenoss.MonTemplateSelectionModel = Ext.extend(Zenoss.BubblingSelectionModel, {
    constructor: function(config) {
        Ext.applyIf(config, {
            listeners: {
                beforeselect: function(sm, node) {
                    return node.isLeaf();
                }
            }
        });
        Zenoss.MonTemplateSelectionModel.superclass.constructor.call(this, config);
    }
});

Zenoss.templates.MonTemplateTreePanel = Ext.extend(Ext.tree.TreePanel, {
    constructor: function(config){
        Ext.applyIf(config, {
            useArrows: true,
            border: false,
            cls: 'x-tree-noicon',
            selModel: new Zenoss.MonTemplateSelectionModel({
                bubbleTarget: config.bubbleTarget
            }),
            loader: {
                directFn: REMOTE.getTemplates,
                baseAttrs: {singleClickExpand: true}
            },
            root: {
                text: _t('Monitoring Templates')
            }
        });
        Zenoss.templates.MonTemplateTreePanel.superclass.constructor.call(this, config);
    },
    setContext: function(uid){
        if ( uid.match('^/zport/dmd/Devices') ) {
            this.show();
            this.setRootNode({
                nodeType: 'async',
                id: uid,
                text: _t('Monitoring Templates'),
                expanded: true
            });
        } else {
            this.hide();
        }
    },
    onSelectionChange: function(node) {
        var detail;
        if (node) {
            detail = Ext.getCmp(this.initialConfig.detailPanelId);
            if ( ! detail.items.containsKey('montemplate') ) {
                detail.add({
                    xtype: 'templatecontainer',
                    id: 'montemplate',
                    ref: 'montemplate'
                });
            }
            detail.montemplate.setContext(node.attributes.uid);
            detail.getLayout().setActiveItem('montemplate');
        }
    }
});
Ext.reg('montemplatetreepanel', Zenoss.templates.MonTemplateTreePanel);

Zenoss.BindTemplatesItemSelector = Ext.extend(Ext.ux.form.ItemSelector, {
        constructor: function(config) {
        Ext.applyIf(config, {
            imagePath: "/++resource++zenui/img/xtheme-zenoss/icon",
            drawUpIcon: false,
            drawDownIcon: false,
            drawTopIcon: false,
            drawBotIcon: false,
            displayField: 'name',
            valueField: 'id',
            multiselects: [{
                cls: 'multiselect-dialog',
                width: 250,
                height: 200,
                displayField: 'name',
                valueField: 'id',
                appendOnly: true,
                store: {
                    xtype: 'arraystore',
                    fields: ['id', 'name']
                }
            },{
                cls: 'multiselect-dialog',
                width: 250,
                height: 200,
                displayField: 'name',
                valueField: 'id',
                appendOnly: true,
                store: {
                    xtype: 'arraystore',
                    fields: ['id', 'name']
                }
            }]
        });
        Zenoss.BindTemplatesItemSelector.superclass.constructor.apply(this, arguments);
    },
    setContext: function(uid) {
        REMOTE.getUnboundTemplates({uid: uid}, function(provider, response){
            this.fromMultiselect.store.loadData(response.result.data);
        }, this);
        REMOTE.getBoundTemplates({uid: uid}, function(provider, response){
            this.toMultiselect.store.loadData(response.result.data);
        }, this);
    }
});
Ext.reg('bindtemplatesitemselector', Zenoss.BindTemplatesItemSelector);

Zenoss.AddLocalTemplatesDialog = Ext.extend(Zenoss.HideFitDialog, {
    constructor: function(config){
        var me = this;
        Ext.applyIf(config, {
            title: _t('Add Local Template'),
            layout: 'form',
            items: [{
                xtype: 'form',
                ref: 'formPanel',
                border: false,
                monitorValid: true,
                listeners: {
                    clientvalidation: function(formPanel, valid) {
                        var dialogWindow = formPanel.refOwner;
                        dialogWindow.submitButton.setDisabled( ! valid );
                    }
                },
                items: [{
                    xtype: 'idfield',
                    fieldLabel: _t('Name'),
                    ref: 'templateName',
                    context: config.context
                }]
            }],
            listeners: {
                show: function() {
                    this.formPanel.templateName.setValue('');
                }

            },
            buttons: [
            {
                xtype: 'HideDialogButton',
                ref: '../submitButton',
                text: _t('Submit'),
                handler: function(){
                    var templateId = me.formPanel.templateName.getValue();

                    REMOTE.addLocalTemplate({
                       deviceUid: me.context,
                       templateId: templateId
                    }, refreshTemplateTree);

                }
            }, {
                xtype: 'HideDialogButton',
                text: _t('Cancel')
            }]
        });
        Zenoss.AddLocalTemplatesDialog.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        this.context = uid;
    }
});
Ext.reg('addlocaltemplatesdialog', Zenoss.AddLocalTemplatesDialog);

Zenoss.BindTemplatesDialog = Ext.extend(Zenoss.HideFitDialog, {
    constructor: function(config){
        var me = this;
        Ext.applyIf(config, {
            title: _t('Bind Templates'),
            items: {
                xtype: 'bindtemplatesitemselector',
                ref: 'itemselector',
                context: config.context
            },
            listeners: {
                show: function() {
                    this.itemselector.setContext(this.context);
                }
            },
            buttons: [
            {
                xtype: 'HideDialogButton',
                text: _t('Save'),
                handler: function(){
                    var records, data, templateIds;
                    if (Zenoss.Security.hasPermission('Manage DMD')) {
                        records = me.itemselector.toMultiselect.store.getRange();
                        data = Ext.pluck(records, 'data');
                        templateIds = Ext.pluck(data, 'id');
                        REMOTE.setBoundTemplates({
                            uid: me.context,
                            templateIds: templateIds
                        }, refreshTemplateTree);
                    }
                }
            }, {
                xtype: 'HideDialogButton',
                text: _t('Cancel')
            }]
        });
        Zenoss.BindTemplatesDialog.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        this.context = uid;
    }
});
Ext.reg('bindtemplatesdialog', Zenoss.BindTemplatesDialog);

Zenoss.ResetTemplatesDialog = Ext.extend(Zenoss.MessageDialog, {
    constructor: function(config) {
        var me = this;
        Ext.applyIf(config, {
            title: _t('Reset Template Bindings'),
            message: _t('Are you sure you want to delete all local template bindings and use default values?'),
            buttons: [
                {
                    xtype: 'HideDialogButton',
                    text: _t('Reset Bindings'),
                    handler: function() {
                        if (Zenoss.Security.hasPermission('Manage DMD')) {
                            REMOTE.resetBoundTemplates(
                                { uid: me.context },
                                refreshTemplateTree);
                        }
                    }
                }, {
                    xtype: 'HideDialogButton',
                    text: _t('Cancel')
                }
            ]
        });
        Zenoss.ResetTemplatesDialog.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        this.context = uid;
    }
});

Ext.reg('resettemplatesdialog', Zenoss.ResetTemplatesDialog);

Zenoss.OverrideTemplatesDialog = Ext.extend(Zenoss.HideFitDialog, {
    constructor: function(config){
        var me = this;
        Ext.applyIf(config, {
            title: _t('Override Templates'),
            listeners: {
                show: function() {
                    // completely reload the combobox every time
                    // we show the dialog
                    me.submit.setDisabled(true);
                    me.comboBox.setValue(null);
                    me.comboBox.store.setBaseParam('query', me.context);
                    me.comboBox.store.setBaseParam('uid', me.context);
                    me.comboBox.store.load();
                }
            },
            items: [{
                xtype: 'label',
                border: false,
                html: _t('Select the bound template you wish to override.')
            },{
                xtype: 'combo',
                forceSelection: true,
                emptyText: _t('Select a template...'),
                minChars: 0,
                ref: 'comboBox',
                selectOnFocus: true,
                typeAhead: true,
                valueField: 'uid',
                displayField: 'label',
                resizable: true,
                store: {
                    xtype: 'directstore',
                    ref:'store',
                    directFn: REMOTE.getOverridableTemplates,
                    fields: ['uid', 'label'],
                    root: 'data'
                },
                listeners: {
                    select: function(){
                        // disable submit if nothing is selected
                        me.submit.setDisabled(!me.comboBox.getValue());
                    }
                }
            }],
            buttons: [
            {
                xtype: 'HideDialogButton',
                ref: '../submit',
                disabled: true,
                text: _t('Submit'),
                handler: function(){
                    var records, data, templateIds;
                    if (Zenoss.Security.hasPermission('Manage DMD')) {
                        var templateUid = me.comboBox.getValue();
                        Zenoss.remote.TemplateRouter.copyTemplate({
                            uid: templateUid,
                            targetUid: me.context
                        }, refreshTemplateTree);
                    }
                }
            }, {
                xtype: 'HideDialogButton',
                text: _t('Cancel')
            }]
        });
        Zenoss.OverrideTemplatesDialog.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        this.context = uid;
    }
});
Ext.reg('overridetemplatesdialog', Zenoss.OverrideTemplatesDialog);

Zenoss.removeLocalTemplateDialog = Ext.extend(Zenoss.HideFitDialog, {
    constructor: function(config){
        var me = this;
        Ext.applyIf(config, {
            title: _t('Remove Local Template'),
            listeners: {
                show: function() {
                    // completely reload the combobox every time
                    // we show the dialog
                    me.submit.setDisabled(true);
                    me.comboBox.setValue(null);
                    me.comboBox.store.setBaseParam('query', me.context);
                    me.comboBox.store.setBaseParam('uid', me.context);
                    me.comboBox.store.load();
                }
            },
            items: [{
                xtype: 'label',
                border: false,
                html: _t('Select the locally defined template you wish to remove.')
            },{
                xtype: 'combo',
                forceSelection: true,
                emptyText: _t('Select a template...'),
                minChars: 0,
                ref: 'comboBox',
                selectOnFocus: true,
                valueField: 'uid',
                displayField: 'label',
                typeAhead: true,
                store: {
                    xtype: 'directstore',
                    ref:'store',
                    directFn: REMOTE.getLocalTemplates,
                    fields: ['uid', 'label'],
                    root: 'data'
                },
                listeners: {
                    select: function(){
                        // disable submit if nothing is selected
                        me.submit.setDisabled(!me.comboBox.getValue());
                    }
                }
            }],
            buttons: [
            {
                xtype: 'HideDialogButton',
                ref: '../submit',
                disabled: true,
                text: _t('Submit'),
                handler: function(){
                    var records, data, templateIds;
                    if (Zenoss.Security.hasPermission('Manage DMD')) {
                        var templateUid = me.comboBox.getValue();
                        REMOTE.removeLocalTemplate({
                            deviceUid: me.context,
                            templateUid: templateUid
                        }, refreshTemplateTree);
                    }
                }
            }, {
                xtype: 'HideDialogButton',
                text: _t('Cancel')
            }]
        });
        Zenoss.OverrideTemplatesDialog.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        this.context = uid;
    }
});
Ext.reg('removelocaltemplatesdialog', Zenoss.removeLocalTemplateDialog);

})();
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2009, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

Ext.ns('Zenoss');
function createClickHandler(bubbleTargetId) {
    return function(button, event) {
        Ext.getCmp(bubbleTargetId).fireEvent('buttonClick', button.id);
    };
}

// options comprises the following:
//      addToZenPack: true puts the Add To ZenPack icon in, default true
//      hasOrganizers: true puts the Add ___  Organizer in, default true
//      customAddDialog: config for a SmartFormDialog to override the default
//      deleteMenu: needs separate menu items for deleting organizers and items
//      contextGetter: fetches the context UIDs for the specific page

Zenoss.footerHelper = function(itemName, footerBar, options) {
    var addToZenPackDialog, items;

    options = Ext.applyIf(options || {}, {
        addToZenPack: true,
        hasOrganizers: true,
        hasContextMenu: true,
        customAddDialog: {},
        buttonContextMenu: {},
        contextGetter: null,
        onGetDeleteMessage: function (itemName) {
            return String.format(_t('The selected {0} will be deleted.'), itemName.toLowerCase());
        },
        onGetAddDialogItems: function () {
            return [{
                xtype: 'textfield',
                name: 'id',
                fieldLabel: _t('Name'),
                allowBlank: false
            }];
        },
        onGetItemName: function() {
            return itemName;
        }
    });

    Ext.applyIf(options.buttonContextMenu, {
        xtype: 'ContextMenu',
        tooltip: _t('Context-sensitive actions'),
        menu: {
            items: []
        },
        ref: 'buttonContextMenu'
    });

    footerBar = footerBar || Ext.getCmp('footer_bar');

    // For now, we will monkey-patch a setContext onto it.
    footerBar.setContext = function(contextUid) {
        Ext.each(this.items.items, function(i) {
            if (i.setContext) { i.setContext(contextUid); }
        });
    };


    function showAddDialog(template, event) {
        var handler, dialog, addDialogConfig;

        // Shallow copy config to avoid mangling the original config
        addDialogConfig = Ext.apply({}, options.customAddDialog);

        handler = function(values) {
            footerBar.fireEvent('buttonClick', event, values.id, values);
        };

        addDialogConfig = Ext.applyIf(addDialogConfig, {
            submitHandler: handler,
            items: options.onGetAddDialogItems(),
            title: String.format(template, options.onGetItemName())
        });

        dialog = new Zenoss.SmartFormDialog(addDialogConfig);
        dialog.show();
    }

    items = [
        {
            xtype: 'FlexButton',
            id: 'footer_add_button',
            iconCls: 'add',
            hidden: Zenoss.Security.doesNotHavePermission('Add DMD Objects'),
            tooltip: _t('Add a child to the selected organizer'),
            menu: {
                items: [
                    {
                        text: String.format(_t('Add {0}'), itemName),
                        listeners: {
                            click: showAddDialog.createCallback(_t('Add {0}'), 'addClass')
                        }
                    }
                ]
            },
            ref: 'buttonAdd'
        },
        {
            xtype: 'FlexButton',
            id: 'footer_delete_button',
            iconCls: 'delete',
            hidden: Zenoss.Security.doesNotHavePermission('Delete objects'),
            tooltip: String.format(_t('Delete {0}'), itemName),
            listeners: {
                click: function() {
                    var itemName = options.onGetItemName();
                    Ext.MessageBox.show({
                        title: String.format(_t('Delete {0}'), itemName),
                        msg: options.onGetDeleteMessage(itemName),
                        fn: function(buttonid){
                            if (buttonid=='ok') {
                                footerBar.fireEvent('buttonClick', 'delete');
                            }
                        },
                        buttons: Ext.MessageBox.OKCANCEL
                    });
                }
            },
            ref: 'buttonDelete'
        }
    ];

    if ( options.hasContextMenu || options.addToZenPack ) {
        items.push(' ');
        items.push(options.buttonContextMenu);
    }

    items.push('-');

    footerBar.add(items);

    if (options.hasOrganizers)
    {
        footerBar.buttonAdd.add({
            text: String.format(_t('Add {0} Organizer'), itemName),
            listeners: {
                click: showAddDialog.createCallback(_t('Add {0} Organizer'), 'addOrganizer')
            },
            ref: 'buttonAddOrganizer'
        });

        footerBar.buttonDelete.add({
            text: String.format(_t('Delete {0} Organizer'), itemName),
            ref: 'buttonDeleteOrganizer',
            listeners: {
                click: function() {
                    var itemName = options.onGetItemName();
                    Ext.MessageBox.show({
                        title: String.format(_t('Delete {0} Organizer'), itemName),
                        msg: String.format(_t('The selected {0} organizer will be deleted.'),
                                itemName.toLowerCase()),
                        fn: function(buttonid){
                            if (buttonid=='ok') {
                                footerBar.fireEvent('buttonClick', 'deleteOrganizer');
                            }
                        },
                        buttons: Ext.MessageBox.OKCANCEL
                    });
                }
            }
        });
    }

    if (options.addToZenPack) {
        addToZenPackDialog = new Zenoss.AddToZenPackWindow();

        footerBar.buttonContextMenu.menu.add({
            ref: 'buttonAddToZenPack',
            text: String.format(_t('Add {0} to ZenPack'), itemName),
            listeners: {
                click: function() {
                    var target = options.contextGetter.getUid();
                    if ( ! target ) {
                        return;
                    }
                    addToZenPackDialog.setTarget(target);
                    addToZenPackDialog.show();
                }
            }
        });

        if ( options.contextGetter.hasTwoControls() ) {

            footerBar.buttonContextMenu.menu.add({
                ref: 'buttonAddOrganizerToZenPack',
                text: String.format(_t('Add {0} Organizer to ZenPack'), itemName),
                listeners: {
                    click: function() {
                        var target = options.contextGetter.getOrganizerUid();
                        if ( ! target ) {
                            return;
                        }
                        addToZenPackDialog.setTarget(target);
                        addToZenPackDialog.show();
                    }
                }
            });

        }

    }

};

})();
(function(){

Ext.ns('Zenoss');

function makeIpAddress(val) {
    var octets = val.split('.');
    if(octets.length>4) 
        return false;
    while(octets.length < 4) {
        octets.push('0');
    }
    for(var i=0;i<octets.length;i++) {
        var octet=parseInt(octets[i], 10);
        if (!octet && octet!==0) return false;
        try {
            if (octet>255) return false;        
        } catch(e) {
            return false;
        }
        octets[i] = octet.toString();
    }
    return octets.join('.');
}

function count(of, s) {
    return of.split(s).length-1;
}

/**
 * @class Zenoss.IpAddressField
 * @extends Ext.form.TextField
 * @constructor
 */
Zenoss.IpAddressField = Ext.extend(Ext.form.TextField, {
    constructor: function(config){
        config.maskRe = true;
        Zenoss.IpAddressField.superclass.constructor.call(this, config);
    },
    filterKeys: function(e, dom) {
        if(e.ctrlKey || e.isSpecialKey()){
            return;
        } 
        e.stopEvent();
        var full, result, newoctet,
            cursor = dom.selectionStart,
            selend = dom.selectionEnd,
            beg = dom.value.substring(0, cursor),
            end = dom.value.substring(selend),
            s = String.fromCharCode(e.getCharCode());
        if (s=='.') {
            result = beg + end;
            cursor += end.indexOf('.');
            newoctet = end.split('.')[1];
            if (selend==cursor+1) 
                cursor++; 
            if(newoctet) 
                dom.setSelectionRange(cursor+1, cursor+newoctet.length+1);
        } else {
            result = makeIpAddress(beg + s + end);
            if (result) {
                cursor++;
                dom.value = result;
                dom.setSelectionRange(cursor, cursor);
            }
        }
    }

}); // Ext.extend

Ext.reg('ipaddressfield', Zenoss.IpAddressField);

})();
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

Zenoss.SlidingCardLayout = Ext.extend(Ext.layout.CardLayout, {
    setActiveItem: function(index) {
        var C = this.container,
            B = C.body,
            card = C.getComponent(index),
            active = this.activeItem,
            activeIndex = C.items.items.indexOf(active);
        if(card!=active) {
            if(active) {
                if(card) {
                    C.fireEvent('beforecardchange', C, card, index, active, activeIndex);
                    if (!card.rendered) {
                        this.renderItem(card, index, C.getLayoutTarget());
                    }
                    card.show();
                    if(card.doLayout && (this.layoutOnCardChange ||
                                         !card.rendered)) {
                        card.doLayout();
                    }
                    var _done = 0;
                    function shiftsCallback() {
                        _done++;
                        if (_done==2)
                            C.fireEvent('cardchange', C, card, index, active,
                                        activeIndex);
                    }
                    var x = B.getX(),
                        w = B.getWidth(),
                        s = [x - w, x + w],
                        cfg = {
                            duration: 0.25,
                            easing: 'easeInStrong',
                            opacity: 0,
                            callback: shiftsCallback
                        };
                    card.el.setY(B.getY());
                    card.el.setX((activeIndex < index ? s[1] : s[0]));
                    active.el.shift(Ext.applyIf({
                        x: activeIndex < index ? s[0] : s[1]
                    }, cfg));
                    card.el.shift(Ext.applyIf({
                        x:x,
                        opacity: 1
                    }, cfg));
                }
            }
            this.activeItem = card;
            this.layout();
        }

    }

});

Ext.Container.LAYOUTS['slide'] = Zenoss.SlidingCardLayout;


Zenoss.HorizontalSlidePanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.headerText = new Ext.Toolbar.TextItem({
            html: (config && config.text) ? config.text : ''
        });
        config = Ext.applyIf(config || {}, {
            cls: 'subselect',
            border: false,
            defaults: {
                border: false
            },
            layout: 'slide',
            activeItem: 0
        });
        var items = [];
        Ext.each(config.items, function(item, index){
            var headerText = new Ext.Toolbar.TextItem({
                html: (item && item.text) ? item.text : ''
            });
            var navButton = new Ext.Button({
                text: (item && item.buttonText) ? item.buttonText : '',
                cls: index ? 'toleft' : 'toright',
                handler: function() {
                    this.layout.setActiveItem(index ? 0 : 1);

                },
                scope: this
            }, this);
            navButton.hide();
            items.push({
                layout: 'fit',
                border: false,
                defaults: {
                    border: false
                },
                tbar: {
                    border: false,
                    cls: 'subselect-head',
                    height: 37,
                    items: [headerText, '->', navButton]
                },
                items:[item],
                listeners: {
                    render: function(card) {
                        card.card = card.items.items[0];
                        card.card.parentCard = card;
                        card.headerText = headerText;
                        card.navButton = navButton;
                        var setHeaderText = function(text, qtip) {
                            this.headerText.setText(text);
                            Ext.QuickTips.unregister(this.headerText);

                            if ( qtip ) {
                                Ext.QuickTips.register({
                                    target: this.headerText,
                                    text: qtip
                                });
                            }
                        };
                        var setButtonText = function(text) {
                            this.navButton.setText(text);
                        };
                        card.setHeaderText = setHeaderText.createDelegate(card);
                        card.setButtonText = setButtonText.createDelegate(card);
                    }
                }
            });
        }, this);
        config.items = items;
        Zenoss.HorizontalSlidePanel.superclass.constructor.call(this, config);
    },
    initEvents: function() {
        this.addEvents('beforecardchange');
        this.addEvents('cardchange');
        Zenoss.HorizontalSlidePanel.superclass.initEvents.call(this);
    }
});

Ext.reg('horizontalslide', Zenoss.HorizontalSlidePanel);

/**
 * A MixedCollection for nav configs.
 *
 * Each item in the config is a normal dictionary with an xtype. The xtype specified will be used for the display
 * when the item is clicked.
 *
 * Example item structure:
 *
 * {
 *     text: 'Menu label',
 *     id: 'View id',
 *     xtype: 'somextype', // optional, defaults to Panel
 *     filterNav: function(navTree) {
 *         // An optional function that is called to determine if this item should be shown
 *         // Use navTree.contextUid for the context.
 *         // return true to display the item or false to remove it
 *     }
 * }
 */
Zenoss.NavConfig = Ext.extend(Ext.util.MixedCollection, {
    constructor: function() {
        Zenoss.NavConfig.superclass.constructor.call(this, true, function(item) {
            return item.id;
        });
    }
});

Zenoss.NavManager = Ext.extend(Object, {
    all: null,
    constructor: function() {
        this.all = new Zenoss.NavConfig();
        this.all.addEvents('navready');
    },
    /**
     * Register a nav item tree. See `Zenoss.NavConfig` for item structure.
     *
     * Zenoss.nav.register({
     *     DeviceGroup: [
     *         {
     *             id: 'device_grid',
     *             text: 'Devices',
     *             listeners: {
     *                 render: updateNavTextWithCount
     *             }
     *         },
     *         {
     *             id: 'events_grid',
     *             text: _t('Events')
     *         }
     *     ]
     * });
     *
     * @param navSpec
     */
    register: function(navSpec) {
        for ( var type in navSpec ) {
            this.add(type, navSpec[type]);
        }
    },
    /**
     * Adds a nav of `type` with its `items`. If a nav of `type` already exists, the items are appended.
     *
     * @param type Type of nav menu to add (ex: Device, DeviceGroup)
     * @param items An array of nav items. See `Zenoss.NavConfig` for item structure.
     */
    add: function(type, items) {
        if ( !this.all.containsKey(type) ) {
            // Create an empty nav container
            var newNav = new Zenoss.NavConfig();
            newNav.id = type;
            this.all.add(type, newNav);

            this.appendTo(type, items);

            this.all.fireEvent('navready', newNav);
        }
        else {
            this.appendTo(type, items);
        }
    },
    /**
     * Get a nav config if it exists.
     *
     * @param type
     */
    get: function(type) {
        return this.all.key(type);
    },
    /**
     * Add menu nodes to the end of a nav config
     *
     * @param type Nav config type
     * @param items Array of items. See `Zenoss.NavConfig` for item structure.
     */
    appendTo: function(type, items) {
        if ( this.all.containsKey(type) ) {
            var nav = this.all.key(type);
            nav.addAll(items);
        }
        else {
            this.onAvailable(type, function(item) {
                item.addAll(items);
            });
        }
    },
    /**
     * Registers a callback to be called as soon as a nav tree matching navId is added.
     *
     * @param navId
     * @param callback
     */
    onAvailable: function(navId, callback, scope) {
        function onAdd(item) {
            if ( item.id == navId ) {
                callback.call(scope || item, item);
                this.all.un('navready', onAdd, scope);
            }
        }

        this.all.on('navready', onAdd, this);
    }
});

Zenoss.nav = new Zenoss.NavManager();

Zenoss.SubselectionNodeUI = Ext.extend(Ext.tree.TreeNodeUI, {
    render: function() {
        Zenoss.SubselectionNodeUI.superclass.render.call(this);
        Ext.removeNode(this.getIconEl());
    }
});

Zenoss.SubselectionNode = Ext.extend(Ext.tree.TreeNode, {
    constructor: function(config) {
        Ext.applyIf(config, {
            leaf: true,
            uiProvider: Zenoss.SubselectionNodeUI
        });
        Zenoss.SubselectionNode.superclass.constructor.call(this, config);
        this.addEvents('render');
    },
    render: function(bulkRender) {
        Zenoss.SubselectionNode.superclass.render.call(this, bulkRender);
        this.fireEvent('render', this);
    }

});

Ext.tree.TreePanel.nodeTypes.subselect = Zenoss.SubselectionNode;


Zenoss.SubselectionPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        var id = config.id || Ext.id();
        Ext.applyIf(config, {
            id: id,
            layout: 'fit',
            border: false,
            bodyStyle: { 'margin-top' : 10 },
            items: [{
                xtype:'treepanel',
                ref: 'treepanel',
                selModel: new Ext.tree.DefaultSelectionModel({
                    listeners: {
                        selectionchange: function(sm, node) {
                            if (node) {
                                var action = node.attributes.action;
                                if (action) {
                                    if (Ext.isString(this.target)) {
                                        this.target = Ext.getCmp(this.target);
                                    }
                                    action.call(node, node, this.target);
                                }
                            }
                        },
                        scope: this
                    }
                }),
                id: 'subselecttreepanel' + id,
                border: false,
                rootVisible: false,
                root : {nodeType: 'node'}            }]
        });
        Zenoss.SubselectionPanel.superclass.constructor.call(this, config);
    },
    initComponent: function() {
        Zenoss.SubselectionPanel.superclass.initComponent.call(this);
        this.treepanel = Ext.getCmp('subselecttreepanel' + this.id);
    },
    setContext: function(uid) {
        var type = Zenoss.types.type(uid),
            nodes = Zenoss.nav.get(type);
        if (nodes) {
            Zenoss.util.each(nodes, function(node) {
                Ext.applyIf(node, {
                    nodeType: 'subselect'
                });
            });
            var root = new Ext.tree.AsyncTreeNode({
                children: nodes,
                listeners: {
                    load: function(node){
                        var toselect = node.firstChild;
                        if (toselect) {
                            if (toselect.rendered) {
                                toselect.select();
                            } else {
                                toselect.on('render', function(node){
                                    node.select();
                                });
                            }
                        }
                    },
                    scope: this
                }
            });
            this.treepanel.setRootNode(root);
        }
    }
});

Zenoss.DetailNavTreePanel = Ext.extend(Ext.tree.TreePanel, {
    constructor: function(config){
        Ext.applyIf(config, {
            useArrows: true,
            autoHeight: true,
            selModel: new Zenoss.BubblingSelectionModel({
                bubbleTarget: config.bubbleTarget
            }),
            id: 'subselecttreepanel' + config.idSuffix,
            ref: 'subselecttreepanel',
            border: false,
            rootVisible: false,
            cls: 'x-tree-noicon',
            root : {nodeType: 'node'}
        });
        Zenoss.DetailNavTreePanel.superclass.constructor.call(this, config);
    }
});
Ext.reg('detailnavtreepanel', Zenoss.DetailNavTreePanel);

Ext.reg('subselection', Zenoss.SubselectionPanel);
/**
 * Used to manage and display detail navigation tree for a contextId
 *
 * @class Zenoss.DetailNavPanel
 * @extends Zenoss.SubselectionPanel
 */
Zenoss.DetailNavPanel = Ext.extend(Zenoss.SubselectionPanel,{
    /**
     * @cfg {function} onGetNavConfig abstract function; hook to provide more nav items
     * @param {string} uid; item to get nav items for
     */
    onGetNavConfig: function(uid){ return []; },
    /**
     * Called when an item in detail nav is selected
     * @param {object} this, the DetailNavPanel
     * @param {object} the navigation node selected
     */
    onSelectionChange: Ext.emptyFn,
    /**
     * Filter out from being in the detail nav; used to filter out nav nodes
     * and the content panels. Return true if it should be kept, false otherwise
     * @param {DetailNavPanel}
     * @param {Object} config
     */
    filterNav: function(detailNav, config) {return true;},
    /**
     * The object id for which the detail navigation belongs to
     */
    contextId: null,
    /**
     * Menu ids used to get non dialog menus to be used in navigation;
     * Empty or null if no nav from old menu items is desired
     */
    menuIds: ['More','Manage','Edit', 'Actions','Add','TopLevel'],
    /**
     * map of nav id to panel configuration
     */
    loaded: false,
    panelConfigMap: null,
    selectFirstNode: true,
    constructor: function(config) {
        Ext.applyIf(config, {
            id: Ext.id(),
            bodyCssClass: 'detailnav',
            layout: 'fit',
            border: false,
            bodyStyle: { 'margin-top' : 10 }
        });
        // call second applyIf so config.id is set correctly
        Ext.applyIf(config, {
            items: {
                xtype: 'detailnavtreepanel',
                ref: 'navtreepanel',
                idSuffix: config.id,
                bubbleTarget: config.bubbleTarget
            }
        });
        Zenoss.DetailNavPanel.superclass.constructor.call(this, config);
    },
    initEvents: function() {
        this.addEvents(
            /**
             * @event navloaded
             * Fires after the navigation has been loaded
             * @param {DetailNavPanel} this The DetailNavPanel
             * @param {AsyncTreeNode} root The root node
             */
             'navloaded',
             /*
             * @event nodeloaded
             * Fires after each navigation node has been loaded
             * @param {DetailNavPanel} this The DetailNavPanel
             * @param {Object} The Navigation config loaded
             */
             'nodeloaded',
            /**
             * @event contextchange
             * Fires after the navigation has been loaded
             * @param {DetailNavPanel} this The DetailNavPanel
             */
            'contextchange'
        );
        Zenoss.DetailNavPanel.superclass.initEvents.call(this);
        if (this.selectFirstNode) {
            this.on('navloaded', this.selectFirst, this);
        }
    },
    setContext: function(uid) {
        //called to load the nav tree
        this.loaded = false;
        this.contextId = uid;
        this.treepanel.setRootNode([]);
        this.getNavConfig(uid);
        this.fireEvent('contextchange', this);
    },
    getSelectionModel: function() {
        return this.treepanel.getSelectionModel();
    },
    getNavConfig: function(uid){
        //Direct call to get nav configs from server
        var me = this;
        var myCallback = function(provider, response) {
            var detailConfigs = response.result.detailConfigs;
            var filterFn = function(val) {
                var show = true;
                if (  Ext.isFunction(val.filterNav) ) {
                    show = val.filterNav(me);
                }

                return show && me.filterNav(me, val);
            };

            detailConfigs = Zenoss.util.filter(detailConfigs, filterFn, me);

            var nodes = me.onGetNavConfig(me.contextId);
            if (!Ext.isDefined(nodes) || nodes === null) {
                nodes = [];
            }

            nodes = Zenoss.util.filter(nodes, filterFn, me);

            if ( detailConfigs ) {
                nodes = nodes.concat(detailConfigs);
            }

            me.panelConfigMap = [];
            Zenoss.util.each(nodes, function(val) {
                me.panelConfigMap[val.id] = val;
            });

            me.setNavTree(nodes);
        };
        var args = {
            'uid': uid
        };
        if (this.menuIds !== null && this.menuIds.length >= 1){
            args['menuIds'] = this.menuIds;
        }
        Zenoss.remote.DetailNavRouter.getDetailNavConfigs(args, myCallback, this);
    },
    reset: function() {
        var root = new Ext.tree.AsyncTreeNode();
        this.treepanel.setRootNode(root);
        return root;
    },
    selectFirst: function(me, root) {
        var sel = root.getOwnerTree().getSelectionModel().getSelectedNode(),
            token = Ext.History.getToken(),
            firstToken = me.id + Ext.History.DELIMITER + root.firstChild.id;
        if (!sel) {
            if (!token || (token && token==firstToken)) {
                root.firstChild.select();
            }
        }
    },
    /**
     * Set the nodes to display in the nav tree.
     * @param nodes Zenoss.NavConfig Nodes to set the nav to
     */
    setNavTree: function(nodes){
        //get any configs registered by the page
        var root;
        if (nodes) {
            root = this.reset();
            Zenoss.util.each(nodes, function(node){
                Ext.applyIf(node, {
                    nodeType: 'subselect'
                });
                root.appendChild(node);
            });

            // Send an alert for all nodes after all nodes have been loaded
            Zenoss.util.each(nodes, function(navConfig){
                this.fireEvent('nodeloaded', this, navConfig);
            }, this);
        }
        this.loaded = true;
        this.fireEvent('navloaded', this, root);
    }
});
Ext.reg('detailnav', Zenoss.DetailNavPanel);


Zenoss.DetailNavCombo = Ext.extend(Ext.form.ComboBox, {
    target: null,
    contextUid: null,
    lastSelItem: null,
    panelConfigMap: null,
    mode: 'local',
    editable: false,
    forceSelection: true,
    typeAhead: false,
    triggerAction: 'all',
    filterNav: function(config) {return true;},
    menuIds: ['More','Manage','Edit', 'Actions','Add','TopLevel'],
    onSelectionChange: Ext.emptyFn,
    onGetNavConfig: function(uid){ return []; },
    getTarget: function() {
        var target = this.target;
        return Ext.isString(target) ? Ext.getCmp(target) : target;
    },
    initEvents: function() {
        Zenoss.DetailNavCombo.superclass.initEvents.call(this);
        this.on('select', this.onItemSelected, this);
    },
    onItemSelected: function(me, item){
        var target = this.getTarget(),
            id = item.id,
            config = this.panelConfigMap[id],
            action = config.action || function(node, target) {
                if (!(id in target.items.map)) {
                    Ext.applyIf(config, {refreshOnContextChange: true});
                    if(config) {
                        target.add(config);
                        target.doLayout();
                    }
                }
                target.items.map[id].setContext(this.contextUid);
                target.layout.setActiveItem(id);
            };
        this.lastSelItem = item;
        action.call(this, item, target, this);
    },
    selectAt: function(idx) {
        var record = this.store.getAt(idx);
        this.lastSelItem = record;
        this.onSelect(record, idx);
    },
    selectByItem: function(item) {
        var idx = 0;
        if (item) {
            idx = this.store.indexOfId(item.id);
            if (idx < 0) {
                idx = 0;
            }
        }
        this.selectAt(idx);
    },
    setContext: function(uid) {
        this.contextUid = uid;
        var args = {uid:uid};
        if (!Ext.isEmpty(this.menuIds)) {
            args.menuIds = this.menuIds;
        }
        Zenoss.remote.DetailNavRouter.getDetailNavConfigs(args, function(r){
            var detailConfigs = r.detailConfigs,
                items = [],
                nodes = [],
                panelMap = {};

            var filterFn = function(val) {
                var show = true;
                if (  Ext.isFunction(val.filterNav) ) {
                    show = val.filterNav(this);
                }

                return show && this.filterNav(val);
            };

            nodes = this.onGetNavConfig(uid);
            nodes = Zenoss.util.filter(nodes, filterFn, this);

            detailConfigs = Zenoss.util.filter(detailConfigs, filterFn, this);
            nodes = nodes.concat(detailConfigs);

            Ext.each(nodes, function(cfg){
                items.push([cfg.id, cfg.text]);
                panelMap[cfg.id] = cfg;
            });

            this.panelConfigMap = panelMap;
            this.store = new Ext.data.ArrayStore({
                'id':0,
                fields: ['value', 'text'],
                data: items,
                autoDestroy: true
            });
            this.valueField = 'value';
            this.displayField = 'text';
            this.list = null;
            this.initList();
            // "sticky" menu selection, show same item as was shown for last context
            this.selectByItem(this.lastSelItem);
        }, this);
    }
});

Ext.reg('detailnavcombo', Zenoss.DetailNavCombo);

Zenoss.DetailContainer = Ext.extend(Ext.Panel, {
    constructor: function(config){
        Ext.applyIf(config, {
            border: false,
            autoScroll: true,
            listeners: {
                selectionchange: this.onSelectionChange,
                scope: this
            }
        });
        Ext.each(config.items, function(item){
            item.bubbleTarget = this;
        }, this);
        Zenoss.DetailContainer.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        this.items.each(function(item){
            item.setContext(uid);
        });
    },
    onSelectionChange: function(eventSelModel, node) {
        var itemSelModel;
        this.items.each(function(item){
            itemSelModel = item.getSelectionModel();
            if ( itemSelModel === eventSelModel ) {
                item.onSelectionChange(node);
            } else {
                if ( itemSelModel.getSelectedNode() ) {
                    itemSelModel.getSelectedNode().unselect(true);
                }
            }
        });
    }
});
Ext.reg('detailcontainer', Zenoss.DetailContainer);

})();
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

Zenoss.IFramePanel = Ext.extend(Ext.BoxComponent, {
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            autoEl: {
                tag: 'iframe',
                id: Ext.id(),
                src: config.url || '',
                frameborder: 0
            }
        });
        Zenoss.IFramePanel.superclass.constructor.call(this, config);
    },
    initEvents: function() {
        Zenoss.IFramePanel.superclass.initEvents.call(this);
        this.addEvents('frameload');
    },
    onRender: function(ct, position) {
        Zenoss.IFramePanel.superclass.onRender.apply(this, arguments);
        // Hook up load events
        this.frame = this.getEl();
        this.frameParent = this.frame.findParent('div.x-panel-body', null, true);
        var evname = Ext.isIE?'onreadystatechange':'onload';
        this.frame.dom[evname] = this.onFrameLoad.createDelegate(this);
    },
    afterRender: function(container) {
        Zenoss.IFramePanel.superclass.afterRender.apply(this, arguments);
        if (!this.ownerCt) {
            var pos = this.getPosition(),
                size = this.frame.parent().getViewSize();
            this.setSize(size.width - pos[0], size.height-pos[1]);
        }
    },
    getWindow: function() {
        return this.frame.dom.contentWindow || window.frames[this.frame.dom.name];
    },
    setSrc: function(url) {
        this.frame.dom.src = url;
    },
    onFrameLoad: function() {
        this.fireEvent('frameload', this.getWindow());
        // Load any messages that may have been created by the frame
        Zenoss.messenger.checkMessages();
    }
});

Ext.reg('iframe', Zenoss.IFramePanel);


Zenoss.ContextualIFrame = Ext.extend(Zenoss.IFramePanel, {
    contextUid: null,
    refreshOnContextChange: false,
    setContext: function(uid) {
        if (this.refreshOnContextChange || this.contextUid!=uid) {
            this.contextUid = uid;
            var url = uid;
            if (Ext.isDefined(this.viewName) && this.viewName !== null) {
                url = uid + '/' + this.viewName;
            }
            this.setSrc(url);
        }
    }
});

Ext.reg('contextiframe', Zenoss.ContextualIFrame);


/**
 * Panel used for displaying old zenoss ui pages in an iframe. Set Context
 * should be called by page to initialze panel for viewing.
 *
 * NOTE: sets a cookie named "newui"; the presence of this cookie will cause the
 * old ui to render with out the old navigation panels and without the tabs.
 *
 * @class Zenoss.BackCompatPanel
 * @extends Zenoss.ContextualIFrame
 */
Zenoss.BackCompatPanel = Ext.extend(Zenoss.ContextualIFrame, {
    setContext: function(uid) {
        Ext.util.Cookies.set('newui', 'yes');
        Zenoss.BackCompatPanel.superclass.setContext.apply(this, arguments);
        this.on('frameload', function(win){
            if (win.document && win.document.body) {
                win.document.body.className = win.document.body.className + ' z-bc';
            } else {
                win.onload = function() {
                    win.document.body.className = win.document.body.className + ' z-bc';
                };
            }
        });
    }
});

Ext.reg('backcompat', Zenoss.BackCompatPanel);



Zenoss.util.registerBackCompatMenu = function(menu, btn, align, offsets){

    align = align || 'bl';
    offsets = offsets || [0, 0];

    var layer = new Ext.Panel({
        floating: true,
        contentEl: menu,
        border: false,
        shadow: !Ext.isIE,
        bodyCssClass: menu.id=='contextmenu_items' ? 'z-bc-z-menu z-bc-page-menu' : 'z-bc-z-menu'
    });

    layer.render(Ext.getBody());

    function showMenu() {
        var xy = layer.getEl().getAlignToXY(btn.getEl(), align, offsets);
        layer.setPagePosition(xy[0], xy[1]);
        menu.dom.style.display = 'block';
        layer.show();
    }

    function hideMenu() {
        layer.hide();
    }

    function menuClicked(e) {
        var link = e.getTarget('a');
        if (link) {
            // Fake a click
            location.href = link.href;
        }
    }

    btn.on('menushow', showMenu);
    btn.on('menuhide', hideMenu);
    menu.on('mousedown', menuClicked);

};

})();
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){
     
/**********************************************************************
 *
 * Command Panel 
 *
 */
var formTpl = new Ext.Template(
    '<form name="commandform" method="POST" action="{target}">',
    '<textarea style="visibility:hidden" name="data">',
    '{data}',
    '</textarea>',
    '</form>');
formTpl.compile();

Zenoss.CommandPanel = Ext.extend(Zenoss.IFramePanel, {
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            autoEl: {
                tag: 'iframe allowtransparency="true"',
                id: Ext.id(),
                src: config.url || '',
                frameborder: 0
            }
        });
        Zenoss.CommandPanel.superclass.constructor.call(this, config);
        this.on('frameload', this.injectForm, this, {single:true});
    },
    injectForm: function(win){
        var doc = win.document,
            form = formTpl.apply({
                data: Ext.encode(this.data), 
                target: this.target
            });
        doc.body.innerHTML = form;
        doc.commandform.submit();
        this.parentWindow.setSize(this.parentWindow.getSize());
    }
});

Ext.reg('commandpanel', Zenoss.CommandPanel);

/**********************************************************************
 *
 *  Command Window
 *
 */  
Zenoss.CommandWindow = Ext.extend(Ext.Window, {
    constructor: function(config) {
        this.cpanel = Ext.id();
        this.commandData = config.data ||
            { uids: config.uids, command: config.command };
        this.target = config.target;
        config = Ext.applyIf(config || {}, {
            layout: 'fit',
            title: config.command || config.title,
            cls: 'streaming-window',
            constrain: true,
            plain: true,
            items: {
                border: false,
                id: this.cpanel,
                // default to command panel
                xtype: config.panel || 'commandpanel',
                data: this.commandData,
                target: config.target,
                autoLoad: config.autoLoad,
                parentWindow: this
            }
        });
        var fbarItems = [];
        if (config.redirectTarget) {
            fbarItems.push({
                xtype: 'button',
                text: _t('Done'),
                handler: function(c){
                    window.location = config.redirectTarget;
                }.createDelegate(this)
            });
        }
        fbarItems.push({
            xtype: 'checkbox',
            checked: true,
            boxLabel: '<span style="color:white">Autoscroll</span>',
            handler: function(c){
                if (c.checked) {
                    this.startScrolling();
                } else {
                    this.stopScrolling();
                }
            }.createDelegate(this)
        });
        config = Ext.applyIf(config, {
            fbar: {
                items: fbarItems
            }
        });
        Zenoss.CommandWindow.superclass.constructor.call(this, config);
        this.task = new Ext.util.DelayedTask(this.scrollToBottom, this);
        this.on('render', this.startScrolling, this);
        this.on('afterlayout', function(){this.center();}, this, {single:true});
    },
    onRender: function() {
        Zenoss.CommandWindow.superclass.onRender.apply(this, arguments);
        var vsize = Ext.getBody().getViewSize();
        this.setSize({width:vsize.width*0.95, height:vsize.height*0.95});
    },
    getCommandPanel: function() {
        if(Ext.isString(this.cpanel)) {
            this.cpanel = Ext.getCmp(this.cpanel);
        }
        return this.cpanel;
    },
    startScrolling: function() {
        this.task.delay(250);
    },
    stopScrolling: function() {
        this.task.cancel();
    },
    scrollToBottom: function() {
        try {
            var win = this.getCommandPanel().getWindow(),
                body = win.document.body;
            Zenoss.env.BODY = body;
            win.scrollBy(0, body.scrollHeight);
        } catch(e) { Ext.emptyFn(); }
        this.task.delay(250);
    },
    show: function() {
        if (Ext.isChrome || Ext.isSafari) {
            var url = 'no_streaming=1&data=';
            url += Ext.encode(this.commandData);
            if (this.commandData.command) {
                url += "&command=";
                url += this.commandDatacommandData.command;
            }
            window.open(this.target + '?'+ url,'',
            'width=800,height=500,toolbar=0,location=0,directories=0,menubar=0,resizable=1,scrollbars=1');
        } else {
            Zenoss.CommandWindow.superclass.show.apply(this)
        }
    }
});

Ext.reg('commandwindow', Zenoss.CommandWindow);

})();
(function(){

    var REMOTE = Zenoss.remote.DeviceRouter;

    var resetCombo = function(combo, manufacturer) {
        combo.clearValue();
        combo.getStore().setBaseParam('manufacturer', manufacturer);
        delete combo.lastQuery;
        //combo.doQuery(combo.allQuery, true);
    };

    var ClickToEditField = Ext.extend(Zenoss.form.LinkField, {
        constructor: function(config) {
            var editLink = '<a href="javascript:" class="manu-edit-link">'+
                           _t('Edit') + '</a>';
            config.fieldLabel += editLink;
            config.listeners = Ext.apply(config.listeners||{}, {
                render: function(p) {
                    p.editlink = p.label.select('a.manu-edit-link');
                    p.editlink.on('click', function(){
                        p.fireEvent('labelclick', p);
                    }, p);
                }
            });
            ClickToEditField.superclass.constructor.call(this, config);
            this.addEvents('labelclick');
        }
    });
    Ext.reg('clicktoedit', ClickToEditField);

    function editManuInfo (vals, uid) {
        function name(uid) {
            if (!uid){
                return 'Unknown';
            }
            if (!Ext.isString(uid)) {
                uid = uid.uid;
            }
            return uid.split('/').reverse()[0];
        }

        var FIELDWIDTH = 300;

        var hwManufacturers = {
            xtype: 'manufacturercombo',
            width: FIELDWIDTH,
            name: 'hwManufacturer',
            fieldLabel: _t('HW Manufacturer'),
            value: name(vals.hwManufacturer),
            listeners: {'select': function(combo, record, index){
                var productCombo = Ext.getCmp('hwproductcombo');
                resetCombo(productCombo, record.data.name);
            }}
        };

        var hwProduct = {
            xtype: 'productcombo',
            prodType: 'HW',
            width: FIELDWIDTH,
            value: name(vals.hwModel),
            resizable: true,
            name: 'hwProductName',
            fieldLabel: _t('HW Product'),
            id: 'hwproductcombo'
        };

        var osManufacturers = {
            xtype: 'manufacturercombo',
            width: FIELDWIDTH,
            name: 'osManufacturer',
            value: name(vals.osManufacturer),
            fieldLabel: _t('OS Manufacturer'),
            listeners: {'select': function(combo, record, index){
                var productCombo = Ext.getCmp('osproductcombo');
                resetCombo(productCombo, record.data.name);
            }}
        };

        var osProduct = {
            xtype: 'productcombo',
            prodType: 'OS',
            width: FIELDWIDTH,
            value: name(vals.osModel),
            resizable: true,
            name: 'osProductName',
            id: 'osproductcombo',
            fieldLabel: _t('OS Product')
        };

        var win = new Zenoss.FormDialog({
            autoHeight: true,
            width: 390,
            title: _t('Edit Manufacturer Info'),
            items: [{
                xtype: 'container',
                layout: 'form',
                autoHeight: true,
                style: 'padding-bottom:5px;margin-bottom:5px;border-bottom:1px solid #555;',
                items: [hwManufacturers, hwProduct]
            },{
                xtype: 'container',
                layout: 'form',
                autoHeight: true,
                items: [osManufacturers, osProduct]
            }],
            buttons: [{
                text: _t('Save'),
                ref: '../savebtn',
                disabled: Zenoss.Security.doesNotHavePermission('Manage Device'),
                handler: function(btn){
                    var vals = btn.refOwner.editForm.getForm().getFieldValues();
                    Ext.apply(vals, {uid:uid});
                    REMOTE.setProductInfo(vals, function(r) {
                        Ext.getCmp('device_overview').load();
                        win.destroy();
                    });
                }
            },{
                text: _t('Cancel'),
                handler: function(btn){
                    win.destroy();
                }
            }]
        });
        win.show();
        win.doLayout();
    }


    function isField(c) {
        return !!c.setValue && !!c.getValue && !!c.markInvalid && !!c.clearInvalid;
    }

    Zenoss.DeviceOverviewForm = Ext.extend(Ext.form.FormPanel, {
        labelAlign: 'top',
        paramsAsHash: true,
        frame: true,
        defaults: {
            labelStyle: 'font-size: 13px; color: #5a5a5a',
            anchor: '100%'
        },
        buttonAlign: 'left',
        buttons: [{
            text: _t('Save'),
            ref: '../savebtn',
            disabled: true,
            hidden: true,
            handler: function(btn){
                this.refOwner.getForm().submit();
            }
        },{
            text: _t('Cancel'),
            ref: '../cancelbtn',
            disabled: true,
            hidden: true,
            handler: function() {
                this.refOwner.getForm().reset();
            }
        }],
        cls: 'device-overview-form-wrapper',
        bodyCssClass: 'device-overview-form',
        listeners: {
            'add': function(me, field, index){
                if (isField(field)) {
                    this.onFieldAdd.call(this, field);
                }
            }
        },
        constructor: function(config) {
            config = Ext.applyIf(config || {}, {
                trackResetOnLoad: true
            });
            config.listeners = Ext.applyIf(config.listeners||{}, this.listeners);
            Zenoss.DeviceOverviewForm.superclass.constructor.call(this, config);
        },
        showButtons: function() {
            if (!this.rendered) {
                this.on('render', this.showButtons, this);
            } else {
                this.savebtn.show();
                this.cancelbtn.show();
            }
        },
        doButtons: function() {
            this.setButtonsDisabled(!this.form.isDirty());
        },
        setButtonsDisabled: function(b) {
            if (Zenoss.Security.hasPermission('Manage Device')) {
                this.savebtn.setDisabled(b);
            }
            this.cancelbtn.setDisabled(b);  
        },
        onFieldAdd: function(field) {
            if (!field.isXType('displayfield')) {
                this.showButtons();
                this.mon(field, 'valid', this.doButtons, this);
            }
        },
        hideFooter: function() {
            this.footer.hide();
        },
        showFooter: function() {
            this.footer.show();
        },
        addField: function(field) {
            this.items.push( field );
        },
        addFieldAfter: function(field, afterFieldName) {
            this.items.splice(this._indexOfFieldName(afterFieldName)+1, 0, field);
        },
        _indexOfFieldName: function(name) {
            var idx = -1
            for ( i = 0; i < this.items.length; i++ ){
                if (this.items[i].name == name){
                    idx = i
                    break
                }
            }
        return idx
        },
        replaceField: function(name,field) {
            idx = this._indexOfFieldName(name)
            this.items[idx] = field
        },
        removeField: function(name) {
            idx = this._indexOfFieldName(name)
            this.items.splice(idx,1)
        },
        getField: function(name) {
            return this.items[this._indexOfFieldName(name)]
        }

    });

    Ext.reg('devformpanel', Zenoss.DeviceOverviewForm);

    Zenoss.DeviceOverviewPanel = Ext.extend(Ext.Panel, {
        constructor: function(config) {
            config = Ext.applyIf(config||{}, {
                autoScroll: true,
                bodyCssClass: 'device-overview-panel',
                padding: '10',
                border: false,
                frame: false,
                defaults: {
                    border: false
                },
                forms: [],
                listeners: {
                    add: function(item) {
                        if (item.isXType('form')) {
                            var f = item.getForm();
                            f.api = this.api;
                            f.baseParams = this.baseParams;
                            this.forms.push(item);
                        }
                    }
                },
                items: [{
                    layout: 'hbox',
                    defaults: {
                        flex: 1
                    },
                    layoutConfig: {
                        align: 'stretchmax',
                        defaultMargins: '10'
                    },
                    defaultType: 'devformpanel',
                    items: [{
                        id:'deviceoverviewpanel_summary',
                        defaultType: 'displayfield',
                        items: [{
                            fieldLabel: _t('Uptime'),
                            name: 'uptime'
                        },{
                            fieldLabel: _t('First Seen'),
                            name: 'firstSeen'
                        },{
                            fieldLabel: _t('Last Change'),
                            name: 'lastChanged'
                        },{
                            fieldLabel: _t('Model Time'),
                            name: 'lastCollected'
                        },{
                            fieldLabel: _t('Locking'),
                            name: 'locking'
                        },{
                            xtype: 'displayfield',
                            name: 'memory',
                            fieldLabel: _t('Memory/Swap')
                        }]
                    },{
                        id:'deviceoverviewpanel_idsummary',
                        defaultType: 'displayfield',
                        autoHeight: true,
                        listeners: {
                            actioncomplete: function(form, action) {
                                if (action.type=='directsubmit') {
                                    var bar = Ext.getCmp('devdetailbar');
                                    if (bar) {
                                        bar.refresh();
                                    }
                                }
                            }
                        },
                        items: [{
                            xtype: 'textfield',
                            name: 'name',
                            fieldLabel: _t('Device Name')
                        },{
                            xtype: 'ProductionStateCombo',
                            fieldLabel: _t('Production State'),
                            name: 'productionState'
                        },{
                            xtype: 'PriorityCombo',
                            fieldLabel: _t('Priority'),
                            name: 'priority'
                        },{
                            fieldLabel: _t('Collector'),
                            name: 'collector'
                        },{
                            xtype: 'linkfield',
                            fieldLabel: _t('Systems'),
                            name: 'systems'
                        },{
                            xtype: 'linkfield',
                            fieldLabel: _t('Groups'),
                            name: 'groups'
                        },{
                            xtype: 'linkfield',
                            fieldLabel: _t('Location'),
                            name: 'location'
                        }]
                    },{
                        id:'deviceoverviewpanel_descriptionsummary',
                        defaultType: 'textfield',
                        items: [{
                            fieldLabel: _t('Tag'),
                            name: 'tagNumber'
                        },{
                            fieldLabel: _t('Serial Number'),
                            name: 'serialNumber'
                        },{
                            fieldLabel: _t('Rack Slot'),
                            name: 'rackSlot'
                        },{
                            xtype: 'clicktoedit',
                            listeners: {
                                labelclick: function(p){
                                    editManuInfo(this.getValues(), this.contextUid);
                                },
                                scope: this
                            },
                            name: 'hwManufacturer',
                            fieldLabel: _t('Hardware Manufacturer')
                        },{
                            xtype: 'clicktoedit',
                            listeners: {
                                labelclick: function(p){
                                    editManuInfo(this.getValues(), this.contextUid);
                                },
                                scope: this
                            },
                            name: 'hwModel',
                            fieldLabel: _t('Hardware Model')
                        },{
                            xtype: 'clicktoedit',
                            listeners: {
                                labelclick: function(p){
                                    editManuInfo(this.getValues(), this.contextUid);
                                },
                                scope: this
                            },
                            name: 'osManufacturer',
                            fieldLabel: _t('OS Manufacturer')
                        },{
                            xtype: 'clicktoedit',
                            listeners: {
                                labelclick: function(p){
                                    editManuInfo(this.getValues(), this.contextUid);
                                },
                                scope: this
                            },
                            name: 'osModel',
                            fieldLabel: _t('OS Model') 
                        }]
                    }]
                },{
                    id:'deviceoverviewpanel_customsummary',
                    defaultType: 'devformpanel',
                    autoHeight: true,
                    layout: 'hbox',
                    layoutConfig: {
                        align: 'stretchmax',
                        defaultMargins: '10'
                    },
                    items: [{
                        defaultType: 'displayfield',
                        flex: 2,
                        items: [{
                            fieldLabel: _t('Links'),
                            name: 'links'
                        },{
                            xtype: 'textarea',
                            grow: true,
                            fieldLabel: _t('Comments'),
                            name: 'comments'
                        }]
                    },{
                    id:'deviceoverviewpanel_snmpsummary',
                        defaultType: 'displayfield',
                        flex: 1,
                        minHeight: 230,
                        items: [{
                            fieldLabel: _t('SNMP SysName'),
                            name: 'snmpSysName'
                        },{
                            fieldLabel: _t('SNMP Location'),
                            name: 'snmpLocation'
                        },{
                            fieldLabel: _t('SNMP Contact'),
                            name: 'snmpContact'
                        },{
                            fieldLabel: _t('SNMP Agent'),
                            name: 'snmpAgent'
                        }]
                    }]
                }]
            });
            Zenoss.DeviceOverviewPanel.superclass.constructor.call(this, config);
        },
        api: {
            load: REMOTE.getInfo,
            submit: function(form, success, scope) {
                var o = {},
                    vals = scope.form.getFieldValues(true);
                Ext.apply(o, vals, success.params);
                REMOTE.setInfo(o, function(result){
                    this.form.clearInvalid();
                    this.form.setValues(vals);
                    this.form.afterAction(this, true);
                    this.form.reset();
                }, scope);
            }
        },
        baseParams: {},
        setContext: function(uid) {
            this.contextUid = uid;
            this.baseParams.uid = uid;
            this.load();
        },
        getFieldNames: function() {
            var keys = [];
            Ext.each(this.forms, function(f){
                for (var key in f.getForm().getFieldValues(false)) {
                    if (keys.indexOf(key)==-1) {
                        keys.push(key);
                    }
                }
            });
            return keys;
        },
        load: function() {
            var o = Ext.apply({keys:this.getFieldNames()}, this.baseParams);
            this.api.load(o, function(result) {
                var systems = [], groups = [], D = result.data;
                if (D.locking) {
                    D.locking = Zenoss.render.locking(D.locking);
                }
                if (D.memory) {
                    D.memory = D.memory.ram + '/' + D.memory.swap;
                } else {
                    D.memory = 'Unknown/Unknown';
                }
                this.setValues(D);
                this.doLayout();
            }, this);
        },
        getValues: function() {
            var o = {};
            Ext.each(this.forms, function(form){
                Ext.apply(o, form.getForm().getFieldValues());
            }, this);
            return o;
        },
        setValues: function(d) {
            Ext.each(this.forms, function(form){
                form.getForm().setValues(d);
            });
        }
    });

    Ext.reg('deviceoverview', Zenoss.DeviceOverviewPanel);

})();
(function(){

Zenoss.DeviceDetailItem = Ext.extend(Ext.Container, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            hideParent: true,
            cls: 'devdetailitem',
            items: [{
                cls: 'devdetail-textitem ' + (config.textCls||''),
                ref: 'textitem',
                xtype: 'tbtext',
                text: config.text || ''
            },{
                cls: 'devdetail-labelitem ' + (config.labelCls||''),
                ref: 'labelitem',
                xtype: 'tbtext',
                text: config.label || ''
            }]
        });
        Zenoss.DeviceDetailItem.superclass.constructor.call(this, config);
    },
    setText: function(t) {
        this.textitem.setText(t);
    },
    setLabel: function(t) {
        this.labelitem.setText(t);
    }
});

Ext.reg('devdetailitem', Zenoss.DeviceDetailItem);

Zenoss.DeviceNameItem = Ext.extend(Ext.Container, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            //layout: 'vbox',
            defaults: {
                xtype: 'tbtext'
            },
            items: [{
                cls: 'devdetail-devname',
                ref: 'devname'
            },{
                ref: 'devclass',
                cls: 'devdetail-devclass'
            },{
                ref: 'ipAddress',
                cls: 'devdetail-ipaddress'
            }]
        });
        Zenoss.DeviceNameItem.superclass.constructor.call(this, config);
    }
});
Ext.reg('devnameitem', Zenoss.DeviceNameItem);


Zenoss.DeviceDetailBar = Ext.extend(Zenoss.LargeToolbar, {
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            cls: 'largetoolbar devdetailbar',
            height: 55,
            directFn: Zenoss.remote.DeviceRouter.getInfo,
            defaultType: 'devdetailitem',
            items: [{
                ref: 'iconitem',
                cls: 'devdetail-icon'
            },{
                cls: 'evdetail-sep',
                style: 'margin-right:4px;'
            },{
                xtype: 'devnameitem',
                ref: 'deviditem',
                style: 'margin-right: 8px;'
            },'-',{
                xtype: "eventrainbow",
                ref: 'eventsitem',
                label: _t('Events'),
                count: 4
            },'-',{
                ref: 'statusitem',
                label: _t('Device Status')
            },'-',{
                ref: 'prodstateitem',
                label: _t('Production State')
            }]  
        });
        Zenoss.DeviceDetailBar.superclass.constructor.call(this, config);
    },
    contextCallbacks: [],
    addDeviceDetailBarItem: function(item, fn) {
      this.items.push('-');
      this.items.push(item);
      this.on('contextchange',fn,this);
    },
    refresh: function() {
        this.setContext(this.contextUid);
    },
    setContext: function(uid) {
        this.contextUid = uid;
        this.directFn({uid:uid}, function(result){
            var ZR = Zenoss.render,
                data = result.data;
            Zenoss.env.icon = this.iconitem;
            this.iconitem.getEl().setStyle({
                'background-image' : 'url(' + data.icon + ')'
            });
            this.deviditem.devname.setText(data.name);
            var ipAddress = ZR.ipAddress(data.ipAddress);
            this.deviditem.ipAddress.setHeight(Ext.isEmpty(ipAddress) ? 0 : 'auto');
            this.deviditem.ipAddress.setText(ipAddress);
            this.deviditem.devclass.setText(ZR.DeviceClass(data.deviceClass.uid));
            this.eventsitem.updateRainbow(data.events);
            this.statusitem.setText(
                ZR.pingStatusLarge(data.status));
            this.prodstateitem.setText(data.productionState);
            this.fireEvent('contextchange', this, data);
        }, this);
    }
});

Ext.reg('devdetailbar', Zenoss.DeviceDetailBar);

})();
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */
(function(){

Ext.ns('Zenoss');

Zenoss.ConsoleBar = Ext.extend(Zenoss.LargeToolbar, {
    constructor: function(config) {
        var title = config.title || 'Title';
        delete config.title;
        config = Ext.apply(config||{}, {
            cls: 'largetoolbar consolebar',
            height: 35,
            items: [{
                xtype: 'tbtext',
                text: title
            }].concat(config.leftItems||[]).concat([{
                xtype: 'tbfill'
            }]).concat(config.items||[]).concat(['-',{
                iconCls: 'collapse',
                ref: 'togglebutton',
                handler: function() {
                    this.ownerCt.ownerCt.toggleCollapse();
                }
            }])
        });
        if (config.collapsed) {
            config.iconCls = 'expand';
        }
        Zenoss.ConsoleBar.superclass.constructor.call(this, config);
        var panel = this.ownerCt;
        // Set the icons properly
        panel.on('collapse', function(p) {
            this.togglebutton.setIconClass('expand');
        }, this);
        panel.on('expand', function(p) {
            this.togglebutton.setIconClass('collapse');
        }, this);
        /*
        * Have the toolbar remain visible when collapsed. This is accomplished
        * by physically moving the element into the collapsedEl of the region
        * on collapse, and moving it back before expanding.
        */
        panel.on('collapse', function(p) {
            var region = p.layout.container.ownerCt.layout[p.region],
                collapsedEl = region.getCollapsedEl(),
                tbEl = p.topToolbar.getEl(),
                tbHeight = tbEl.getComputedHeight();
            p.tbParent = tbEl.parent();
            collapsedEl.insertFirst(tbEl);
            collapsedEl.setHeight(tbHeight);
            p.layout.container.ownerCt.doLayout();
        });
        panel.on('beforeexpand', function(p) {
            var tbEl = p.topToolbar.getEl();
            p.tbParent.insertFirst(tbEl);
        });
        /*
        * Force the region to be unfloatable by detaching the listener. This
        * avoids a few problems by disabling a feature we probably won't ever
        * use.
        */
        panel.on('afterlayout', function(p) {
            var region = p.layout.container.ownerCt.layout[p.region];
            if (region && region.floatable) {
                region.getCollapsedEl().un('click', region.collapseClick,
                                           region);
            }
        });
    }
});

Ext.reg('consolebar', Zenoss.ConsoleBar);


})(); // End local namespace
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

Ext.ns('Zenoss');

/**
 * @class Zenoss.AddToZenPackWindow
 * @extends Ext.Window
 */
Zenoss.AddToZenPackWindow = Ext.extend(Ext.Window, {
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            id: 'addToZenPackWindow',
            title: _t('Add to Zen Pack'),
            layout: 'fit',
            modal: true,
            autoHeight: true,
            width: 310,
            closeAction: 'hide',
            plain: true,
            items: [{
                id: 'addzenpackform',
                xtype: 'form',
                monitorValid: true,
                defaults: {width: 180},
                autoHeight: true,
                border: false,
                frame: false,
                labelWidth: 100,
                labelAlign: 'top',
                buttonAlign: 'left',
                items: [{
                    fieldLabel: _t('Zen Pack'),
                    name: 'zpname',
                    xtype: 'combo',
                    emptyText: _t('Please select a zenpack...'),
                    listEmptyText: _t('No zenpacks available'),
                    allowBlank: false,
                    store: new Ext.data.DirectStore({
                        id: 'myzpstore',
                        fields: ['name'],
                        root: 'packs',
                        totalProperty: 'totalCount',
                        directFn:
                            Zenoss.remote.ZenPackRouter.getEligiblePacks
                    }),
                    valueField: 'name',
                    displayField: 'name',
                    forceSelection: true,
                    triggerAction: 'all',
                    selectOnFocus: true,
                    id: 'zpcombobox'
                }],
                buttons: [{
                    text: _t('Submit'),
                    formBind: true,
                    handler: function () {
                        var form;
                        form = Ext.getCmp('addzenpackform');
                        var chosenzenpack =
                            form.getForm().findField('zpname').getValue();
                            Zenoss.remote.ZenPackRouter.addToZenPack({
                                topack: Ext.getCmp('addToZenPackWindow').target,
                                zenpack: chosenzenpack
                            },
                            function (data) {
                                Ext.getCmp('addToZenPackWindow').hide();
                                Zenoss.message.info(_t("The item was added to the zenpack, {0}"), chosenzenpack);
                            }
                        );
                    }
                },{
                    text: _t('Cancel'),
                    handler: function () {
                        Ext.getCmp('addToZenPackWindow').hide();
                    }
                }]
            }]
        });
        Zenoss.AddToZenPackWindow.superclass.constructor.call(this, config);
    },
    setTarget: function (target) {
        this.target = target;
    }
});

Ext.reg('AddToZenPackWindow', Zenoss.AddToZenPackWindow);

})();
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

Ext.ns('Zenoss');

Zenoss.VerticalBrowsePanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            layout: 'hbox',
            border: false,
            layoutConfig: {
                align: 'stretch'
            },
            defaults: {
                flex: 1,
                autoScroll: true
            }
        });
        Zenoss.VerticalBrowsePanel.superclass.constructor.call(this, config);
    }
});

})();
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/

(function(){

var ZC = Ext.ns('Zenoss.component'),
    ZEvActions = Zenoss.events.EventPanelToolbarActions,
    NM = ZC.nameMap = {};

ZC.registerName = function(meta_type, name, plural) {
    NM[meta_type] = [name, plural];
}

ZC.displayName = function(meta_type) {
    return NM[meta_type] || [meta_type, meta_type];
}

function componentColumnDefinitions() {
    var defs = Zenoss.env.COLUMN_DEFINITIONS,
        bad = ['component', 'device'];
    return Zenoss.util.filter(defs, function(d){
        return bad.indexOf(d.id)==-1;
    });
}

function render_link(ob) {
    if (ob && ob.uid) {
        return Zenoss.render.link(ob.uid);
    } else {
        return ob;
    }
}

Zenoss.nav.register({
    Component: [{
        nodeType: 'subselect',
        id: 'Graphs',
        text: _t('Graphs'),
        action: function(node, target, combo) {
            var uid = combo.contextUid,
                cardid = uid+'_graphs',
                graphs = {
                    id: cardid,
                    xtype: 'backcompat',
                    viewName: 'graphs',
                    text: _t('Graphs')
                };
            if (!(cardid in target.items.keys)) {
                target.add(graphs);
            }
            target.layout.setActiveItem(cardid);
            target.layout.activeItem.setContext(uid);
        }
    },{
        nodeType: 'subselect',
        id: 'Events',
        text: _t('Events'),
        action: function(node, target, combo) {
            var uid = combo.contextUid,
                cardid = uid + '_events',
                showPanel = function() {
                    target.layout.setActiveItem(cardid);
                    target.layout.activeItem.setContext(uid);
                };
            if (!(cardid in target.items.keys)) {
                var panel = target.add({
                    id: cardid,
                    xtype: 'SimpleEventGridPanel',
                    stateful: false,
                    columns: componentColumnDefinitions()
                });
                var tbar = target.getTopToolbar();
                if (tbar._btns) {
                    Ext.each(tbar._btns, tbar.remove, tbar);
                }
                var btns = tbar.add([
                    '-',
                    ZEvActions.acknowledge,
                    ZEvActions.close,
                    ZEvActions.newwindow,
                    ZEvActions.refresh
                ]);
                Ext.each(btns, function(b){b.grid = panel;});
                tbar.doLayout();
                tbar._btns = btns;
                combo.on('select', function(c, selected){
                    if (selected.id!="Events") {
                        Ext.each(btns, tbar.remove, tbar);
                    }
                }, this, {single:true});
            }
            showPanel();
        }
    },{
        nodeType: 'subselect',
        id: 'Edit',
        text: _t('Details'),
        action: function(node, target, combo) {
            var uid = combo.contextUid;
            if (!(uid in target.items.keys)) {
                Zenoss.form.getGeneratedForm(uid, function(config){
                    target.add(Ext.apply({id:uid}, config));
                    target.layout.setActiveItem(uid);
                });
            } else {
                target.layout.setActiveItem(uid);
            }
        }
    }]
});

ZC.ComponentDetailNav = Ext.extend(Zenoss.DetailNavPanel, {
    constructor: function(config) {
        Ext.applyIf(config, {
            autoHeight: true,
            autoScroll: true,
            containerScroll: true,
            border: false,
            menuIds: []
        });
        ZC.ComponentDetailNav.superclass.constructor.call(this, config);
        this.relayEvents(this.getSelectionModel(), ['selectionchange']);
        this.on('selectionchange', this.onSelectionChange);
    },
    onGetNavConfig: function(contextId) {
        var grid = this.ownerCt.ownerCt.ownerCt.componentgrid,
            items = [],
            monitor = false;
        Zenoss.env.GRID = grid;
        Ext.each(grid.store.data.items, function(record){
            if (record.data.monitor) { monitor = true; }
        });
        Zenoss.util.each(Zenoss.nav.get('Component'), function(item){
            if (!(item.id=='Graphs' && !monitor)) {
                items.push(item);
            }
        });
        return items;
    },
    filterNav: function(navpanel, config){
        //nav items to be excluded
        var excluded = [
            'status',
            'events',
            'resetcommunity',
            'pushconfig',
            'objtemplates',
            'modeldevice',
            'historyevents'
        ];
        return (excluded.indexOf(config.id)==-1);
    },
    onSelectionChange: function(sm, node) {
        var target = this.target || Ext.getCmp('component_detail_panel'),
            action = node.attributes.action || function(node, target) {
                var id = node.attributes.id;
                if (!(id in target.items.map)) {
                    var config = this.panelConfigMap[id];
                    Ext.applyIf(config, {refreshOnContextChange: true});
                    if(config) {
                        target.add(config);
                        target.doLayout();
                    }
                }
                target.items.map[node.attributes.id].setContext(this.contextId);
                target.layout.setActiveItem(node.attributes.id);
            }.createDelegate(this);
        action(node, target);
    }
});
Ext.reg('componentnav', ZC.ComponentDetailNav);


ZC.ComponentPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        var tbar = config.gridtbar,
            tbarid = Ext.id();
        if (tbar) {
            if (Ext.isArray(tbar)) {
                tbar = {items:tbar};
            }
            Ext.apply(tbar, {
                id: tbarid
            });
        }
        config = Ext.applyIf(config||{}, {
            tbarid: tbarid,
            border: false,
            defaults: {border: false},
            layout: 'border',
            items: [{
                region: 'north',
                height: 250,
                defaults: {border: false},
                border: false,
                split: true,
                ref: 'gridcontainer',
                tbar: tbar,
                layout: 'fit'
            },{
                xtype: 'contextcardpanel',
                region: 'center',
                ref: 'detailcontainer',
                split: true,
                defaults: {
                    border: false
                },
                tbar: {
                    cls: 'largetoolbar componenttbar',
                    border: false,
                    height: 32,
                    items: [{
                        xtype: 'tbtext',
                        html: _t("Display: ")
                    },{
                        xtype: 'detailnavcombo',
                        menuIds: [],
                        onGetNavConfig: function(uid) {
                            var grid = this.componentgrid,
                                items = [],
                                monitor = false;
                            Ext.each(grid.store.data.items, function(record){
                                if (record.data.uid==uid && record.data.monitor) {
                                    monitor = true;
                                }
                            });
                            Zenoss.util.each(Zenoss.nav.get('Component'), function(item){
                                if (!(item.id=='Graphs' && !monitor)) {
                                    items.push(item);
                                }
                            });
                            return items;
                        }.createDelegate(this),
                        filterNav: function(cfg) {
                            var excluded = [
                                'status',
                                'events',
                                'resetcommunity',
                                'pushconfig',
                                'objtemplates',
                                'modeldevice',
                                'historyevents'
                            ];
                            return (excluded.indexOf(cfg.id)==-1);
                        },
                        ref: '../../componentnav',
                        getTarget: function() {
                            return this.detailcontainer;
                        }.createDelegate(this)
                    }]
                }
            }]
        });
        ZC.ComponentPanel.superclass.constructor.call(this, config);
        this.addEvents('contextchange');
    },
    getGridToolbar: function(){
        return Ext.getCmp(this.tbarid);
    },
    selectByToken: function(token) {
        var grid = this.componentgrid,
            store = grid.getStore(),
            sm = grid.getSelectionModel(),
            view = grid.getView();
        if (token) {
            store.findByUid(token, function(record, index) {
                if (!sm.isSelected(index)) {
                    if (index<=this.store.bufferRange[1]) {
                        sm.selectRow(index);
                    } else {
                        view.on('buffer', function(){
                            sm.selectRow(index);
                        }, this.store, {single:true});
                    }
                    view.ensureVisible(index);
                }
            }, grid);
        }
    },
    setContext: function(uid, type) {
        this.contextUid = uid;
        if (type!=this.componentType) {
            this.componentType = type;
            var compType = this.componentType + 'Panel',
                xtype = Ext.ComponentMgr.isRegistered(compType) ? compType : 'ComponentGridPanel';
            this.gridcontainer.removeAll();
            this.gridcontainer.add({
                xtype: xtype,
                componentType: this.componentType,
                ref: '../componentgrid',
                listeners: {
                    render: function(grid) {
                        grid.setContext(uid);
                    },
                    rangeselect: function(sm) {
                        this.detailcontainer.removeAll();
                        this.componentnav.reset();
                    },
                    selectionchange: function(sm, node) {
                        var row = sm.getSelected();
                        if (row) {
                            this.componentnav.setContext(row.data.uid);
                            var delimiter = Ext.History.DELIMITER,
                                token = Ext.History.getToken().split(delimiter, 2).join(delimiter);
                            Ext.History.add(token + delimiter + row.data.uid);
                            Ext.getCmp('component_monitor_menu_item').setDisabled(!row.data.usesMonitorAttribute);
                        } else {
                            this.detailcontainer.removeAll();
                            this.componentnav.reset();
                        }
                    },
                    scope: this
                }
            });
            this.gridcontainer.doLayout();
        } else {
            this.componentgrid.setContext(uid);
        }
        this.fireEvent('contextchange', this, uid, type);
    }
});
Ext.reg('componentpanel', ZC.ComponentPanel);


ZC.ComponentGridPanel = Ext.extend(Ext.ux.grid.livegrid.GridPanel, {
    lastHash: null,
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            border: false,
            autoExpandColumn: 'name',
            stripeRows: true,
            store: new ZC.BaseComponentStore({
                fields:config.fields,
                directFn:config.directFn || Zenoss.remote.DeviceRouter.getComponents
            }),
            colModel: new ZC.BaseComponentColModel({
                columns:config.columns
            }),
            selModel: new Zenoss.ExtraHooksSelectionModel({
                suppressDeselectOnSelect: true
            }),
            view: new Ext.ux.grid.livegrid.GridView({
                rowHeight: 22,
                nearLimit: 20,
                loadMask: {msg: _t('Loading. Please wait...'),
                           msgCls: 'x-mask-loading'},
                listeners: {
                    beforeload: this.onBeforeLoad,
                    beforebuffer: this.onBeforeBuffer,
                    scope: this
                }
            })
        });
        ZC.ComponentGridPanel.superclass.constructor.call(this, config);
        this.relayEvents(this.getSelectionModel(), ['selectionchange']);
        this.relayEvents(this.getSelectionModel(), ['rangeselect']);
        this.store.proxy.on('load',
            function(proxy, o, options) {
                this.lastHash = o.result.hash || this.lastHash;
            },
            this);
        Zenoss.util.addLoadingMaskToGrid(this);
    },
    onBeforeLoad: function(store, options) {
        this.applyOptions(options);
    },
    onBeforeBuffer: function(view, store, rowIndex, visibleRows,
                             totalCount, options) {
        this.applyOptions(options);
    },
    applyOptions: function(options){
        Ext.apply(options.params, {
            uid: this.contextUid,
            keys: Ext.pluck(this.getStore().fields.getRange(), 'name'),
            meta_type: this.componentType,
            name: this.componentName
        });
    },
    refresh: function() {
        this.setContext(this.contextUid);
    },
    filter: function(name) {
        if (this.componentName!=name) {
            this.componentName = name;
            this.view.updateLiveRows(0, true, true, false);
        }
    },
    setContext: function(uid) {
        this.contextUid = uid;
        this.getStore().on('load', function(){
            var token = Ext.History.getToken();
            if (token.split(Ext.History.DELIMITER).length!=3) {
                this.getSelectionModel().selectRow(0);
            }
        }, this, {single:true});
        this.view.updateLiveRows(0, true, true, false);
    }
});

Ext.reg('ComponentGridPanel', ZC.ComponentGridPanel);

ZC.BaseComponentStore = Ext.extend(Ext.ux.grid.livegrid.Store, {
    constructor: function(config) {
        var fields = config.fields || [
            {name: 'uid'},
            {name: 'severity'},
            {name: 'name'},
            {name: 'usesMonitorAttribute'},
            {name: 'monitor'},
            {name: 'monitored'},
            {name: 'status'}
        ];
        Ext.applyIf(config, {
            bufferSize: 50,
            proxy: new Ext.data.DirectProxy({
                directFn: config.directFn
            }),
            reader: new Ext.ux.grid.livegrid.JsonReader({
                root: 'data',
                totalProperty: 'totalCount',
                fields: fields
            })
        });
        ZC.BaseComponentStore.superclass.constructor.call(this, config);
        this.on('load', function(){this.loaded = true;}, this);
    },
    findByUid: function(uid, callback, scope) {
        var doit = function() {
            var i = 0, found = false;
            this.each(function(r){
                if (r.data.uid==uid) {
                    callback.call(scope||this, r, i);
                    found = true;
                    return false;
                }
                i++;
                return true;
            });
            if (!found) {
                var o = {componentUid:uid};
                Ext.apply(o, this.lastOptions.params);
                Zenoss.remote.DeviceRouter.findComponentIndex(o, function(r){
                    callback.call(scope||this, null, r.index);
                });
            }
        }.createDelegate(this);
        if (!this.loaded) {
            this.on('load', doit, this);
        } else {
            doit();
        }
    }
});

ZC.BaseComponentColModel = Ext.extend(Ext.grid.ColumnModel, {
    constructor: function(config) {
        var cols = config.columns || [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('Name')
            }, {
                id: 'monitored',
                dataIndex: 'monitored',
                header: _t('Monitored'),
                width: 70,
                sortable: true
            }, {
                id: 'status',
                dataIndex: 'status',
                header: _t('Status'),
                renderer: Zenoss.render.pingStatus,
                width: 60
            }];
        config = Ext.applyIf(config||{}, {
            defaults: {
                menuDisabled: true
            },
            columns: cols
        });
        ZC.BaseComponentColModel.superclass.constructor.call(this, config);
    }
});

ZC.IpInterfacePanel = Ext.extend(ZC.ComponentGridPanel, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'IpInterface',
            autoExpandColumn: 'description',
            fields: [
                {name: 'uid'},
                {name: 'severity'},
                {name: 'status'},
                {name: 'name'},
                {name: 'description'},
                {name: 'ipAddress'},//, mapping:'ipAddress.uid'},
                {name: 'network'},//, mapping:'network.uid'},
                {name: 'macaddress'},
                {name: 'status'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'},
                {name: 'locking'},
                {name: 'duplex'},
                {name: 'netmask'},
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 50
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('IP Interface'),
                width: 150
            },{
                id: 'ipAddress',
                dataIndex: 'ipAddress',
                header: _t('IP Address'),
                renderer: function(ipaddress) {
                    if (ipaddress && Ext.isObject(ipaddress) && ipaddress.netmask) {
                        ipaddress.name += '/' + ipaddress.netmask;
                    }
                    return Zenoss.render.link(ipaddress);
                }
            },{
                id: 'description',
                dataIndex: 'description',
                header: _t('Description')
            },{
                id: 'macaddress',
                dataIndex: 'macaddress',
                header: _t('MAC Address'),
                width: 120
            },{
                id: 'status',
                dataIndex: 'status',
                header: _t('Status'),
                renderer: Zenoss.render.pingStatus,
                width: 60
            },{
                id: 'monitored',
                dataIndex: 'monitored',
                header: _t('Monitored'),
                width: 60
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                width: 72,
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.IpInterfacePanel.superclass.constructor.call(this, config);
    }
});

Ext.reg('IpInterfacePanel', ZC.IpInterfacePanel);
ZC.registerName('IpInterface', _t('Interface'), _t('Interfaces'));

ZC.WinServicePanel = Ext.extend(ZC.ComponentGridPanel, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'WinService',
            fields: [
                {name: 'uid'},
                {name: 'severity'},
                {name: 'status'},
                {name: 'name'},
                {name: 'locking'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'},
                {name: 'caption'},
                {name: 'startMode'},
                {name: 'startName'},
                {name: 'serviceClassUid'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('Service Name'),
                renderer: Zenoss.render.WinServiceClass
            },{
                id: 'caption',
                dataIndex: 'caption',
                header: _t('Caption')
            },{
                id: 'startMode',
                dataIndex: 'startMode',
                header: _t('Start Mode')
            },{
                id: 'startName',
                dataIndex: 'startName',
                header: _t('Start Name')
            },{
                id: 'status',
                dataIndex: 'status',
                header: _t('Status'),
                renderer: Zenoss.render.pingStatus,
                width: 60
            },{
                id: 'monitored',
                dataIndex: 'monitored',
                header: _t('Monitored'),
                width: 60
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.WinServicePanel.superclass.constructor.call(this, config);
    }
});

Ext.reg('WinServicePanel', ZC.WinServicePanel);
ZC.registerName('WinService', _t('Windows Service'), _t('Windows Services'));


ZC.IpRouteEntryPanel = Ext.extend(ZC.ComponentGridPanel, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'IpRouteEntry',
            autoExpandColumn: 'destination',
            fields: [
                {name: 'uid'},
                {name: 'destination'},
                {name: 'nextHop'},
                {name: 'id'}, // needed for nextHop link
                {name: 'device'}, // needed for nextHop link
                {name: 'interface'},
                {name: 'protocol'},
                {name: 'type'},
                {name: 'locking'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitored'}
            ],
            columns: [{
                id: 'destination',
                dataIndex: 'destination',
                header: _t('Destination'),
                renderer: render_link
            },{
                id: 'nextHop',
                dataIndex: 'nextHop',
                header: _t('Next Hop'),
                renderer: Zenoss.render.nextHop,
                width: 250
            },{
                id: 'interface',
                dataIndex: 'interface',
                header: _t('Interface'),
                renderer: render_link
            },{
                id: 'protocol',
                dataIndex: 'protocol',
                header: _t('Protocol')
            },{
                id: 'type',
                dataIndex: 'type',
                header: _t('Type'),
                width: 50
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.IpRouteEntryPanel.superclass.constructor.call(this, config);
    }
});

Ext.reg('IpRouteEntryPanel', ZC.IpRouteEntryPanel);
ZC.registerName('IpRouteEntry', _t('Network Route'), _t('Network Routes'));

ZC.IpServicePanel = Ext.extend(ZC.ComponentGridPanel, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'IpService',
            fields: [
                {name: 'uid'},
                {name: 'name'},
                {name: 'status'},
                {name: 'severity'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'},
                {name: 'locking'},
                {name: 'protocol'},
                {name: 'description'},
                {name: 'ipaddresses'},
                {name: 'port'},
                {name: 'serviceClassUid'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('Name'),
                renderer: Zenoss.render.IpServiceClass
            },{
                id: 'protocol',
                dataIndex: 'protocol',
                header: _t('Protocol')
            },{
                id: 'port',
                dataIndex: 'port',
                header: _t('Port')
            },{
                id: 'ipaddresses',
                dataIndex: 'ipaddresses',
                header: _t('IPs'),
                renderer: function(ips) {
                    return ips.join(', ');
                }
            },{
                id: 'description',
                dataIndex: 'description',
                header: _t('Description')
            },{
                id: 'monitored',
                dataIndex: 'monitored',
                header: _t('Monitored'),
                width: 60
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.IpServicePanel.superclass.constructor.call(this, config);
    }
});

Ext.reg('IpServicePanel', ZC.IpServicePanel);
ZC.registerName('IpService', _t('IP Service'), _t('IP Services'));


ZC.OSProcessPanel = Ext.extend(ZC.ComponentGridPanel, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'OSProcess',
            autoExpandColumn: 'processName',
            fields: [
                {name: 'uid'},
                {name: 'processName'},
                {name: 'status'},
                {name: 'severity'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'},
                {name: 'locking'},
                {name: 'processClass'},
                {name: 'alertOnRestart'},
                {name: 'failSeverity'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'processName',
                dataIndex: 'processName',
                header: _t('Name')
            },{
                id: 'processClass',
                dataIndex: 'processClass',
                header: _t('Process Class'),
                renderer: function(cls) {
                    if (cls && cls.uid) {
                        return Zenoss.render.link(cls.uid);
                    } else {
                        return cls;
                    }
                }
            },{
                id: 'alertOnRestart',
                dataIndex: 'alertOnRestart',
                header: _t('Alert On Restart')
            },{
                id: 'failSeverity',
                dataIndex: 'failSeverity',
                header: _t('Fail Severity')
            },{
                id: 'monitored',
                dataIndex: 'monitored',
                header: _t('Monitored'),
                width: 60
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.OSProcessPanel.superclass.constructor.call(this, config);
    }
});

Ext.reg('OSProcessPanel', ZC.OSProcessPanel);
ZC.registerName('OSProcess', _t('OS Process'), _t('OS Processes'));


ZC.FileSystemPanel = Ext.extend(ZC.ComponentGridPanel, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            autoExpandColumn: 'mount',
            componentType: 'FileSystem',
            fields: [
                {name: 'uid'},
                {name: 'name'},
                {name: 'status'},
                {name: 'severity'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'},
                {name: 'locking'},
                {name: 'mount'},
                {name: 'totalBytes'},
                {name: 'availableBytes'},
                {name: 'usedBytes'},
                {name: 'capacityBytes'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'mount',
                dataIndex: 'mount',
                header: _t('Mount Point')
            },{
                id: 'totalBytes',
                dataIndex: 'totalBytes',
                header: _t('Total Bytes'),
                renderer: Zenoss.render.bytesString
            },{
                id: 'usedBytes',
                dataIndex: 'usedBytes',
                header: _t('Used Bytes'),
                renderer: Zenoss.render.bytesString
            },{
                id: 'availableBytes',
                dataIndex: 'availableBytes',
                header: _t('Free Bytes'),
                renderer: function(n){
                    if (n<0) {
                        return _t('Unknown');
                    } else {
                        return Zenoss.render.bytesString(n);
                    }

                }
            },{
                id: 'capacityBytes',
                dataIndex: 'capacityBytes',
                header: _t('% Util'),
                renderer: function(n) {
                    if (n=='unknown' || n<0) {
                        return _t('Unknown');
                    } else {
                        return n + '%';
                    }
                }

            },{
                id: 'monitored',
                dataIndex: 'monitored',
                header: _t('Monitored'),
                width: 60
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.FileSystemPanel.superclass.constructor.call(this, config);
    }
});

Ext.reg('FileSystemPanel', ZC.FileSystemPanel);
ZC.registerName('FileSystem', _t('File System'), _t('File Systems'));


ZC.CPUPanel = Ext.extend(ZC.ComponentGridPanel, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'CPU',
            autoExpandColumn: 'product',
            fields: [
                {name: 'uid'},
                {name: 'socket'},
                {name: 'manufacturer'},
                {name: 'product'},
                {name: 'clockspeed'},
                {name: 'extspeed'},
                {name: 'cacheSizeL1'},
                {name: 'cacheSizeL2'},
                {name: 'voltage'},
                {name: 'locking'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitored'}
            ],
            columns: [{
                id: 'socket',
                dataIndex: 'socket',
                header: _t('Socket'),
                width: 45
            },{
                id: 'manufacturer',
                dataIndex: 'manufacturer',
                header: _t('Manufacturer'),
                renderer: render_link
            },{
                id: 'product',
                dataIndex: 'product',
                header: _t('Model'),
                renderer: render_link
            },{
                id: 'clockspeed',
                dataIndex: 'clockspeed',
                header: _t('Speed'),
                width: 70,
                renderer: function(x){ return x + ' MHz';}
            },{
                id: 'extspeed',
                dataIndex: 'extspeed',
                header: _t('Ext Speed'),
                width: 70,
                renderer: function(x){ return x + ' MHz';}
            },{
                id: 'cacheSizeL1',
                dataIndex: 'cacheSizeL1',
                header: _t('L1'),
                width: 70,
                renderer: function(x){ return x + ' KB';}
            },{
                id: 'cacheSizeL2',
                dataIndex: 'cacheSizeL2',
                header: _t('L2'),
                width: 70,
                renderer: function(x){ return x + ' KB';}
            },{
                id: 'voltage',
                dataIndex: 'voltage',
                header: _t('Voltage'),
                width: 70,
                renderer: function(x){ return x + ' mV';}
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.CPUPanel.superclass.constructor.call(this, config);
    }
});

Ext.reg('CPUPanel', ZC.CPUPanel);
ZC.registerName('CPU', _t('Processor'), _t('Processors'));


ZC.PowerSupplyPanel = Ext.extend(ZC.ComponentGridPanel, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'PowerSupply',
            autoExpandColumn: 'name',
            fields: [
                {name: 'uid'},
                {name: 'severity'},
                {name: 'name'},
                {name: 'watts'},
                {name: 'type'},
                {name: 'state'},
                {name: 'millivolts'},
                {name: 'locking'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('Name')
            },{
                id: 'watts',
                dataIndex: 'watts',
                header: _t('Watts')
            },{
                id: 'type',
                dataIndex: 'type',
                header: _t('Type')
            },{
                id: 'state',
                dataIndex: 'state',
                header: _t('State')
            },{
                id: 'millivolts',
                dataIndex: 'millivolts',
                header: _t('Millivolts')
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.PowerSupplyPanel.superclass.constructor.call(this, config);
    }
});

Ext.reg('PowerSupplyPanel', ZC.PowerSupplyPanel);
ZC.registerName('PowerSupply', _t('Power Supply'), _t('Power Supplies'));


ZC.TemperatureSensorPanel = Ext.extend(ZC.ComponentGridPanel, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'TemperatureSensor',
            autoExpandColumn: 'name',
            fields: [
                {name: 'uid'},
                {name: 'severity'},
                {name: 'name'},
                {name: 'state'},
                {name: 'temperature'},
                {name: 'locking'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('Name')
            },{
                id: 'state',
                dataIndex: 'state',
                header: _t('State')
            },{
                id: 'temperature',
                dataIndex: 'temperature',
                header: _t('Temperature'),
                renderer: function(x) {
                    if (x == null) {
                        return "";
                    } else {
                        return x + " F";
                    }
                }
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.TemperatureSensorPanel.superclass.constructor.call(this, config);
    }
});

Ext.reg('TemperatureSensorPanel', ZC.TemperatureSensorPanel);
ZC.registerName('TemperatureSensor', _t('Temperature Sensor'), _t('Temperature Sensors'));


ZC.FanPanel = Ext.extend(ZC.ComponentGridPanel, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            componentType: 'Fan',
            autoExpandColumn: 'name',
            fields: [
                {name: 'uid'},
                {name: 'severity'},
                {name: 'name'},
                {name: 'state'},
                {name: 'type'},
                {name: 'rpm'},
                {name: 'locking'},
                {name: 'usesMonitorAttribute'},
                {name: 'monitor'},
                {name: 'monitored'}
            ],
            columns: [{
                id: 'severity',
                dataIndex: 'severity',
                header: _t('Events'),
                renderer: Zenoss.render.severity,
                width: 60
            },{
                id: 'name',
                dataIndex: 'name',
                header: _t('Name')
            },{
                id: 'state',
                dataIndex: 'state',
                header: _t('State')
            },{
                id: 'type',
                dataIndex: 'type',
                header: _t('Type')
            },{
                id: 'rpm',
                dataIndex: 'rpm',
                header: _t('RPM')
            },{
                id: 'locking',
                dataIndex: 'locking',
                header: _t('Locking'),
                renderer: Zenoss.render.locking_icons
            }]
        });
        ZC.FanPanel.superclass.constructor.call(this, config);
    }
});

Ext.reg('FanPanel', ZC.FanPanel);
ZC.registerName('Fan', _t('Fan'), _t('Fans'));

})();
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

/* package level */
(function() {
     var Severity = Ext.extend(Ext.form.ComboBox, {
         constructor: function(config) {
             config = config || {};
             
             Ext.applyIf(config, {
                 fieldLabel: _t('Severity'),
                 name: 'severity',
                 editable: false,
                 forceSelection: true,
                 autoSelect: true,
                 triggerAction: 'all',
                 mode: 'local',
                 // this is defined in zenoss.js so should always be present
                 store: Zenoss.env.SEVERITIES
             });
             Severity.superclass.constructor.apply(this, arguments);
         }
     });
     Ext.reg('severity', Severity);
}());

/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

/* package level */
(function() {
     var EventClass = Ext.extend(Ext.form.ComboBox, {
         constructor: function(config) {
             config = config || {};
             if (!Zenoss.env.EVENT_CLASSES) {
                 throw "You must include the js-snippets viewlet before you use the eventClass control";
             }
             Ext.applyIf(config, {
                 fieldLabel: _t('Event Class'),
                 name: 'eventClass',
                 typeAhead: true,
                 editable: false,
                 forceSelection: true,
                 autoSelect: true,
                 resizable: true,
                 triggerAction: 'all',
                 mode: 'local',
                 store: Zenoss.env.EVENT_CLASSES
             });
             EventClass.superclass.constructor.apply(this, arguments);
         }
     });
     Ext.reg('eventclass', EventClass);
}());/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

/* package level */
(function() {
var DataPointItemSelector = Ext.extend(Ext.ux.form.ItemSelector, {
    constructor: function(config) {
        var record = config.record;
        Ext.applyIf(config, {
            name: 'dataPoints',
            fieldLabel: _t('Data Points'),
            id: 'thresholdItemSelector',
            imagePath: "/++resource++zenui/img/xtheme-zenoss/icon",
            drawUpIcon: false,
            drawDownIcon: false,
            drawTopIcon: false,
            drawBotIcon: false,
            multiselects: [{
                cls: 'multiselect-dialog',
                width: 250,
                height: 200,
                store: record.allDataPoints                            
            },{
                cls: 'multiselect-dialog',
                width: 250,
                height: 200,
                // datapoints comes back as a string from the server
                store: record.dataPoints.split(",") || []                            
            }]
        });
        DataPointItemSelector.superclass.constructor.apply(this, arguments);
    }
});
Ext.reg('datapointitemselector', DataPointItemSelector);
}());
/*
###########################################################################
#
# This program is part of Zenoss Core, an open source monitoring platform.
# Copyright (C) 2010, Zenoss Inc.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# For complete information please visit: http://www.zenoss.com/oss/
#
###########################################################################
*/
(function () {
Ext.ns('Zenoss');

Zenoss.ContextMenu = Ext.extend( Ext.Button, {
    /**
     * onSetContext abstract function called when the context for the menu is
     * potentially changed
     */
    onSetContext: Ext.emptyFn,
    /**
     * The object id for which the context menu belongs to
     */
    contextUid: null,
    constructor: function(config) {
        config = config || {};
        Ext.applyIf(config, {
            id: 'context-configure-menu',
            iconCls: 'customize'
        });
        Zenoss.ContextMenu.superclass.constructor.call(this, config);
    },
    setContext: function(uid) {
        this.contextUid = uid;
    }
});

Ext.reg('ContextMenu', Zenoss.ContextMenu);


Zenoss.ContextConfigureMenu = Ext.extend( Zenoss.ContextMenu,{
    /**
     * onGetMenuItems abstract function; hook to provide
     * more menu items based on context
     * @param {string} uid; item to get menu items for
     */
    onGetMenuItems: function(uid){ return []; },
    /**
     * Called when an item in the menu is selected
     * @param {object} this, the ContextConfigureMenu
     * @param {object} the menu item selection
     */
    onSelectionChange: Ext.emptyFn,
    /**
     * Filter out menu items;  Return true if it should be kept, false otherwise
     * @param {ContextConfigureMenu}
     * @param {Object} itemConfig
     */
    filterMenu: function(contextMenu, config) {return true;},
     /**
     * Menu ids used to get dialog menus that will be used in context menu;
     * Empty or null if no items from old menu items is desired
     */
    menuIds: ['More','Manage','Edit', 'Actions','Add','TopLevel'],
    menu: {items:[]},
    disabled: true,
    /**
     * @cfg {array} menutItems
     * items that should always be included in the menu
     */
    menuItems: [],
    /**
     * set the UID, menus for the corresponding object will be determined and
     * set
     * @param {Object} uid
     */
    setContext: function(uid) {
        this.disable();
        this.onSetContext(uid);
        this.contextUid = uid;
        var menu = this.menu;
        menu.removeAll();
        this.getMenuItems(uid);
    },
    /**
     * private
     * handler for add to zenpack menu item; displays dialog
     * @param {Object} uid
     */
     addToZenPackHandler: function(e) {
       if (!this.addtozenpack) {
                this.addtozenpack = new Zenoss.AddToZenPackWindow();
            }
            this.addtozenpack.setTarget(this.contextUid);
            this.addtozenpack.show();
    },
    /**
     * private
     * gets the menu items to be displayed
     * @param {Object} uid
     */
    getMenuItems: function(uid){
        var callback = function(provider, response){
            var menuItems = [], visibleMenuCount = 0;
            //get statically defined menu items
            if (this.menuItems.length !== 0) {
                menuItems = menuItems.concat(this.menuItems);
            }
            //get any context specific menus if defined
            var moreMenuItems = this.onGetMenuItems(this.contextUid);
            if (moreMenuItems !== null || moreMenuItems.length > 0){
                menuItems = menuItems.concat(moreMenuItems);
            }
            //menus from router
            var itemConfigs = response.result.menuItems;
            var filterFn = function(val) {
                return this.filterMenu(this, val);
            };
            itemConfigs = Zenoss.util.filter(itemConfigs, filterFn, this);
            if (itemConfigs){
                menuItems = menuItems.concat(itemConfigs);
            }
            //add all menus and set handlers if needed
            Ext.each(menuItems, function(item){
                //add to zenpack has a different/new handler
                if (item.id === 'addtozenpack'){
                    item.handler = this.addToZenPackHandler.createDelegate(this);
                }
                if (!Ext.isDefined(item.handler)) {
                    item.handler = this.defaultHandler.createDelegate(this);
                }
                         
                // do now show as enabled if we only have hidden items (or spacers)
                if (!item.hidden && item != '-') {
                    visibleMenuCount += 1;
                }
                this.menu.add(item);
            }, this);
            
            // if we have stuff then enable this control
            if(this.menu.items.length !== 0 && visibleMenuCount){
                this.enable();
            }
        };
        var args = {
            'uid': uid
        };
        if (this.menuIds !== null && this.menuIds.length >= 1){
            args.menuIds = this.menuIds;
        }

        Zenoss.remote.DetailNavRouter.getContextMenus(args, callback, this);    
    },
                                              
    /**
     * private
     * handler used if a menu item does not have a handler defined
     * @param {Object} button
     */
    defaultHandler: function(button) {
        var dialog = new Zenoss.dialog.DynamicDialog();
        dialog.setTitle(_t(button.text.replace(/\.\.\./g, '')));
        dialog.show();
        dialog.body.load({
            scripts: true,
            url: this.contextUid + '/' + button.viewName
        });
    }
});

Ext.reg('ContextConfigureMenu', Zenoss.ContextConfigureMenu);

})();
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

/* package level */
(function() {
     var parser = Ext.extend(Ext.form.ComboBox, {
         constructor: function(config) {
             var record = config.record;
             config = config || {};
             Ext.applyIf(config, {
                 fieldLabel: _t('Parser'),
                 name: 'parser',
                 editable: false,
                 forceSelection: true,
                 autoSelect: true,
                 triggerAction: 'all',
                 minListWidth: 250,
                 resizable: true,
                 mode: 'local',
                 store: record.availableParsers
             });
             parser.superclass.constructor.apply(this, arguments);
         }
     });
     Ext.reg('parser', parser);
}());/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

/* package level */
(function() {
     var rrdtype = Ext.extend(Ext.form.ComboBox, {
         constructor: function(config) {
             var record = config.record;
             config = config || {};
             Ext.apply(config, {
                 fieldLabel: _t('RRD Type'),
                 name: 'rrdtype',
                 editable: false,
                 forceSelection: true,
                 autoSelect: true,
                 triggerAction: 'all',
                 mode: 'local',
                 store: record.availableRRDTypes
             });
             rrdtype.superclass.constructor.apply(this, arguments);
         }
     });
     Ext.reg('rrdtype', rrdtype);
}());/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

/* package level */
(function() {

Ext.ns('Zenoss.form');

Zenoss.form.Password = Ext.extend(Ext.form.TextField, {
     constructor: function(config) {
         config.inputType = 'password';
         Zenoss.form.Password.superclass.constructor.apply(this, arguments);
     }
 });
 Ext.reg('password', Zenoss.form.Password);

})();
/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */

/* package level */
(function() {

Ext.ns('Zenoss.form');

Zenoss.form.UpDownField = Ext.extend(Ext.form.ComboBox, {
     constructor: function(config) {
         config = Ext.applyIf(config||{}, {
             editable: false,
             forceSelection: true,
             autoSelect: true,
             triggerAction: 'all',
             mode: 'local',
             store: [
                 [1, 'Up'],
                 [0, 'Down']
             ]
         });
         Zenoss.form.UpDownField.superclass.constructor.call(this, config);
     }
 });
 Ext.reg('updownfield', Zenoss.form.UpDownField);

})();
(function() {
     Ext.namespace('Zenoss.form.Alias');

     /**
      * When you press the "Add" button this will add a new 
      * alias row to the panel. 
      **/
     function addAliasRow() {
         var cmp = that,
             button = Ext.getCmp('add_alias_button'),
             row = {};
         
         cmp.remove(button);
         cmp.add(getRowTemplate());
         cmp.add(addAliasButton);
         cmp.doLayout();
     }
     
     /**
      * Variable Definitions
      **/
     var addAliasButton = {
         id: 'add_alias_button',
         xtype: 'button',
         text: 'Add',
         handler: addAliasRow
     },
     that;

     /**
      * Returns the panel definition for the id/formula row
      **/
     function getRowTemplate(id, formula) {
         return {
             xtype: 'container',
             bodyCls: 'alias',
             layout: 'hbox',
             items: [{
                 aliasType: 'id',
                 xtype: 'textfield',
                 value: id
             },{
                 aliasType: 'formula',
                 xtype: 'textfield',
                 value: formula
             },{
                 xtype: 'button',
                 text: 'Delete',
                 width: '10',
                 handler: function() {
                     var container = this.findParentByType('container'),
                         cmp = that;
                     cmp.remove(container);
                     cmp.doLayout();
                 }
             }]
        };
     }
     
     /**
      * Alias Panel. NOTE: This control is designed to have
      * only one instance per page. 
      **/
     Zenoss.form.Alias = Ext.extend(Ext.Panel, {
         constructor: function(config) {
             var aliases = config.record.aliases,
             items = [],
             i,
             alias;
             // use "that" as a closure so we have a reference to it
             that = this;
             items.push({
                xtype: 'panel',
                layout: 'form',
                border: false,
                html: _t('ID / FORMULA')
             });
             config = config || {};
             // add a row for each alias defined
             for (i=0; i < aliases.length; i++) {
                 alias = aliases[i];
                 items.push(getRowTemplate(alias.name, alias.formula));
             }   
             // always show an extra blank row
             items.push(getRowTemplate());
             
             // add the button
             items.push(addAliasButton);
             Ext.applyIf(config, {
                 items:items
             });
             
             Zenoss.form.Alias.superclass.constructor.apply(this, arguments);
         },
                                        
         /**
          * This returns a list of all of the  aliases in object form.
          * Since this is a Panel it must explicitly be called.
          * Ex:
          *    Ext.getCmp('aliasId').getValue();
          **/
         getValue: function(){
             var cmp = that,
                 textfields = cmp.findByType('textfield'),
                 results = [],
                 i, field;
             
             // initialize the return structure. We want an array of object literals
             // (will be dicts in python)
             for (i = 0; i < textfields.length / 2; i++ ) {
                 results[i] = {};
             }

             // turn each entry into an object literal with the properties
             // id and formula
             for (i = 0; i < textfields.length; i++ ) {
                 field = textfields[i];
                 
                 // aliasType was defined on the dynamically created rows above
                 results[Math.floor(i/2)][field.aliasType] = field.getValue();       
             }
             
             return results;
         }
     });
     Ext.reg('alias', Zenoss.form.Alias);
}());/*
 ###########################################################################
 #
 # This program is part of Zenoss Core, an open source monitoring platform.
 # Copyright (C) 2010, Zenoss Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 2 as published by
 # the Free Software Foundation.
 #
 # For complete information please visit: http://www.zenoss.com/oss/
 #
 ###########################################################################
 */


(function() {

Ext.ns('Zenoss.form');
/**
 * This is a special case of a text area. It is designed to take up the entire column
 * on a two column layout.
 **/
Zenoss.form.TwoColumnTextArea = Ext.extend(Ext.form.TextArea, {
     constructor: function(config) {
         config.width = 500;
         config.height = 220;
         Zenoss.form.TwoColumnTextArea.superclass.constructor.apply(this, arguments);
     }
 });
 Ext.reg('twocolumntextarea', Zenoss.form.TwoColumnTextArea);
})();
/*
  ###########################################################################
  #
  # This program is part of Zenoss Core, an open source monitoring platform.
  # Copyright (C) 2010, Zenoss Inc.
  #
  # This program is free software; you can redistribute it and/or modify it
  # under the terms of the GNU General Public License version 2 as published by
  # the Free Software Foundation.
  #
  # For complete information please visit: http://www.zenoss.com/oss/
  #
  ###########################################################################
*/

/* package level */
(function() {

    Ext.ns('Zenoss.form');
    /**
     * This is a display field that takes an array as its value and
     * renders them with one element per line
     **/
    Zenoss.form.MultiLineDisplayField = Ext.extend(Ext.form.DisplayField, {
        constructor: function(config) {
            // if a string was passed in then defer to the parents behavior
            if ((typeof config.value) != 'string') {
                config.value = config.value.join('<br />');     
            }
            
            Zenoss.form.MultiLineDisplayField.superclass.constructor.apply(this, arguments);
        }
    });
    Ext.reg('multilinedisplayfield', Zenoss.form.MultiLineDisplayField);
}());
(function(){ // Local scope

var ZI = Ext.namespace('Zenoss.inspector');

/**
 * Manages inspector windows to ensure that only one window matching `uid` is active at a time.
 */
ZI.SingleInstanceManager = Ext.extend(Object, {
    _instances: null,
    constructor: function() {
        this._instances = {};
    },
    register: function(uid, instance) {
        this.remove(uid);
        this._instances[uid] = instance;
        instance.on('destroy', function() { delete this._instances[uid]; }, this);
    },
    get: function(uid) {
        return this._instances[uid];
    },
    remove: function(uid) {
        Ext.destroyMembers(this._instances, uid);
    }
});

/**
 * Represents a single item in the inspector panel.
 *
 * config:
 *   - valueTpl An XTemplate that can be used to render the field. Will be passed the data that is passed to the
 *              inspector.
 */
ZI.InspectorProperty = Ext.extend(Ext.Container, {
    constructor: function(config) {
        config = Ext.applyIf(config||{}, {
            cls: 'inspector-property',
            layout: 'form',
            items: [
                {
                    cls: 'inspector-property-label',
                    ref: 'labelItem',
                    xtype: 'label',
                    text: config.label ? config.label + ':' : ''
                },
                {
                    cls: 'inspector-property-value',
                    ref: 'valueItem',
                    text: config.value || '',
                    xtype: 'box',
                    tpl: config.valueTpl || '{.}'
                }
            ]
        });
        ZI.InspectorProperty.superclass.constructor.call(this, config);
    },
    setValue: function(t) {
        this.valueItem.update(t);
    },
    setLabel: function(t) {
        this.labelItem.setText(t + ':');
    }
});

Ext.reg('inspectorprop', ZI.InspectorProperty);


ZI.BaseInspector = Ext.extend(Ext.Panel, {
    _data: null,
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            defaultType: 'devdetailitem',
            layout: 'form',
            border: false,
            bodyBorder: false,
            items: [],
            titleTpl: '<div class="name">{name}</div>'
        });

        config.items = [
            {
                layout: 'hbox',
                cls: 'inspector-header',
                defaults: {border: false},
                ref: 'headerItem',
                items: [
                    {
                        cls: 'header-icon',
                        ref: 'iconItem',
                        xtype: 'box'
                    },
                    {
                        cls: 'header-title',
                        ref: 'titleItem',
                        xtype: 'label',
                        tpl: config.titleTpl
                    }
                ]
            },
            {
                xtype: 'container',
                cls: 'inspector-body',
                autoEl: 'div',
                layout: 'form',
                ref: 'bodyItem',
                defaultType: 'inspectorprop',
                items: config.items
            }
        ];

        ZI.BaseInspector.superclass.constructor.call(this, config);
    },
    setIcon: function(url) {
        this.headerItem.iconItem.getEl().setStyle({
            'background-image' : 'url(' + url + ')'
        });
    },
    onLayout: function(shallow, forceLayout) {
        ZI.BaseInspector.superclass.onLayout.call(this, shallow, forceLayout);

        if ( this._data ) {
            // Load any pending data
            var data = this._data;
            delete this._data;
            this.update(data);
        }
    },
    /**
     * Overwrite to add any properties dynamically from the data. Must return true if added any.
     */
    addNewDataItems: function(data) {
        return false;
    },
    update: function(data) {
        if ( this.rendered ) {
            if ( this.addNewDataItems(data) ) {
                this.doLayout();
            }

            // update all the children that have templates
            var self = this;
            this.cascade(function(item) {
                if ( item != self && item.tpl ) {
                    item.update(data);
                }

                return true;
            });

            if ( data.icon ) {
                this.setIcon(data.icon);
            }

            if ( this.ownerCt ) {
                this.ownerCt.doLayout(false, true);
            }
            else {
                this.doLayout(false, true);
            }
        }
        else {
            // Can't load the data yet, the components aren't ready
            // Set this up so we can set the data after layout
            this._data = data;
        }
    },
    /**
     * Add a property to the inspector panel for display.
     * @param label string
     * @param id string The key from the data to display
     */
    addProperty: function(label, id) {
        this.addPropertyTpl(label, '{[values.' + id + ' || \'\']}');
    },
    /**
     * Add a property to the inspector panel using a template to display.
     * @param label
     * @param tpl string A string in XTemplate format to display this property. Data values are in `values`.
     */
    addPropertyTpl: function(label, tpl) {
        this.bodyItem.add({
            xtype: 'inspectorprop',
            label: label,
            valueTpl: tpl
        });
    }
});

/**
 * An inspector that gets it's data via a directFn remote call.
 */
ZI.DirectInspector = Ext.extend(ZI.BaseInspector, {
    _contextUid: null,
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            directFn: Ext.emptyFn
        });

        ZI.DirectInspector.superclass.constructor.call(this, config);
    },
    initComponent: function() {
        ZI.DirectInspector.superclass.initComponent.call(this);
        this.addEvents('contextchange');
        this.on('contextchange', this._onContextChange, this);
    },
    refresh: function() {
        this.load();
    },
    load: function() {
        if ( this._contextUid ) {
            this.directFn(
                { uid: this._contextUid, keys: this.keys },
                function(result){
                    if ( result.success ) {
                        this.fireEvent('contextchange', result.data, this);
                    }
                },
                this
            );
        }
    },
    setContext: function(uid, load) {
        this._contextUid = uid;
        load = Ext.isDefined(load) ? load : true;

        if ( load ) {
            this.load();
        }
    },
    _onContextChange: function(data) {
        this.onData(data);
    },
    onData: function(data) {
        this.update(data);
    }
});

ZI.DeviceInspector = Ext.extend(ZI.DirectInspector, {
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            directFn: Zenoss.remote.DeviceRouter.getInfo,
            keys: ['ipAddress', 'device', 'deviceClass'],
            cls: 'inspector',
            titleTpl: '<div class="name"><a href="{uid}" target="_top">{name}</a></div><div class="info">{[Zenoss.render.DeviceClass(values.deviceClass.uid)]}</div><div class="info">{[Zenoss.render.ipAddress(values.ipAddress)]}</div>'
        });

        ZI.DeviceInspector.superclass.constructor.call(this, config);

        this.addPropertyTpl(_t('Events'), '{[Zenoss.render.events(values.events, 4)]}');
        this.addPropertyTpl(_t('Device Status'), '{[Zenoss.render.pingStatus(values.status)]}');
        this.addProperty(_t('Production State'), 'productionState');
        this.addPropertyTpl(_t('Location'), '{[(values.location && values.location.name) || ""]}');
    }
});

Ext.reg('deviceinspector', ZI.DeviceInspector);

ZI.ComponentInspector = Ext.extend(ZI.DirectInspector, {
    constructor: function(config) {
        config = Ext.applyIf(config || {}, {
            directFn: Zenoss.remote.DeviceRouter.getInfo,
            keys: ['ipAddress', 'device'],
            cls: 'inspector',
            titleTpl: '<div class="name"><a href="{uid}" target="_top">{name}</a></div><div class="info"><a href="{[values.device.uid]}" target="_top">{[values.device.device]}</a></div><div class="info">{[Zenoss.render.ipAddress(values.ipAddress)]}</div>'
        });

        config.items = [
            {
                label: _t('Events'),
                valueTpl: '{[Zenoss.render.events(values.events, 4)]}'
            }
        ];

        ZI.ComponentInspector.superclass.constructor.call(this, config);
    }
});

Ext.reg('componentinspector', ZI.ComponentInspector);

var windowManager = new ZI.SingleInstanceManager();
ZI.createWindow = function(uid, xtype, x, y) {
    var win = new Ext.Window({
        x: (x || 0),
        y: (y || 0),
        cls: 'inspector-window',
        frame: true,
        constrain: true,
        resizable: false,
        layout: 'form',
        width: 300,
        plain: true,
        border: false,
        items: [{
            xtype: xtype,
            ref: 'panelItem'
        }]
    });

    windowManager.register(uid, win);
    return win;
}

ZI.registeredInspectors = {
    device : 'deviceinspector'
};

ZI.registerInspector = function(inspector_type, inspector_xtype) {
    ZI.registeredInspectors[inspector_type.toLowerCase()] = inspector_xtype
}

ZI.show = function(uid, x, y) {
    Zenoss.remote.DeviceRouter.getInfo({ uid: uid }, function(result) {
        if ( result.success ) {
            // Grasping at straws but assume it's a component unless otherwise stated
            var xtype = 'componentinspector';

            var itype = result.data.inspector_type || result.data.meta_type;
            if ( itype ) {
                itype = itype.toLowerCase();
                if ( ZI.registeredInspectors[itype] ) {
                    xtype = ZI.registeredInspectors[itype];
                }
            }

            var win = ZI.createWindow(uid, xtype, x, y);
            win.panelItem.setContext(uid, false);
            win.panelItem.update(result.data);
            win.show();
        }
    });
};

})();
